/*
 * Copyright (C) 1983 Intel Corp.
 */

/*
 * "@(#) c188 $State: XENIX_R3-5 $ $Date: 87/02/27 14:32:57 $";
 * c188.c
 *	iSBC 188 Specific Configuration file.
 *
 *	I001	brad	12/10/86
 *		Edited in additional baud rates
 *		Edited in I188LINES in place of 12 in tty struct line
 *	I002	2/11/87
 *		Edited in more boards in the 188cfg structure
 *	I003	2/19/87
 *		Added in special configuration structure
 *	I004	seckin	2/27/87
 *		Changed format of the i188cfg structure declaration,
 *		so that SCP can redefine addresses, ports, interrupts
 */
#include "../h/param.h"
#include "../h/dir.h"	/* system directory structures */
#include "../h/a.out.h"	/* needed for user.h */
#include "../h/tty.h"	/* device structures (system)	*/
#include "../h/i188.h"	/* 188/48 drive structures and constants */

/*
 * To configure the maximum number of 188 boards in the system,
 * change NUM188. There is a maximum of 4 and a minimum of 1
 */
#define	NUM188	2
int	N188 = NUM188;

/*
 * i188cfg:	The physical starting addresses of the
 *		serial boards' memory mapped I/O space.
 *
 *		I002
 * The 'third serial board' as defined below is listed in the Intel
 * price book as board 4.  The 'fourth serial board' as defined below
 * is listed in the Intel Price book as board 3.  This is because 'by
 * definition' the host communication products will reside on board 3.
 * Note that a 188/XX board cannot reside at base address 0xF80000.
 *
 * 188 serial board at 0xFB0000 is reserved for host-comm
 *
 *		I004
 * The initialization list will be reconstructed by SCP depending on
 * which serial boards exist at which addresses, ports, interrupts.
 * Note that if iVPM is installed, it takes over all the 188 boards.
 *
 */

struct	i188cfg	i188cfg[] = {
	0xF90000, 0x08A6, 03,	/* board 1 */
	0xFA0000, 0x08A7, 02,	/* board 2 */
	0xF80000, 0x08A4, 0101,	/* board 3 */
	0xFB0000, 0x08A5, 0100,	/* board 4 */
	};

/*
 * baud rate translation table
 */
int	i188baud[] = {
		US_B0,    US_B50,   US_B75,   US_B110,  US_B134,  US_B150,
		US_B200,  US_B300,  US_B600 , US_B1200, US_B1800, US_B2400,
		US_B4800, US_B9600, US_B19200,US_B38400,US_B51800,US_B76800
	};

/*
 * special configuration structure
 *	There is one unsigned value per line
 *	The bit masks are defined in i188.h
 *	If one value is to be initialized to a value other than 0, then
 *	the rest of the array should be initialized to 0
 */

unsigned	i188special[NUM188*I188LINES];
int		i188timeout = I188_WAIT_DEFAULT;	/* close timeout */

/*
 * Xenix standard tty structure.
 * Device driver's private data. Space for 12 structures
 * is allocated for each board configured in the driver.
 */
struct	tty	  i188tty[NUM188*I188LINES];

/*
 * Line and board state data for the iSBC 188/48.
 * Space allocated for each board configured in the driver.
 */
struct	i188board i188board[NUM188];

/*
 * Intermediate output buffers for block moves with poke.
 * One 256 byte buffer allocated for each board configured.
 */
char	i188o_buf[NUM188*TTYHOG];

