/* c208.c
 *
 *	iSBC 208 Specific Configuration file.
 *
 *  "@(#) c208 $State: XENIX_R3-5 $ $Date: 86/05/30 13:22:34 $";
 * This split  out from c.c to avoid name-clashing with other device-
 * specific configuration files.  
 *
 *	Contains conditional code for the following configurations:
 *	#if FULL	8-inch full-sized drives
 *	#if MINI	5.25-inch mini drives
 *	#if TPI_48	mini drives with 48 tracks per inch, 40 cylinders total
 *	#if TPI_96	mini drives with 96 tracks per inch, 80 cylinders total
 */

#define		FULL		0
#define		MINI		1
#define		TPI_48		1
#define		TPI_96		0

#include  "../h/param.h"
#include   "../h/i208.h"
#include   "../h/buf.h"

/* Configurable parameters */
int RETRY = 10;			/* # of retries on soft error */
int i208TIME = 100;		/* # of clock ticks till mini times out */



#if MINI
/*******************************************************************************
 *
 * 5.25 Floppy drives
 *
 ******************************************************************************/


#if TPI_48
/*******************************************************************************
 *
 * 48 tracks per inch, 40 cylinders total
 *
 ******************************************************************************/


/* Partition tables */

/*
 * DS/DD 5.25" Floppy, 1024-byte sector Partitions.  Track 0 unused.
 */
struct	i208part Sdf0[] = {
	4,		316		/* rest: 316K */
};

/*
 * DS/DD 5.25" Floppy, 1024-byte sector Partitions.  Tracks 0 and 1 unused.
 */
struct	i208part Sboot[] = {
	8,		184,		/* boot: 184k */
	192,		128		/* ram:  128k */
};

/*
 * SS/SD 5.25" Floppy, 128-byte sector Partitions.
 * Needed only for writing the boot track, to create boot floppies.
 */
struct	i208part Ssyf0[] = {
	0,		1280		/* all of disk: 158K */
};

/*
 * DS/DD 5.25" Floppy, 512-byte sectors, 8 sectors per track.
 */
struct	i208part Sdzf0[] = {
	  0,		640		/* all of disk: 320K */
};

/*
 * DS/DD 5.25" Floppy, 512-byte sectors, 9 sectors per track.
 */
struct	i208part Sdvf0[] = {
	  0,		720		/* all of disk: 360K */
};


/* Constant drive table characteristics */

/*
 * 208 Board 0, Unit 0 Device-table Definitions (drtab's)
 */
struct i208cdrt i208d00[] = {
/*cyls density  bps   "n"  spt hds gplrw  gplfor   size    partition */
  40, FLPY_MFM, 1024, 0x03, 4,  2, 0x80,  0xF0,  DEV5FLPY, Sdf0, /* DS/DD 1024*/
  40, FLPY_FM,  128,  0x00, 16, 1, 0x10,  0x19,  DEV5FLPY, Ssyf0,/* SS/SD 128 */
  40, FLPY_MFM, 512,  0x02, 8,  2, 0x2A,  0x50,  DEV5FLPY, Sdzf0,/* DS/DD 512 */
  40, FLPY_MFM, 512,  0x02, 9,  2, 0x33,  0x70,  DEV5FLPY, Sdvf0,/* DS/DD 512 */
  40, FLPY_MFM, 1024, 0x03, 4,  2, 0x80,  0xF0,  DEV5FLPY, Sboot /* DS/DD 1024*/
};

#endif TPI_48


#if TPI_96
/*******************************************************************************
 *
 * 96 tracks per inch, 80 cylinders total
 *
 ******************************************************************************/

/* Partition tables */

/*
 * DS/DD 5.25" Floppy, 1024-byte sector Partitions.  Track 0 unused.
 */
struct	i208part Sdf0[] = {
	4,		636		/* rest: 636k */
};

/*
 * DS/DD 5.25" Floppy, 1024-byte sector Partitions.  Tracks 0 and 1 unused.
 */
struct	i208part Sboot[] = {
	8,		184,		/* boot: 184k */
	192,		128		/* ram:  128k */
};

/*
 * SS/SD 5.25" Floppy, 128-byte sector Partitions.
 * Needed only for writing the boot track, to create boot floppies.
 */
struct	i208part Ssyf0[] = {
	0,		2560		/* all of disk: 320K */
};

/*
 * DS/DD 5.25" Floppy, 512-byte sectors, 8 sectors per track.
 */
struct	i208part Sdzf0[] = {
	  0,		1280		/* all of disk: 640k */
};

/*
 * DS/DD 5.25" Floppy, 512-byte sectors, 9 sectors per track.
 */
struct	i208part Sdvf0[] = {
	  0,		1440		/* all of disk: 720k */
};


/* Constant drive table characteristics */

/*
 * 208 Board 0, Unit 0 Device-table Definitions (drtab's)
 */
struct i208cdrt i208d00[] = {
/*cyls density  bps   "n"  spt hds gplrw  gplfor   size    partition */
  80, FLPY_MFM, 1024, 0x03, 4,  2, 0x80,  0xF0,  DEV5FLPY, Sdf0, /* DS/DD 1024*/
  80, FLPY_FM,  128,  0x00, 16, 1, 0x10,  0x19,  DEV5FLPY, Ssyf0,/* SS/SD 128 */
  80, FLPY_MFM, 512,  0x02, 8,  2, 0x2A,  0x50,  DEV5FLPY, Sdzf0,/* DS/DD 512 */
  80, FLPY_MFM, 512,  0x02, 9,  2, 0x33,  0x70,  DEV5FLPY, Sdvf0,/* DS/DD 512 */
  80, FLPY_MFM, 1024, 0x03, 4,  2, 0x80,  0xF0,  DEV5FLPY, Sboot /* DS/DD 1024*/
};

#endif TPI_96


/*
 * 208 Board Configuration
 *
 * This configures one 208 board with unit [0] valid. 
 */
struct i208cfg  i208cfg[] = {
/* Address  Devcod   int_level  HUT   HLT   SRT  Device-table[unit] */
   0x0180, DEV5FLPY,     3,    0x0F, 0x26, 0x05, i208d00,     /* unit [0] */
						 0,	      /* unit [1] */
						 0,	      /* unit [2] */
						 0	      /* unit [3] */
};


/*
 * i208minor
 *
 * This table maps the minor number to the board number, unit,
 * drtab subscript, and partition subscript. 
 */
unsigned i208minor[]={

/*	i208MINOR(board#, unit#, drtab#, partition#)		*/
					/* [minor] device */
	i208MINOR(4,0,0,0),		/* [0] invalid */
	i208MINOR(4,0,0,0),		/* [1] invalid */
	i208MINOR(4,0,0,0),		/* [2] invalid */
	i208MINOR(4,0,0,0),		/* [3] invalid */
	i208MINOR(4,0,0,0),		/* [4] invalid */
	i208MINOR(4,0,0,0),		/* [5] invalid */
	i208MINOR(4,0,0,0),		/* [6] invalid */
	i208MINOR(4,0,0,0),		/* [7] invalid */
	i208MINOR(0,0,0,0),		/* [8] df0 floppy */
	i208MINOR(0,0,1,0),		/* [9] syf0 floppy */
	i208MINOR(0,0,2,0),		/* [10] dzf0 floppy */
	i208MINOR(0,0,3,0),		/* [11] dvf0 floppy */
	i208MINOR(0,0,4,0),		/* [12] boot part boot flpy */
	i208MINOR(0,0,4,1),		/* [13] ram part boot flpy  */
	i208MINOR(4,0,0,0),		/* [14] invalid */
	i208MINOR(4,0,0,0),		/* [15] invalid */
	i208MINOR(4,0,0,0)		/* [16] invalid */
};

#endif MINI


#if FULL
/*******************************************************************************
 *
 * 8-inch (full-size) floppy drives
 *
 ******************************************************************************/


/* Partition tables */

/*
 * SS/DD Floppy, 1024-byte sector Partitions.  Track 0 unused.
 */
struct	i208part Sf0[] = {
	8,		608		/* [1] rest:  608K    */
};

/*
 * DS/DD Floppy, 1024-byte sector Partitions.  Track 0 unused.
 */
struct	i208part Sdf0[] = {
	8,		1224		/* [1] rest:  1224K   */
};

/*
 * SS/SD Floppy, 128-byte sector Partitions.
 * The main use for this is to access track 0 of a boot floppy,
 * but it can also be used to read "standard" SS/SD distribution disks.
 */
struct	i208part Ssf0[] = {
	   0,		2002		/* [2] The whole disk: 250K  */
};

/*
 * DS/DD Floppy, 256-byte sector Partitions.  Track 0 unused.
 */
struct	i208part Sdxf0[] = {
	  26,		3978		/* [0] rest:  3978 sectors: 994.5K */
};

/*
 * DS/DD Floppy, 256-byte sector Partitions.  Track 0 unused.
 *		boot floppy format.
 */
struct	i208part Sboot[] = {
	  26,		780,		/* [0] root:  780 sectors: 195k */
	 806,		832		/* [1] ram: 832 sectors: 208k */
};


/* Constant drive table characteristics */

/*
 * 208 Board 0, Unit 0 Device-table Definitions (drtab's)
 */

struct i208cdrt i208d00[]={
/*cyls  mode    bps    "n" spt hds gplrw gplfor   size    partition */
  77, FLPY_MFM, 1024, 0x03, 8,  1, 0x35, 0x74,  DEV8FLPY, Sf0,  /* SS/DD 1024 */
  77, FLPY_MFM, 1024, 0x03, 8,  2, 0x35, 0x74,  DEV8FLPY, Sdf0, /* DS/DD 1024 */
  77, FLPY_FM,  128,  0x00, 26, 1, 0x07, 0x1B,  DEV8FLPY, Ssf0, /* SS/SD 128  */
  77, FLPY_MFM, 256,  0x01, 26, 2, 0x0E, 0x36,  DEV8FLPY, Sdxf0,/* DS/DD 256  */
  77, FLPY_MFM, 256,  0x01, 26, 2, 0x0E, 0x36,  DEV8FLPY, Sboot /* DS/DD 256  */
};


/*
 * 208 Board Configuration
 *
 * This configures one 208 board with unit [0] valid. 
 */

struct i208cfg  i208cfg[] = {
/* Address   Devcod   int_level  HUT   HLT   SRT  Device-table[unit] */
   0x0180,  DEV8FLPY,    3,     0x0F, 0x26, 0x05, i208d00,     /* unit [0] */
						  0,	       /* unit [1] */
						  0,	       /* unit [2] */
						  0	       /* unit [3] */
};


/*
 * i208minor
 *
 * This table maps the minor number to the board number, unit,
 * drtab subscript, and partition subscript. 
 */
unsigned i208minor[]={

/*	i208MINOR(board#, unit#, drtab#, partition#)		*/
					/* [minor] device */
	i208MINOR(4,0,0,0),		/* [0] invalid */
	i208MINOR(4,0,0,0),		/* [1] invalid */
	i208MINOR(4,0,0,0),		/* [2] invalid */
	i208MINOR(4,0,0,0),		/* [3] invalid */
	i208MINOR(4,0,0,0),		/* [4] invalid */
	i208MINOR(4,0,0,0),		/* [5] invalid */
	i208MINOR(4,0,0,0),		/* [6] invalid */
	i208MINOR(4,0,0,0),		/* [7] invalid */
	i208MINOR(0,0,0,0),		/* [8] f0 floppy */
	i208MINOR(0,0,2,0),		/* [9] syf0 floppy */
	i208MINOR(0,0,3,0),		/* [10] dxf0 floppy */
	i208MINOR(0,0,1,0),		/* [11] df0 floppy */
	i208MINOR(0,0,4,0),		/* [12] root part boot floppy */
	i208MINOR(0,0,4,1),		/* [13] ram part boot floppy */
	i208MINOR(4,0,0,0),		/* [14] invalid */
	i208MINOR(4,0,0,0),		/* [15] invalid */
	i208MINOR(4,0,0,0)		/* [16] invalid */
};

#endif FULL



/*******************************************************************************
 *
 * The following are static initialization variables
 * which are based on the configuration.
 * These values are self-configuring based on the above, and should
 * not be changed.
 *
 ******************************************************************************/

/* Number of boards configured */
#define	NUM208	((sizeof i208cfg) / (sizeof (struct i208cfg)))

int N208 = NUM208;		/* # of 208 boards in system */
struct buf		i208tab[NUM208];	/* buffer headers */
struct i208dev		i208dev[NUM208];  	/* device char table */
struct buf		i208rbuf[NUM208]; 	/* raw buffer headers */
struct i208dev		*i208bdd[NUM208];	/* dev table; board indexed */
