/*
 * 	@(#) c226.c $State: XENIX_R3-5 $ $Date: 86/07/25 16:55:53 $
 *	INTEL CORPORATION PROPRIETARY INFORMATION
 *
 *	Copyright (c) 1986 Intel Corp.
 *	This software is supplied under the terms
 *	of a license agreement or nondisclosure
 *	agreement with Intel Corporation and may
 *	not be copied nor disclosed except in
 *	accordance with the terms of that agreement.
 */

/**********************************************************************
 *
 * TITLE:	iSBC 226 Device Driver
 *
 * DATE:	January 4, 1985
 *
 * AUTHOR:	Richard N. Bryant
 *
 * SUMMARY:
 *	iSBC 226 driver configuration file.
 *
 *	There is one such for each disk type known to /usr/lib/smd/smdcfg.  
 *	These are used by instl when Xenix is installed from boot floppies.  
 *	They are used here to specify the type of known winchesters.
 *
 * MODIFICATIONS:
 *	I001	01/08/86	jdm
 *		Added bad block testing configuration.
 *	
 **********************************************************************/

/*
 * Required include files.
 * Order is important.
 */
#include "../h/param.h"
#include "../h/buf.h"
#include "../h/conf.h"
#include "../h/dir.h"
#include "../h/a.out.h"
#include "../h/user.h"
#include "../h/systm.h"
#include "../h/iobuf.h"
#include "../h/i226.h"



/*
 * List of drive sizes describing each drive's geometry.
 * Note:
 *	The eighth (MSB) bit must be set in
 *	the header offset field (hdoffset) for
 *	embedded servo (hard sectored) drives.
 *
 *	The total physical sectors per track is
 *	calculated as (sec/trk + slipsectors), where
 *	'slipsectors' is the number of reserved sectors
 *	per track. 'sec/trk' must be used to calcuate
 *	the partition table and as formatting parameter.
 *
 * Size Fields:
 *	cylin,	fhds,	rhds,	hdoffset,	sec/trk,    secsiz,   slipsectors
 */
struct	i226size

	/* PPC332, PPC DX332, 332MB SMD Drive, 1 Slip Sector(s) */
	s_PPC332_1[1] = {
		1649,	10,	0,	0,	17,	1024,	1,
	},

	/* PPC332, PPC DX332, 332MB SMD Drive, 3 Slip Sector(s) */
	s_PPC332_3[1] = {
		1649,	10,	0,	0,	15,	1024,	3,
	},

	/* FUJ2312, Fujitsu 2312, 84 MB SMD Drive, 1 Slip Sector(s) */
	s_FUJ2312_1[1] = {
		589,	7,	0,	0,	17,	1024,	1,
	},

	/* FUJ2312, Fujitsu 2312, 84 MB SMD Drive, 3 Slip Sector(s) */
	s_FUJ2312_3[1] = {
		589,	7,	0,	0,	15,	1024,	3,
	},

	/* AMS315, Century 315-1, 315 MB SMD Drive, 3 Slip Sector(s) */
	s_AMS315S_3[1] = {
		845,	19,	0,	SERVO+0,	15,	1024,	3,
};

/*
 * List of partition table configurations.
 * One configuration for each unique drive type.
 * Note:
 *	Partitions must start on track boundries,
 *	and is preferable to start on cylinder
 *	boundries for efficiency reasons.
 */

struct	i226part

	/* PPC332, PPC DX332, 332MB SMD Drive, 1 Slip Sector(s) */
	p_PPC332_1[] = {
		0,	280160,	/* #1,	partition '0' Whole disk drive. */
		17,	9231,	/* #2,	partition 'a' 9 Meg. */
		9248,	16388,	/* #3,	partition 'b' 16 Meg. */
		25636,	254507,	/* #4,	partition 'c' Whole usable disk minus the size of partitions a & b. */
		17,	140063,	/* #5,	partition 'd' one-half of the whole usable disk. */
		140080,	140063,	/* #6,	partition 'e' one-half of the whole usable disk. */
		17,	93364,	/* #7,	partition 'f' one-third of the whole usable disk. */
		93381,	93364,	/* #8,	partition 'g' one-third of the whole usable disk. */
		186745,	93364,	/* #9,	partition 'h' one-third of the whole usable disk. */
		17,	280126,	/* #10,	partition 'i' Whole usable disk. */
		17,	70023,	/* #11,	partition 'j' one-quarter of the whole usable disk. */
		70040,	70023,	/* #12,	partition 'k' one-quarter of the whole usable disk. */
		140063,	70023,	/* #13,	partition 'l' one-quarter of the whole usable disk. */
		210086,	70023,	/* #14,	partition 'm' one-quarter of the whole usable disk. */
		0,	17,	/* #15,	partition 'n' TRACK 0, Volume Label */
		280143,	17,	/* #16,	partition 'o' error partition. */
	},

	/* PPC332, PPC DX332, 332MB SMD Drive, 3 Slip Sector(s) */
	p_PPC332_3[] = {
		0,	247200,	/* #1,	partition '0' Whole disk drive. */
		15,	9225,	/* #2,	partition 'a' 9 Meg. */
		9240,	16395,	/* #3,	partition 'b' 16 Meg. */
		25635,	221550,	/* #4,	partition 'c' Whole usable disk minus the size of partitions a & b. */
		15,	123585,	/* #5,	partition 'd' one-half of the whole usable disk. */
		123600,	123585,	/* #6,	partition 'e' one-half of the whole usable disk. */
		15,	82380,	/* #7,	partition 'f' one-third of the whole usable disk. */
		82395,	82380,	/* #8,	partition 'g' one-third of the whole usable disk. */
		164775,	82380,	/* #9,	partition 'h' one-third of the whole usable disk. */
		15,	247170,	/* #10,	partition 'i' Whole usable disk. */
		15,	61785,	/* #11,	partition 'j' one-quarter of the whole usable disk. */
		61800,	61785,	/* #12,	partition 'k' one-quarter of the whole usable disk. */
		123585,	61785,	/* #13,	partition 'l' one-quarter of the whole usable disk. */
		185370,	61785,	/* #14,	partition 'm' one-quarter of the whole usable disk. */
		0,	15,	/* #15,	partition 'n' TRACK 0, Volume Label */
		247185,	15,	/* #16,	partition 'o' error partition. */
	},

	/* FUJ2312, Fujitsu 2312, 84 MB SMD Drive, 1 Slip Sector(s) */
	p_FUJ2312_1[] = {
		0,	69972,	/* #1,	partition '0' Whole disk drive. */
		17,	9231,	/* #2,	partition 'a' 9 Meg. */
		9248,	16388,	/* #3,	partition 'b' 16 Meg. */
		25636,	44319,	/* #4,	partition 'c' Whole usable disk minus the size of partitions a & b. */
		17,	34969,	/* #5,	partition 'd' one-half of the whole usable disk. */
		34986,	34969,	/* #6,	partition 'e' one-half of the whole usable disk. */
		17,	23324,	/* #7,	partition 'f' one-third of the whole usable disk. */
		23341,	23324,	/* #8,	partition 'g' one-third of the whole usable disk. */
		46665,	23307,	/* #9,	partition 'h' one-third of the whole usable disk. */
		17,	69938,	/* #10,	partition 'i' Whole usable disk. */
		17,	17476,	/* #11,	partition 'j' one-quarter of the whole usable disk. */
		17493,	17476,	/* #12,	partition 'k' one-quarter of the whole usable disk. */
		34969,	17476,	/* #13,	partition 'l' one-quarter of the whole usable disk. */
		52445,	17476,	/* #14,	partition 'm' one-quarter of the whole usable disk. */
		0,	17,	/* #15,	partition 'n' TRACK 0, Volume Label */
		69955,	17,	/* #16,	partition 'o' error partition. */
	},

	/* FUJ2312, Fujitsu 2312, 84 MB SMD Drive, 3 Slip Sector(s) */
	p_FUJ2312_3[] = {
		0,	61740,	/* #1,	partition '0' Whole disk drive. */
		15,	9225,	/* #2,	partition 'a' 9 Meg. */
		9240,	16395,	/* #3,	partition 'b' 16 Meg. */
		25635,	36090,	/* #4,	partition 'c' Whole usable disk minus the size of partitions a & b. */
		15,	30855,	/* #5,	partition 'd' one-half of the whole usable disk. */
		30870,	30855,	/* #6,	partition 'e' one-half of the whole usable disk. */
		15,	20580,	/* #7,	partition 'f' one-third of the whole usable disk. */
		20595,	20580,	/* #8,	partition 'g' one-third of the whole usable disk. */
		41175,	20565,	/* #9,	partition 'h' one-third of the whole usable disk. */
		15,	61710,	/* #10,	partition 'i' Whole usable disk. */
		15,	15420,	/* #11,	partition 'j' one-quarter of the whole usable disk. */
		15435,	15420,	/* #12,	partition 'k' one-quarter of the whole usable disk. */
		30855,	15420,	/* #13,	partition 'l' one-quarter of the whole usable disk. */
		46275,	15420,	/* #14,	partition 'm' one-quarter of the whole usable disk. */
		0,	15,	/* #15,	partition 'n' TRACK 0, Volume Label */
		61725,	15,	/* #16,	partition 'o' error partition. */
	},

	/* AMS315, Century 315-1, 315 MB SMD Drive, 3 Slip Sector(s) */
	p_AMS315S_3[] = {
		0,	240540,	/* #1,	partition '0' Whole disk drive. */
		15,	9225,	/* #2,	partition 'a' 9 Meg. */
		9240,	16395,	/* #3,	partition 'b' 16 Meg. */
		25635,	214890,	/* #4,	partition 'c' Whole usable disk minus the size of partitions a & b. */
		15,	120255,	/* #5,	partition 'd' one-half of the whole usable disk. */
		120270,	120255,	/* #6,	partition 'e' one-half of the whole usable disk. */
		15,	80160,	/* #7,	partition 'f' one-third of the whole usable disk. */
		80175,	80160,	/* #8,	partition 'g' one-third of the whole usable disk. */
		160335,	80160,	/* #9,	partition 'h' one-third of the whole usable disk. */
		15,	240510,	/* #10,	partition 'i' Whole usable disk. */
		15,	60120,	/* #11,	partition 'j' one-quarter of the whole usable disk. */
		60135,	60120,	/* #12,	partition 'k' one-quarter of the whole usable disk. */
		120255,	60120,	/* #13,	partition 'l' one-quarter of the whole usable disk. */
		180375,	60120,	/* #14,	partition 'm' one-quarter of the whole usable disk. */
		0,	15,	/* #15,	partition 'n' TRACK 0, Volume Label */
		240525,	15,	/* #16,	partition 'o' error partition. */
};

/*
 * Configuration table. One entry for each controller:
 *	interrupt level from PIC, I/O wakeup address,
 *	and four drive entries each with number of partitions
 *	and pointers to drive size structure and drive partition
 *	table structure.
 */
struct	i226cfg	i226cfg [] = {

		/* Configuration table for board 0 */
	2,			/* interrupt level in octal, see conf/master */
	0xee40,			/* base address for I/O ports in hexadecimal */

	16,			/* number of partitions for drive 0, 1 indexed */
	s_PPC332_3,			/* pointer to size table for drive 0 */
	p_PPC332_3,			/* pointer to partition table for drive 0 */

	0,			/* number of partitions for drive 1, 1 indexed */
	0,			/* pointer to size table for drive 1 */
	0,			/* pointer to partition table for drive 1 */

	0,			/* number of partitions for drive 2, 1 indexed */
	0,			/* pointer to size table for drive 2 */
	0,			/* pointer to partition table for drive 2 */

	0,			/* number of partitions for drive 3, 1 indexed */
	0,			/* pointer to size table for drive 3 */
	0,			/* pointer to partition table for drive 3 */
};


/*
 * Driver's private data.
 */

/* Number of iSBC 226 controllers defined. */
#define	N226		1		/* number of controllers defined */
int i226num = N226;

/*
 * PB_SIZE specifies the number of IOPB's in the pool used by the
 * controllers. This parameter may be changed for performance or
 * memory conservation reasons. It is recommended that PB_SIZE
 * is increased by at least four for each controller.
 */
#define	PB_SIZE		(1 * N226)	/* number of IOPB's in driver */

/* One board structure for each controller. */
struct i226board i226board[N226];

/* One IOPB queue for free list. */
struct i226pb_queue i226freeq;

/* Pool of available IOPB's. */
struct i226iopb i226iopb[PB_SIZE];

/* Number of IOPB's. Used to calculate maximum queue length. */
ushort i226qsize = PB_SIZE;

/* Buffer headers for Raw Character I/O, one per drive. */
struct buf i226rbuf[N226*NDRV];

/* One request queue for each controller. */
struct iobuf i226tab[N226];

/*
 * Control value to select the number of test patterns to
 * use during format. Must be between 1 and 6.
 * Affects all drives.
 */
int i226patterns = 1;


#ifdef BBFAKE
/***********************************************************************
		I001 DEBUGGING: Fake bad block configuration.
*/

struct i226_bb_fake i226fake[3] = {
	100L,	0x0602,	PB_ERR,	HARDECCERR,
	200L,	0x0602,	PB_ERR,	HARDECCERR,
	300L,	0x0602,	PB_ERR,	HARDECCERR
};

int	i226_bb_max = 3;

/*
	END I001 DEBUGGING
************************************************************************/
#endif BBFAKE
