/*
 *	INTEL CORPORATION PROPIETARY INFORMATION
 *
 *	This software is supplied under the terms
 *	of a license agreement or nondisclosure
 *	agreement with Intel Corporation and may
 *	not be copied nor disclosed except in
 *	accordance with the terms of that agreent.
 */

/**********************************************************************
 *
 * TITLE:	iSBX 344 BITBUS controller driver include file.
 *
 * DATE:	June 20, 1985
 *
 * AUTHOR:	Jim Harriger
 *
 * SUMMARY:	this file holds declarations for the configuration
 *		structure and various constants for the BITBUS	
 *		device driver.
 *
 *
 *
 **********************************************************************/
#include "mmu.h"

/**********************************************************************
 *	configuration structure.  initialized in c344.c
 *********************************************************************/
struct	i344cfg {
	unsigned	i344data;
	unsigned	i344cmd;
	unsigned	i344stat;
	unsigned	i344rint;
	unsigned	i344tint;
};

/**********************************************************************
 *	some general purpose constants
 **********************************************************************/
#define	TRUE		1
#define	FALSE		0
#define	INPUT		1
#define	OUTPUT		0
#define	IGNORE_INPUT	2
#define	TRANSMIT	1
#define	RECEIVE		0
#define	I344PRI		27	  /* sleeping priority (first crack=27)	*/
#define	i344_ACC	DSA_DATA  /* access rights for exported segment	*/
#define	MINORMASK	0x1f	  /* for getting minor num from dev	*/
/******************************************************************
 *	constants used for reading and comparing status from
 *		iSBX 344's status register.
 ******************************************************************/
/*	RFNE:	Receive FIFO Not Empty: 1 means it is empty: got nothing
 *					0 means it is not empty: got something
 */
#define	RFNE	0x10
/*	TFNF:	Transmit FIFO Not Full: 1 means it is full: may not transmit
 *					0 means it is not full: may transmit
 */
#define	TFNF	0x01
/*	RCMD:	Receive Command:	1 means the current byte in the RFIFO
 *						is data.
 *					0 means the current byte in the RFIFO
 *						is the end of complete message.
 */
#define	RCMD	0x20
/*	RESET:	code sent to command port to do a software reset of
 *		the DCM soft/firm ware.
 */
#define	RESET	0x08
#define	EOM	0

/*********************************************************************
 *	constants for use of ioctl's.
 *********************************************************************/

#define SIOVERSION	0x00
#define SIORESET	0x01
#define SIODATA		0x02
#define SIOCOMMAND	0x03
#define SIOSTATUS	0x04
#define SIOAVAIL	0x05
#define SIOMSIZE	0x06

/*********************************************************************
 *	constants for some of the positions within the BITBUS message
 *		structure.
 *********************************************************************/

#define LENGTH		0
#define	ROUTING		1
#define NODE		2
#define TASKS		3
#define	RESPONSE	4
#define	DATA0		5
#define	DATA1		6
#define	DATA2		7
#define	DATA3		8
#define	DATA4		9
#define	DATA5		10
#define	FW0		11
#define	FW1		12
#define M0		13
#define M1		14
#define M2		15
#define M3		16
#define M4		17

/*
 *   E_UNKNOWN_RAC_COMMAND: This is what a 344 with v1.0 firmware returns in 
 *                          response to a MOD_NAME command.
 */

#define E_U_RAC_CMD	0x96

/***********************************************************************
 *	constants used in i344_state variable
 *
 *	c		set if i344intr waiting for receive queue to drain
 *	 b		set if board is a 345, reset if a 344.
 *   	  r		set if output is currently running
 *	   o		set if i344write waiting for transmit queue to drain
 *	    i		set if i344read waiting for receive queue to fill up
 *	     O		set if device is open
 *	      E		set if opened with exclusive access
 *	       A	set if module is alive
 ***********************************************************************/
#define ALIVE		0x01
#define	EXCLUSIVE	0x02
#define	OPEN		0x04
#define	i_WANTED	0x08
#define o_WANTED	0x10
#define	RUNNING		0x20
#define	iSBX_345	0x40
#define	i_CLOGGED       0x80


#ifdef PCAT

#define SLAVE_MASK_REG	0x20
#define TINT_MASK       0x20   /* 00100000  = IRQ5        */
#define RINT_MASK       0x08   /* 00001000  =        IRQ3 */
#define	BITBUS_MASK	(TINT_MASK | RINT_MASK)

#else

#define SLAVE_MASK_REG	0xc6
#define TINT_MASK       0x10   /* 00010000  = INT4        */
#define RINT_MASK       0x08   /* 00001000  =        INT3 */
#define	BITBUS_MASK	(TINT_MASK | RINT_MASK)

#endif
