
/*
 *  This is for the printer which is attached to the 8255 on the
 *	286/10 board...
 *
 *     8255 is programmed for mode 0 (mode word: 82H)
 *
 *     Port A & C: OUTPUT  Port B : INPUT
 *
 *  Port A bin definition:
 *	
 *	 1-7   - data
 *	  
 *  Port B bit definition: 
 *       
 *       0-3,5 - Not used
 *         4   - Line Printer ACK/
 *         6   - Line Printer Error
 *         7   - Line Printer Busy
 *
 *  Port C bit definition:
 *
 *       1-6   - Not used **** Note bit 1 used by system so don't touch *****
 *         0   - Character strobe to the printer
 *         7   - Clear Line Printer Ack Flop/
 *
 *  This is for the printer which is attached to the 8255 on the
 *	iSBX350 attached to the 386/2x.
 *
 *     8255 is programmed for mode 1 (mode word: a8H)
 *
 *     Port A : OUTPUT  Port B : not used
 *     Port C : printer control signals
 *
 *  Port A bin definition:
 *	
 *	 1-7   - data
 *	  
 *  Port C bit definition: 
 *       
 *         0   - STB/ (strobe char to printer)
 *         1   - not used
 *         2   - not used
 *         3   - MINTR1 (interrupt to the 386/2x)
 *         4   - Line Printer SLCT (printer select)
 *         5   - Line Printer ERROR (paper out)
 *         6   - Line Printer ACK/
 *
 * static char lp_h_version[] ="@(#) lp.h $State: XENIX_R3-5 $ $Date: 87/02/02 10:54:12 $";
*/
#define	LPLWAT		50		/* line printer low water mark */
#define	LPPRI		(PZERO+1)	/* allow sleepers to wakeup early */
#define LP_BDELAY	(50)		/* clicks between not-ready checks */
#define LP_DDELAY	(25)		/* clicks delay for polled wakeup */
#define LP_MSG   	1		/* allow error mesage display */
#define LP_NO_MSG   	0		/* allow error mesage display */
/*
 *	Hardware constants
*/
#define PT_INIT		0x82	/* 8255 command word for port init */
#define PT_OVERRIDE	0x2	/* written to port C to set bit 1 */
#define SET_PR_ACK	0xf	/* set printer ACK bit */
#define CL_PR_ACK	0xe 	/* clear printer ACK bit */
#define TEST		0xaa	/* test pattern read back by probe */
#define PR_ACK_BAR	0x10 	/* printer ACK line */
#define PR_ERROR	0x40 	/* printer error bit */
#define PR_BUSY		0x80 	/* printer busy bit */
#define ONSTROBE	0x1	/* Turn data strobe signal on */
#define OFFSTROBE	0	/* Turn data strobe signal off */
#define LP_PORTA	0xc8
#define LP_PORTB	0xca
#define LP_PORTC	0xcc
#define LP_CONTROL	0xce
#define LP_INT_LEVEL	0107	/* slave pic, level 7 */
/* similar to above, but for iSBX350 */
#define LP_PORTA_350	0x80
#define LP_PORTB_350	0x82
#define LP_PORTC_350	0x84
#define LP_CONTROL_350	0x86
#define LP_INT_LEVEL_350	0104	/* slave pic, level 4 */
#define PT_INIT_350	0xa8
#define PR_SELECT_350	0x10
#define PR_ERROR_350	0x20
#define PR_ACK_BAR_350	0x40
#define CLEAR_OBF_350	0x0e	/* clear OBF/ to turn off interrupt  */
#define ENABLE_INT_350	0x0d	/* enable interrupt - set port c bit 6 */
/*
* lp states
*/
#define LP_ALIVE	0x01	/* lp hardware  present */
#define LP_DAEMON	0x02	/* turn on daemon to do polled wakeup */
#define LP_RESET	0x04	/* interrupt flip/flop needs reseting */
/*
 *	Device Structures
*/
struct lpcfg {
		int p_level;	/* intr level */
		int p_porta;	/* 8255 port a -> data out */
		int p_portb;	/* 8255 port b -> status in */
		int p_portc;	/* 8255 port c -> strobe and interrupt f/f */
		int control;	/* 8255 control port */
};


