/*
*
*   "@(#) $netsys.h $State: XENIX_R3-5 $ $Date: 86/05/30 13:11:21 $";
*   sys/nfs/netsys.h	 Copyright Microsoft Corporation, 1985 
*
*	TITLE:	Xenix Networking Software system entry definition
*
*	AUTHORS: Bob Rinne & Ken Masden
*		
*		Intel Corporation
*		2402 W. Beardsley Road
*		Phoenix, Arizona  85027
*
*	DATE:	01 February 1985
*
*	DESC:	This include file defines the user level system entry
*		for Xenix Networking softare services.
*
*	HIST:	I000	21 March 1985	Ken Masden
*		- added header and comments
*		I001	5 May 1985	Bob Rinne
*		- added LNODENAME and REMCD command values.
*		I002	16 Dec 1985	Bob Rinne
*		- added chain and memory return command values.
*       I003	3 April 1986 Brad Marrs
*		- cleaned up a comment
*/
/* Xenix OpenNET system entry */

#define netsysent(CMD, ARGP)  syscall(40, 21, CMD, ARGP, 0, 0)

/* CMD values */

#define ISNET		0x0085	/* is network consumer enabled		*/
#define ISREMOTE	0x0086	/* is current dir remote		*/
#define NVERS		0x0090	/* Xenix OpenNET banner			*/
#define ISSERV		0x0091	/* is network server enabled		*/
#define LNODENAME	0x0092	/* Local node name			*/
#define REMCD		0x0093	/* is process's current dir remote	*/
#define NIOBUFSIZ	0x0804	/* network data transfer size		*/
#define NFS_GMONBUF	0x0812	/* get a buffer to monitor		*/
#define NFS_RMONBUF	0x0813	/* return monitor'ed buffer		*/
#define NFS_GADDR	0x0814	/* get data at address			*/
#define NFS_RADDR	0x0815	/* return data to address		*/
#define NFS_GCHAIN	0x0816	/* return data from specified token chain */

/*  struct isnet is used with ISNET and ISREMOTE commands.	*/

struct isnet
{
  int  n_delsiz;      /* number of network delimiters (1 or 2) */
  char n_del1;	      /* first delimeter		      */
  char n_del2;	      /* second delimeter (if present)	      */
  char n_netnm[16];   /* local node name if cdir is local     */
  /* remote node name if cdir is remote	  */
  /* (ISREMOTE command only)		  */
};

/*
*   ISNET returns the following:
* 
*   1 in means the network is up
*   0 means it isn't
* 
*   At the location "&isnet", the following is returned.
* 
*   +----------------+
*   | count of chars |	for the network delimiter
*   +----------------+
*   + char1 | char2  |
*   +----------------+
*
*   e.g. int netstatus;
*	 netstatus = netsysent(ISNET, &isnet);
*/
/*
*   ISREMOTE does the following:
*   Check the current working directory
*   place the following information at the location addressed by "&isnet".
* 
*   +----------------+
*   | count of chars |	for the network delimiter
*   +----------------+
*   + char1 | char2  |
*   +----------------+
*   | node    name   |	node name where cdir is located
*   |	for cdir     |	(may be either local node or remote node)
*   +----------------+
*
*   return:	1 -> current directory is remote
*		0 -> current directory is local
*
*   e.g. int remotecdir;
*	 remotecdir = netsysent(ISREMOTE, &isnet);
*
*/
/*
*   NVERS returns the char string defining the Xenix Networking Software
*   version currently installed on the local node.
*
*   e.g. char networkvers[81];								 I005
*	 netsysent(NVERS, &networkvers[0]);
*
*/
/*
*   ISSERV	returns 1 if Xenix OpenNET server is active,
*			0 if server is not active.
*/ 
/*
*   NIOBUFSIZ  Network io buffer size:
*   returns the current size of the Xenix Networking Software 
*   io buffer (# chars for read or write).
*
*   For example:
*   int bufsiz;
*   char *fbuf;
*   bufsiz = netsysent(NIOBUFSIZ, NULL); / * get current buffer size * /
*   if (bufsiz <= 0)
*     bufsiz = BUFSIZE;
*   fbuf = sbrk(bufsiz);
*
*/
