# Copyright Intel Corp. 1985
# Author:  Ed Reeder & J. Jackson
#	oer	11/06/85
#		Added checks for EACH /dev comm device
#		Added support for multiple serial printers
#		of the same type (should really be done in the
#		configuration file before scpbuild.ss gets control).
#	seckin	3/19/87
#		Modified copying/moving of ttys and ttytype files, in order to
#		preserve the original uid/gid and permissions.
#	seckin	3/23/87
#               Corrected modifications of 3/19 (was losing ttys & ttytype).
#	seckin	7/7/87
#               Removing ttyf? entries from the old ttys & ttytype, while
#		building new ones, for upgrades from 286s to 386s.
#
# scpbuild.2.ss_version[] ="@(#) scpbuild.2.ss $State: XENIX_R3-5 $ $Date: 87/07/07 12:18:44 $";

# To test, set up required shell variables (use the current configuration
# as CFPROPOSED) and invoke scpbuild.ss directly
# Typical test scenario:
#   SCPDEBUG=Y
#   SCPPATH=`pwd`
#   INITIAL=Y or N
#   ZAPLOG=/tmp/zaplog
#   INSTPROPOS_2=/tmp/inst.2
#   CFPROPOSED=$SCPPATH/configurations/current
#   export SCPDEBUG SCPPATH INITIAL ZAPLOG INSTPROPOS_2 CFPROPOSED
#   rm /tmp/*
#   cp /sys/conf/xenixconf /tmp
#   cp /sys/conf/master    /tmp
#   cp /sys/cfg/c215g.c    /tmp
#   scpbuild.ss
#   sh /tmp/inst.2
#   more /tmp/*

# Builds a shellscript that will later be executed to 'make' the configuration.
# At various points, prints a dot to console to let user know it is progressing.

# NOTE: INITIAL=Y could exist if a user respecifies his initial configuration.
#       Don't assume that your are starting with an "as built" system.

# Allow the 'sys' directory to be in / or /usr
# If both, /usr has precedence
[ -f /sys/conf/xenixconf ] && SYSDIR=/sys
[ -f /usr/sys/conf/xenixconf ] && SYSDIR=/usr/sys

# Allow for a "non-destructive" scp
# MADE=14, OK=0, ORIGIFS=$IFS	# to allow SCPDEBUG=N and rest of debug env. set

case "$SCPDEBUG" in
  Y)
     COMMENT="#DEBUG"
     SLASH=/tmp/
     ETC=/tmp
     ETCDFLT=/tmp
     SYSCFG=/tmp
     SYSCONF=/tmp
     ;;
  *)
     COMMENT=
     SLASH=/
     ETC=/etc
     ETCDFLT=/etc/default
     SYSCFG=$SYSDIR/cfg
     SYSCONF=$SYSDIR/conf
esac
export COMMENT SLASH ETC ETCDFLT SYSCFG SYSCONF

# Start with a null instruction file
cat <<-HERE > $INSTPROPOS_2
	# `date` INSTRUCTIONS CREATED
	# If any command ends with a non-zero status, end instructions.
	set -e
HERE
##############################################################################
# Set pathnames
cat <<-HERE >> $INSTPROPOS_2
	# Set pathnames and other variables
	COMMENT=$COMMENT
	SLASH=$SLASH
	ETC=$ETC
	ETCDFLT=$ETCDFLT
	SYSCFG=$SYSCFG
	SYSCONF=$SYSCONF
	SYSDIR=$SYSDIR
	export COMMENT SLASH ETC ETCDFLT SYSCFG SYSCONF SYSDIR

HERE

##############################################################################
# Build /etc/ttys and /etc/ttytype

cat <<-HERE >> $INSTPROPOS_2
	echo \`date\` start building /etc/ttys and /etc/ttytype >> $ZAPLOG
	echo -n .
	#
	# Build /etc/ttys and /etc/ttytype
HERE
#
NEXTCUR=$SCPPATH/configurations/nextcurrent
NEXTCUR1=$SCPPATH/configurations/nextcurrent1
export NEXTCUR NEXTCUR1
awk -F, '
   BEGIN {
      printf "." >> "/dev/tty"
   }
   /^TERMBOARD/ {
      termboard[$1] = $3	# put the make argument in array
      print "   ( cd /dev "
      print "   '$COMMENT' make " $3 " > /dev/null 2>&1 "
      print "   ) "
   }
   /^TPORT/ {
      if( $1 != prev ) {
         tport[$1] = $1 "," $3	# put TPORTn,/dev name of 1st device in array
         prev = $1
      }
      # Need a separate array for each board
      devnames[$4] = "tty" $4	# get device names
   }
   END {
      # Build /etc/ttys and /etc/ttytype.
      print "# Build basic /etc/ttys"
      print "# Determines a ports baud rate and whether initially enabled/disabled"
      print "cp /etc/ttys $ETC/ttys.old"
      print "echo Previous /etc/ttys saved as /etc/ttys.old >>'$ZAPLOG'"
      print "egrep \"^..console\" $ETC/ttys.old >$ETC/ttys"
      print "cat <<HERE >>'$NEXTCUR'"
      print "/console/d"
      for( tty in devnames )
         print "/" devnames[tty] "/d"
      print "/ttya/d"
      print "/ttyb/d"
      print "/ttyc/d"
      print "/ttyd/d"
      print "/ttye/d"
      print "/ttyf/d"
      print "HERE"
      print "sed -f '$NEXTCUR' <$ETC/ttys.old >$ETC/ttys2"
      print "cat $ETC/ttys2 >>$ETC/ttys"
      print "rm -f '$NEXTCUR'"
      print "rm -f $ETC/ttys2"
      print "cat <<HERE | sort >>$ETC/ttys"
      for( tty in devnames )
         print "0?" devnames[tty]
      print "HERE"
      print "# Build basic /etc/ttytype"
      print "# Assigns default termcap terminal name for enabled ports"
      print "# Assigns usage name for disabled ports"
      print "cp /etc/ttytype $ETC/ttytype.old"
      print "echo Previous /etc/ttytype saved as /etc/ttytype.old >>'$ZAPLOG'"
      print "egrep \"console[ 	]*$\" $ETC/ttytype.old >$ETC/ttytype"
      print "cat <<HERE | sort >>$ETC/ttytype"
      for( tty in devnames )
          print "unused "devnames[tty]
      print "HERE"
      print "cat <<HERE >'$NEXTCUR1'"
      print "/console/d"
      for( tty in devnames )
         print "/" devnames[tty] "/d"
      print "/ttya/d"
      print "/ttyb/d"
      print "/ttyc/d"
      print "/ttyd/d"
      print "/ttye/d"
      print "/ttyf/d"
      print "HERE"
      print "sed -f '$NEXTCUR1' <$ETC/ttytype.old >$ETC/ttytype2"
      print "cat $ETC/ttytype2 >>$ETC/ttytype"
      print "rm -f '$NEXTCUR1'"
      print "rm -f $ETC/ttytype2"
      printf "." >> "/dev/tty"
      print "echo -n ."
   }
' $CFPROPOSED >> $INSTPROPOS_2
echo "echo \`date\` end building /etc/ttys and /etc/ttytype >> $ZAPLOG\n" >> $INSTPROPOS_2

##############################################################################
# Make certain /dev files are correct for hangup
cat <<-HERE >> $INSTPROPOS_2
	echo \`date\` start /dev hangup check >> $ZAPLOG
	echo -n .
	#
	# Make certain that /dev files are correct for the communications
	# ports.  Dial-in and Dial-out modem ports should have their
	# minor number bumped by 64 to enable hangup detection.
	# Non-modem ports should not have hangup detection enabled.
	ORIGIFS=\$IFS
	IFS=,\$ORIGIFS
	set +e
HERE
awk -F, '
   BEGIN {
      printf "." >> "/dev/tty"
   }
   /^TPORT/ {
      print "set x`ls -o " $3 "`   # get device characteristics"
      # If Dial-in or Dial-out Modem, ensure hangup enabled
      if( $6 == "I" || $6 == "M" ) {
         print "if [ $5 -lt 64 ] ; then  : check if hangup disabled"
         print "'$COMMENT'   rm -f " $3
         print "'$COMMENT'   /etc/mknod " $3 " c `echo $4 ` `expr $5 + 64`"
         print "'$COMMENT'   rm -f /dev/cua" $4 " /dev/cul"$4 " 2>/dev/null "
         print "'$COMMENT'   /etc/mknod  /dev/cua" $4 " c `echo $4 ` `echo $5`"
         print "'$COMMENT'   ln  /dev/cua"$4 " /dev/cul"$4
         print "'$COMMENT'   chmod 666 /dev/tty"$4 " /dev/cua"$4 " /dev/cul"$4
         print "fi"
         print "egrep \"ACU cul"$4 "\" /usr/lib/uucp/L-devices >/dev/null"
         print "if [ $? != 0 ] ; then"
         print "cat <<-HERE >>/usr/lib/uucp/L-devices "
         print "ACU cul" $4 " cua" $4 " "$8 " "
         print "HERE"
         print "fi"
      }
      else {
         print "if [ $5 -gt 63 ] ; then  : check if hangup enabled"
         print "'$COMMENT'   rm -f " $3
         print "'$COMMENT'   /etc/mknod " $3 " c `echo $4 ` `expr $5 - 64`"
         print "fi"
      }
   }
   END {
      printf "." >> "/dev/tty"
      print "echo -n ."
   }
' $CFPROPOSED >> $INSTPROPOS_2
cat <<-HERE >> $INSTPROPOS_2
	IFS=\$ORIGIFS
	set -e
HERE
echo "echo \`date\` end /dev hangup check >> $ZAPLOG\n" >> $INSTPROPOS_2

##############################################################################
# Set up /etc/ttytype entries
cat <<-HERE >> $INSTPROPOS_2
	echo \`date\` start /etc/ttytype mod. >> $ZAPLOG
	echo -n .
	#
	# Setup /etc/ttytype
	# Disabled ports will have a "usage" for terminal name
	sed "
HERE
awk -F, '
   BEGIN {
      printf "." >> "/dev/tty"
      prefix = "tty"
   }
   /^CONSOLE/ {
      printf "/^[^ 	]*[ 	]*%s/c\\\\\n%s %s\n", $4, $9 ,$4
   }
   /^TPORT/ {
      name = $9
      if( $6 == "M" ) name = "dialout"
      if( $6 == "O" ) name = "other"
      if( $6 == "P" ) name = "printer"
      if( $6 == "U" ) name = "unused"
      printf "/^[^ 	]*[ 	]*%s/c\\\\\n%s %s\n", prefix $4,name,prefix $4
   }
   END {
      print "\" < $ETC/ttytype > $ETC/ttytype2"
      print "cp $ETC/ttytype2 $ETC/ttytype"
      print "rm -f $ETC/ttytype2"
      printf "." >> "/dev/tty"
      print "echo -n ."
   }
' $CFPROPOSED >> $INSTPROPOS_2
echo "echo \`date\` end /etc/ttytype mod. >> $ZAPLOG\n" >> $INSTPROPOS_2

##############################################################################
# Set up /etc/ttys entries
cat <<-HERE >> $INSTPROPOS_2
	echo \`date\` start /etc/ttys mod. >> $ZAPLOG
	echo -n .
	#
	# Setup /etc/ttys
	# This technique assumes the system is in "single-user" mode,
	#   and that you do not have to use enable and disable.
	# Note also that the console does not have to be addressed.
	sed "
HERE
awk -F, '
   BEGIN {
      printf "." >> "/dev/tty"
      prefix = "tty"
   }
   /^TPORT/ {
      if( $7 == "E" ) state = "1"
         else state = "0"
      if( $8 == "9600" ) baud = "6"
         else if( $8 == "4800" ) baud = "5"
         else if(( $6 == "I" || $6 == "M" ) && $8 == "2400" ) baud = "9"
         else if( $8 == "2400" ) baud = "4"
         else if( $8 == "1200" ) baud = "3"
         else if( $8 == "300"  ) baud = "1"
         else if( $8 == "150"  ) baud = "0"
         else if( $8 == "110"  ) baud = "-"
         else baud = "?"
      printf "/^..%s/c\\\\\n%s\n", prefix $4, state baud prefix $4
   }
   END {
      print "\" < $ETC/ttys> $ETC/ttys2"
      print "cp $ETC/ttys2 $ETC/ttys"
      print "rm -f $ETC/ttys2"
      printf "." >> "/dev/tty"
      print "echo -n ."
   }
' $CFPROPOSED >> $INSTPROPOS_2
echo "echo \`date\` end /etc/ttys mod. >> $ZAPLOG\n" >> $INSTPROPOS_2

exit 0
