# Copyright Intel Corp. 1985, 1987
# Author:  Ed Reeder
#	oer	11/06/85
#               Added checks for EACH /dev comm device
#		Added support for multiple serial printers of the same type
#		(should be done in the configuration file before scpbuild.ss
#		gets control).
#	seckin	2/27/87
#		Modified everything related to comm boards, due to new 546/7/8
#		boards, new c188.c structures, and new CFPROPOSED structure.
#		Editing etc/default/dump,dumpdir,restor files modified, so that
#		we don't introduce leading spaces before the keywords.
#	seckin	3/19/87
#		Changed mv commands to cp commands, to preserve the uid/gid and
#		permissions for c188.c, xenixconf, and etc/default/* files.
#	seckin	3/24/87
#		Fixed ever-growing etc/local.rc (due to changes on 3/19).
#		Editing etc/default/dump,dumpdir,restor files modified, so that
#		even the old ones that have leading spaces get fixed (also, the
#		keyword in dumpdir file was 'tape', now it is 'archive').
#	seckin	7/14/87
#		For non-existant local.rc, got rid of error messages.
#
# scpbuild.ss_version[] ="@(#) scpbuild.ss $State: XENIX_R3-5_U2 $ $Date: 88/01/11 14:54:20 $";

# To test, set up required shell variables (use the current configuration
# as CFPROPOSED) and invoke scpbuild.ss directly
# Typical test scenario: (it dont work!)
#   SCPDEBUG=Y
#   SCPPATH=`pwd`
#   INITIAL=Y or N
#   INITIAL311=Y or N
#   ZAPLOG=/tmp/zaplog
#   INSTPROPOSED=/tmp/inst
#   CFPROPOSED=$SCPPATH/configurations/current
#   export SCPDEBUG SCPPATH INITIAL INITIAL311 ZAPLOG INSTPROPOSED CFPROPOSED
#   cp /sys/conf/xenixconf /tmp
#   cp /sys/conf/master    /tmp
#   cp /sys/cfg/c215g.c    /tmp
#   rm /tmp/*
#   scpbuild.ss
#   sh /tmp/inst
#   more /tmp/*

# Builds a shellscript that will later be executed to 'make' the configuration.
# At various points, prints a dot (.) to console to let user know
#   it is progressing.

# NOTE: INITIAL=Y could exist if a user respecifies his initial configuation.
#       Don't assume that your are starting with an "as built" system.

# Allow the 'sys' directory to be in / or /usr
# If both, /usr has precedence
[ -f /sys/conf/xenixconf ] && SYSDIR=/sys
[ -f /usr/sys/conf/xenixconf ] && SYSDIR=/usr/sys

# Allow for a "non-destructive" scp
ORIGIFS=$IFS    # to allow SCPDEBUG=N and rest of debug env. set

case "$SCPDEBUG" in
   Y)
      COMMENT="#DEBUG"
      SLASH=/tmp/
      ETC=/tmp
      SYSCFG=/tmp
      SYSCONF=/tmp
   ;;
   *)
      COMMENT=
      SLASH=/
      ETC=/etc
      SYSCFG=$SYSDIR/cfg
      SYSCONF=$SYSDIR/conf
esac
export COMMENT SLASH ETC SYSCFG SYSCONF

# Start with a null instruction file
cat <<-HERE > $INSTPROPOSED
	# `date` INSTRUCTIONS CREATED
	# If any command ends with a non-zero status, end instructions.
	set -e
	echo \`date\` START MAKING CONFIGURATION >> $ZAPLOG

HERE
##############################################################################
# Set pathnames
cat <<-HERE >> $INSTPROPOSED
	# Set pathnames and other variables
	COMMENT=$COMMENT
	SLASH=$SLASH
	ETC=$ETC
	SYSCFG=$SYSCFG
	SYSCONF=$SYSCONF
	SYSDIR=$SYSDIR
	export COMMENT SLASH ETC SYSCFG SYSCONF SYSDIR
HERE

##############################################################################
# Set up xenixconf:
# The assumption is that I will have some "canned" configurations
#   to use as a base and only a few items will need to be changed
#   to reflect the system being built.
#
cat <<-HERE >> $INSTPROPOSED
	echo \`date\` start xenixconf and c215g.c mod. >> $ZAPLOG
	echo -n .
	#
	# Setup parallel printer, timezone, and ulimit (max file size)
	sed -e  "
HERE
awk -F, '
   BEGIN {
      printf "." >> "/dev/tty"
   }
   /^TIMEZ/ {
      printf "/^timezone/c\\\\\ntimezone      (%s*60)\n", $4
      printf "/^daylight/c\\\\\ndaylight      %s\n", $3
   }
   END {
      printf "." >> "/dev/tty"
   }
' $CFPROPOSED >> $INSTPROPOSED
# Build the tail of the sed command
echo "\" < $SYSCONF/xenixconf > /tmp/xc.$$" >> $INSTPROPOSED
echo "cp /tmp/xc.$$ $SYSCONF/xenixconf" >> $INSTPROPOSED

# compute number of 512K units that can be allocated, ulimit
IFS=,
set `egrep \^MAXFILE $CFPROPOSED`
MAXSIZE=`expr $2 \* 2048`
IFS=$ORIGIFS

cat <<-HERE >> $INSTPROPOSED
	# Set the proper ulimit (max file size)
	if egrep '^[ 	]*ulimcfg' $SYSCONF/xenixconf > /dev/null ; then
	    sed -e "/ulimcfg/c\\\\
	ulimcfg       $MAXSIZE" < $SYSCONF/xenixconf > /tmp/xc.$$
	    cp /tmp/xc.$$ $SYSCONF/xenixconf
	else
	    echo "ulimcfg       $MAXSIZE" >> $SYSCONF/xenixconf
	fi
	sync
HERE

cat <<-HERE >> $INSTPROPOSED
	# Set up the c215g.c file, and ensure xenixconf swap size is correct
	# This is only needed for the INITIAL311 build, or if a device is added.
	case $INITIAL311 in
	    Y) (env - COMMENT=$COMMENT CFPROPOSED=$CFPROPOSED SYSDIR=$SYSDIR SYSCONF=$SYSCONF SYSCFG=$SYSCFG SCPPATH=$SCPPATH $SCPPATH/scpicp.ss )
	      ;;
	esac
	echo -n .
	echo \`date\` end xenixconf and c215g.c mod. >> $ZAPLOG
HERE

##############################################################################
# Set up /etc/local.rc:
# The assumption is that I will have some "canned" rc files
#   to use as a base and only a few items will need to be changed
#   to reflect the system being built.  If extra wini's exist their
#   mounts will added to the local.rc file.
echo -n .
cat <<-HERE >> $INSTPROPOSED
	echo \`date\` start /etc/local.rc mod. >> $ZAPLOG
	#
	# Create /etc/local.rc
	# Used to initialize the system when going into Multiuser mode.
	echo -n .
	set +e
	cp /etc/local.rc $ETC/local.rc.old >/dev/null 2>&1
	egrep "Directory /usr/tmp/\$file" $ETC/local.rc.old >/dev/null 2>&1
	if [ \$? != 0 ]; then
	    cp $SCPPATH/base/rc.base $ETC/local.rc
	else
	    > $ETC/local.rc
	fi
	sync
HERE
# See if the local.rc file needs to include more winis (do the mounts).
# Currently this is only for 311 systems that have a wini
# in the 311 box.  The standard 311 local.rc already includes support for the
# first wini in the 310 box (i.e., /usr file system).
set `wc $SCPPATH/base/rc.311.xtradsk`
lines=$1	# Number of lines in local.rc addition
awk -F, '
   BEGIN {
      printf "." >> "/dev/tty"
      FS=","
   }
   /^WINI1/ {
      print "# Add mount for the /usr2 file system to /etc/local.rc"
      print "egrep usr2 '$ETC'/local.rc.old >/dev/null 2>&1"
      print "if [ $? != 0 ]; then"
      print "cat <<HERE | ex '$ETC'/local.rc >/dev/null 2>&1"
      print "/^# <FIRST ADDITIONAL DISK MOUNT>"
      print "r '$SCPPATH'/base/rc.311wini1"
      print "w"
      print "q"
      print "HERE"
      print "fi"
      print "sync"
   }
   /^WINI[2-7]/ {
      disk = substr( $1, 5, 1 )		# Get disk number
      filesystem = disk + 1		# Make file system number
      print "# Add mount for the /usr"filesystem" file system to /etc/local.rc"
      print "egrep usr"filesystem" '$ETC'/local.rc.old >/dev/null 2>&1"
      print "if [ $? != 0 ]; then"
      print "cat <<HERE | ex '$ETC'/local.rc >/dev/null 2>&1"
      print "/^# EXTRA DISK"
      print "r '$SCPPATH'/base/rc.311.xtradsk"
      print "-'$lines'"
      print ".,+'$lines's/DISK#/"filesystem"/g"
      print "w"
      print "q"
      print "HERE"
      print "fi"
      print "sync"
   }
' $CFPROPOSED >> $INSTPROPOSED
cat <<-HERE >> $INSTPROPOSED
	cat /etc/local.rc.old >>$ETC/local.rc 2>/dev/null
	echo -n .
	echo \`date\` end /etc/local.rc mod. >> $ZAPLOG

HERE
echo -n .

##############################################################################
# Set up parallel printer and /etc/printcap
# The following cases exist:
# 1) Parallel printer
#       a) no serial printers
#       b) single serial printer
#       c) multiple serial printers
# 2) No Parallel printer
#       a) no serial printers
#       b) single serial printer
#       c) multiple serial printers

# The default printer will have a printcap id of 'lp'.  If a serial printer,
# it will also have its normal printcap entry, using /usr/spool/lpd as a
# spool directory and /usr/adm/lp.acct as the accounting file.  If the
# parallel printer is not the default, it will use the 'plp' printcap id,
# the /usr/spool/plpd spool directory, and the plp.acct accounting file.

# If multiple serial printers are of the same type, changes are made to the
# printcaps to force uniqueness.  The following example shows the changes
# made for the second printer.
#       FIRST                           SECOND
#       printcap id= mt440              printcap id= mt4402
#       sd=/usr/spool/mt440pd           sd=/usr/spool/mt440pd2
#       af=/usr/spool/mt440.acct        af=/usr/spool/mt4402.acct

# see if we need to configure lp driver into xenix.

MAXBOARDS=`egrep "^PPBOARD" $CFPROPOSED | wc -l`
BOARD=0
LP=N
IFS=,
while [ $BOARD -lt $MAXBOARDS ]
	do
	set `egrep PPBOARD$BOARD $CFPROPOSED`
	if [ "$2" = 310 -o "$2" = 350 ] ; then
		LP=Y
	fi
	BOARD=`expr $BOARD + 1`
done
IFS=$ORIGIFS
if [ $LP = Y ] ; then
cat <<-HERE >> $INSTPROPOSED
	# need regular lp driver
		$ETC/kdevs -s -p $SYSCONF lp 1

HERE
else
cat <<-HERE >> $INSTPROPOSED
	# no need for regular lp driver
		$ETC/kdevs -s -p $SYSCONF lp 0

HERE
fi

if [ -f $PRINTERFLAG ] ; then  #only if printer spec has changed
echo -n .
cat <<-HERE >> $INSTPROPOSED
	echo -n .
	echo \`date\` start printer, printcap setup >> $ZAPLOG

HERE
IFS=,
set `egrep \^DFLTPRINT $CFPROPOSED`
d1=$2           # Save search keys for default printer
d2=$3
case $d1 in
   "")  # No printer exists
      IFS=$ORIGIFS
      ;;
   *)
      set `egrep "^$d1,$d2" $CFPROPOSED`
      IFS=$ORIGIFS
      case $1 in
         PRINTER0)
            DEV=$2
            LABEL=$3
            BRAND=$5
            FILTER=$6
            serial=':\\\\' ;;
         TPORT*)
            DEV=$3
            LABEL=$5
            BAUD=$8
            shift; shift
            BRAND=$8
            FILTER=$9
            serial=':ic:br#'$BAUD':fs#1:\\\\' ;;
      esac

########## Initialize the printcap file #############
      cat <<HERE >>$INSTPROPOSED
      echo "\n
      Your previous printcap file has been saved in /etc/printcap.old
      If you had incorporated any changes to this file manually, you will need
      to apply those changes to the new printcap file" | tee -a $ZAPLOG
      cp /etc/printcap $ETC/printcap.old >/dev/null 2>&1
      echo -n "# " > $ETC/printcap
      what $SCPPATH/scpbuild.ss | grep XENIX >> $ETC/printcap
HERE

      # BEGIN ECHO
      echo "#
# Build /etc/printcap
# Report printer names to the user
# Ensure spool directories are properly set up (assume /usr/spool/lpd is O.K.)
#
# Report the printer names to the user
echo
echo The printer\(s\) on your system are known by the following names:
echo
echo \"$BRAND, Device $DEV, is known as:  lp\"

cat <<HERE >> $ETC/printcap
# Printcap for the default printer
lp| lineprinter| line printer| $BRAND:\\\\\\
        $serial
        :lp=$DEV:sd=/usr/spool/lpd:\\\\\\
        :bd=/bin:af=/usr/adm/lp.acct:\\\\\\
	:ofs#1:\\\\\\
        :if=$FILTER:\\\\\\
        :of=$FILTER:

HERE" >> $INSTPROPOSED
      # END ECHO

      awk -F, '
	BEGIN {
		printeryes = "no"
		}
         # Look at parallel printer
         /^PRINTER/ {
            # See if a printcap id exists (i.e., does printer exist?)
            if( $5 != "none" ) {
               print "# Make /dev/lp if needed"
               print "( cd /dev"
               print "'$COMMENT' [ -c lp ] || make lp > /dev/null 2>&1"
               print ")"
               # Build parallel printer^s printcap entry if needed.
               # If default printer is this one, we are done
               if( "'$d1'" != "PRINTER0" ) {
                  print "cat <<HERE >> $ETC/printcap"
                  print "# Parallel printer"
                  print "plp| " $5 ":\\\\"
                  print "       :lp=" $2 ":sd=/usr/spool/plpd:\\\\"
                  print "       :bd=/bin:af=/usr/adm/plp.acct:\\\\"
                  print "       :ofs#1:\\\\"
                  print "       :if=" $6 ":\\\\"
                  print "       :of=" $6 ":"
                  print ""
                  print "HERE"
                  print "echo \"" $5 ", Device " $2 ", is known as:  plp\""
                  printeryes = "yes"
                  spooldir["/usr/spool/plpd"] = "/usr/spool/plpd"       # record this one
               }
            }
            next
         }
         # See if any serial printers exist
         /^TPORT/ {
            if( $6 == "P" ) {
               # Build this serial printer^s printcap entry.
               # See if this is the first serial printer of this type.
               # If this one is the default, its spool is /usr/spool/lpd.
               serialp[$9] += "1"
               if( serialp[$9] == "1" )
                  suffix = ""
               else
                  suffix = serialp[$9]

               if( "'$d1'" == $1 && "'$d2'" == $2 ) {
                  spool = "/usr/spool/lpd"
                  acctfile = "lp"
               } else {
                  spool = $12 suffix
                  acctfile = $9 suffix
                  printeryes = "yes"
                  spooldir[spool] = spool
               }
               print "cat <<HERE >> $ETC/printcap"
               print $9 suffix "| " $10 ":\\\\"
               print "  :ic:br#" $8 ":fs#1:\\\\"
               print "  :lp=" $3 ":sd=" spool ":\\\\"
               print "  :bd=/bin:af=/usr/adm/" acctfile ".acct:\\\\"
               print "  :ofs#1:\\\\"
               print "  :if=" $11 ":\\\\"
               print "  :of=" $11 ":"
               print ""
               print "HERE"
               print "echo \"" $10 ", Device " $3 ", is known as:  " $9 suffix "\""
            }
         }
         END {
            print "#"
            print "# Ensure spool directories are set up"
            if ( printeryes == "yes" ) {
            for( directory in spooldir ) {
               print "if [ ! -d " directory " ] ; then"
               print ": so debug will work"
               print "'$COMMENT'   mkdir " directory
               print "'$COMMENT'   echo 0 > " directory "/.seq"
               print "'$COMMENT'   chown bin " directory " " directory "/.seq"
               print "'$COMMENT'   chgrp bin " directory " " directory "/.seq"
               print "'$COMMENT'   chmod 777 " directory
               print "'$COMMENT'   chmod 666 " directory "/.seq"
               print "fi"
            }
           }
         }
      ' $CFPROPOSED >> $INSTPROPOSED
      cat <<-HERE >> $INSTPROPOSED
	echo
	echo These names are used with the lpdrestart, lpq, lpr, and lprm commands.
	echo The names are kept in the file /etc/printcap
	echo "\nOne moment, please . . ."
	echo \`date\` make the print filters as required >>$ZAPLOG
	cd /usr/lib/pf; make filters >>$ZAPLOG 2>&1
	echo \`date\` end printer, printcap  setup >> $ZAPLOG

HERE
      ;;
esac
fi                 # end of PRINTERFLAG conditional code
echo -n .

##############################################################################
# Make certain $SYSCFG files are correct for the terminal comm. boards
# Note that we check only one type of board (either 188/546/547 or 544)

cat <<-HERE >> $INSTPROPOSED
	echo \`date\` start cfg comm board check >> $ZAPLOG
	#
	# Setup communication board entries in xenixconf
HERE
# If vpm is configured with "pasync" set in xenixconf,
# then all 188 boards are controlled by the vpm driver,
# i.e. for our purposes those 188s do not exist.
set `egrep '^pasync' $SYSCONF/xenixconf` 0 0
vpm_async=$2
#
# Go through CFPROPOSED and check term. boards
BOARD=none
Bcount=0
n544=0
n188=0
n54x=0
ints=""		# for i188 interrupt vector entries
> /tmp/Scfg$$	# for i188cfg initialization list
echo -n .
IFS=,
exec < $CFPROPOSED
while read type board x x base port int; do
    case "$type" in
        TERMBOARD*)
	    scfg=N
	    case $board in
		none)	;;
		544)	n544=`expr $n544 + 1` ;;
		188)	if [ $vpm_async = 0 ] ; then
			    n188=`expr $n188 + 1`
			    scfg=Y
			fi ;;
		54?)	n54x=`expr $n54x + 1`
			scfg=Y ;;
	    esac
	    if [ $scfg = Y ] ; then
		ints="$ints $int"
		x=`expr $n188 + $n54x`
		echo "	$base, $port, $int,	/* board $x */\\\\\\\\" >> /tmp/Scfg$$
	    fi
	    ;;
	*)  continue
    esac
done
IFS=$ORIGIFS
exec < /dev/tty
echo -n .
#
n188=`expr $n188 + $n54x`	# all 188 types for xenix
#
if [ $n188 -eq 0 ] ; then
    echo "$ETC/kdevs -s -p $SYSCONF i188 0" >> $INSTPROPOSED
    if [ $n544 -eq 0 ] ; then
	echo "$ETC/kdevs -s -p $SYSCONF i544 0" >> $INSTPROPOSED
    else
	echo -n .
	BOARD=544
	Bcount=$n544
	cat <<-HERE >> $INSTPROPOSED
		$ETC/kvectors i544 3 >/dev/null 2>&1
		$ETC/kdevs -s -p $SYSCONF i544 1
HERE
    fi
else
    echo -n .
    BOARD=188
    Bcount=$n188
    cat <<-HERE >> $INSTPROPOSED
	$ETC/kvectors i188 $ints >/dev/null 2>&1
	$ETC/kdevs -s -p $SYSCONF i188 1
	$ETC/kdevs -s -p $SYSCONF i544 0
	# we need to redefine i188cfg in cfg/c188.c
	# cannot check if whole definition was ok
	echo "Y" > $MAKEFLAG
	( cd $SYSCFG
	  cp c188.c c188.c.old
	  echo Previous cfg/c188.c saved as cfg/c188.c.old >> $ZAPLOG
	  sed -e "
	/^\([ 	]*#define[ 	]*NUM188\)/c\\\\
	#define NUM188 $Bcount
	/[ 	]*struct[ 	]*i188cfg[ 	]*i188cfg/,/.*};/c\\\\
	struct	i188cfg i188cfg[] = {\\\\
HERE
    cat /tmp/Scfg$$ >> $INSTPROPOSED
    # build the tail of the sed command
    cat <<-HERE >> $INSTPROPOSED
	};
	" < c188.c > /tmp/Bcfg$$
	  cp /tmp/Bcfg$$ c188.c
	  echo "\nMaking lib_ioc."
	 $COMMENT rm -f c215g.o
	 $COMMENT touch lib_ioc
	  sync
	 $COMMENT sleep 3
	 $COMMENT touch c215g.c
	  sync
	 $COMMENT make | tee -a /dev/console | cat >> $ZAPLOG
	)
	echo -n .

HERE
fi
#
# Note that 188 type boards are handled above
if [ $BOARD != "none" -a $BOARD != "188" ] ; then {
IFS=,
set `egrep "^$BOARD" $SCPPATH/devices/sys.cfg.files`
file=$2
defarg=$3
IFS=$ORIGIFS

cat <<-HERE >> $INSTPROPOSED
	echo -n .
	#
	# ensure we have exactly $Bcount board(s) specified
	set \`egrep "^[ 	]*#define[ 	]*$defarg" $SYSCFG/$file\`
	case "\$3" in
	   $Bcount) ;;  # correct number
	   *)
	      echo "Y" > $MAKEFLAG               # make new xenix
	      ( cd $SYSCFG
	        cp $file $file.old
	        echo Previous cfg/$file saved as cfg/$file.old >> $ZAPLOG
	        sed -e  "
	/^\([ 	]*#define[ 	]*$defarg\)/c\\\\
	#define $defarg  $Bcount
	" < $SYSCFG/$file > /tmp/Bcfg$$
	        cp /tmp/Bcfg$$ $file
	        echo "\nMaking lib_ioc."
	       $COMMENT rm -f c215g.o
	       $COMMENT touch lib_ioc
	        sync
	       $COMMENT sleep 3
	       $COMMENT touch c215g.c
	        sync
	       $COMMENT make | tee -a /dev/console | cat >> $ZAPLOG
	      )
	esac
	echo -n .

HERE
}
fi
echo -n .
echo "echo \`date\` end cfg comm board check >> $ZAPLOG\n" >> $INSTPROPOSED

##############################################################################
# Set up /etc/checklist
cat <<-HERE >> $INSTPROPOSED
	echo \`date\` start /etc/checklist mod. >> $ZAPLOG
	echo -n .
	#
	# Setup /etc/checklist, used by fsck for default file systems to check
HERE
awk -F, '
   BEGIN {
      printf "." >> "/dev/tty"
      print "cat > $ETC/checklist <<HERE"
      print "/dev/root"
   }
   # The following assumes that additional winis are formatted into single
   #   file systems: /usr2, /usr3, ... /usrN
   /^WINI/ {
      if( $1 == "WINI0" )
         if( $3 == "15M") next
            else { print "/dev/usr"
               next
         }
      filesystem = substr( $1, 5, 1 ) + 1
      print "/dev/usr" filesystem
   }
   END {
      print "HERE"
      printf "." >> "/dev/tty"
      print "echo -n ."
   }
' $CFPROPOSED >> $INSTPROPOSED
echo "echo \`date\` end /etc/checklist mod. >> $ZAPLOG\n" >> $INSTPROPOSED

##############################################################################
# Set up /etc/default files
# Use tape as default, if avail, else use floppy.
echo -n .
if egrep \^TAPE0 $CFPROPOSED > /dev/null ; then
   IFS=,
   set `egrep \^TAPE0 $CFPROPOSED`      # get tape characteristics
   IFS=$ORIGIFS
   devname=$2
   tapesize=$5
   density=$6
   kbytes=0
else
   IFS=,
   set `egrep \^FLOPPY0 $CFPROPOSED`    # get floppy characteristics
   IFS=$ORIGIFS
   devname=$2
   tapesize=0
   density=0
   kbytes=`echo $3 | tr K ""`   # remove the trailing K
fi
cat <<-HERE >> $INSTPROPOSED
	echo \`date\` start /etc/default mod. >> $ZAPLOG
	echo -n .
	#
	# Setup /etc/default files, used by dump, dumpdir, restor
	sed "
	/^[ 	]*archive=/c\\\\
	archive=$devname
	/^[ 	]*kbytes=/c\\\\
	kbytes=$kbytes
	/^[ 	]*tapesize=/c\\\\
	tapesize=$tapesize
	/^[ 	]*density=/c\\\\
	density=$density
	" < /etc/default/dump > /tmp/default$$
	cp /tmp/default$$ $ETC/default/dump

	sed "
	/^[ 	]*archive=/c\\\\
	archive=$devname
	" < /etc/default/dumpdir > /tmp/default$$
	cp /tmp/default$$ $ETC/default/dumpdir

	sed "
	/^[ 	]*archive=/c\\\\
	archive=$devname
	" < /etc/default/restor > /tmp/default$$
	cp /tmp/default$$ $ETC/default/restor
	echo -n .
	echo \`date\` end /etc/default mod. >> $ZAPLOG

HERE
echo -n .

##############################################################################
# For Initial configuration, or respecification of a 311 config.,
# format the 311 Winchester expansion disk(s), if any.
# Make the file systems on them.
#
# NOTE: THIS OPERATION CAN NOT BE DONE UNTIL THE KERNEL IS CONFIGURED TO
#       ACCESS THE ADDITIONAL DRIVES.  AS A RESULT, THIS HAS TO BE
#       PERFORMED AS A SEPARATE ACTION, AFTER THE SYSTEM HAS BEEN
#       REBOOTED.
# The scp will create a file $ETC/scpinitialfmt to perform this action.
# Before the system is shutdown, scp will determine if $ETC/scpinitialfmt
# exists.  If so, scp will inform the user what actions are needed.  Once
# scpinitialfmt has run, it will be removed.
echo -n .
env - CFPROPOSED=$CFPROPOSED ZAPLOG=$ZAPLOG ETC=$ETC SCPPATH=$SCPPATH $SCPPATH/scpbuild.1.ss
                        # make a separate script to bypass shell problem
echo -n .
echo -n .

##############################################################################
# Rebuild kernel if necessary, save a working version of the kernel
# Exit with a status that tells whether a new kernel was made or not,
#    this will determine whether to /etc/shutdown or not.

cat <<-HERE >> $INSTPROPOSED
	echo \`date\` start kernel check >> $ZAPLOG
	# Rebuild kernel if needed
	# Save working kernel as /xenix.old
	if [ -f $MAKEFLAG ] ; then
	    ( cd $SYSCONF
	      echo "\nMaking new xenix"
	     $COMMENT touch c.c
	      sync
	     $COMMENT sleep 3
	     $COMMENT touch xenixconf
	     $COMMENT rm -f c.o
	     $COMMENT rm -f xenix
	      sync
	      $COMMENT(  make xenix ; if [ \$? -eq 0 ] ; then echo "y" >$MAKESTAT; fi ) | tee -a /dev/console | cat >> $ZAPLOG
	      if [ -f $MAKESTAT ] ; then
	          mv /xenix ${SLASH}xenix.old
	          echo "\nPrevious operating system kernel saved as ${SLASH}xenix.old"
	         $COMMENT mv xenix /
	         $COMMENT chown sysinfo /xenix
	         $COMMENT chmod 600 /xenix
		  sed -e "/^KERNEL BUILT/d" <$CFPROPOSED >$CFPROPOSED\2
		  mv $CFPROPOSED\2 $CFPROPOSED
		  echo "KERNEL BUILT NORMAL `date` using SCP Version $VERSION" >> $CFPROPOSED
	      fi
	     $COMMENT rm -f c.c
	      echo \`date\` MADE NEW KERNEL >> $ZAPLOG
	    )
	fi
	echo \`date\` end kernel check >> $ZAPLOG

	echo
	echo \`date\` END MAKING CONFIGURATION >> $ZAPLOG

	# Signal whether new kernel made or not
	if [ -f $MAKEFLAG -a -f $MAKESTAT ] ; then
	    exit 14
	else
	    exit 0
	fi
HERE

echo
