: /bin/sh
# Copyright Intel Corp. 1985
# Author:  Ed Reeder
# scplpdmod.ss_version[] ="@(#) scplpdmod.ss $State: XENIX_R3-5 $ $Date: 87/02/24 18:58:12 $";
# Determine default printer
# (/etc/printcap lp entry)
#       oer       11/07/85 Fix problem with default serial printer

# Cases are ($1 equals ):
# -Make
#   Implied change, prior to making system, if during session a printer
#   was changed and $PRINTERFLAG != 0
echo "`date` started default printer specification" >> $ZAPLOG
# -Modify
#   User desires to change default explicitly.  Sets $PRINTERFLAG to 0

# There will always be a DFLTPRINT entry, even if no printer exists.

CASE=$1
while : ; do 
    set `$SCPPATH/scplplist.ss $CFPROPOSED | wc -l`
    if [ $1 -lt 2 ] ; then
        break
    fi
    IFS=,
    set `egrep "^DFLTPRINT" $CFPROPOSED`
    case $2 in
        PRINTER0)
            set `egrep "^$2,$3" $CFPROPOSED`
            if [ $5 = "none" ] ; then
                break
            fi
            DEVICE=$2
            BRAND=$5
            TYPE=Parallel
            ;;
        TPORT*)
            set `egrep "^$2,$3" $CFPROPOSED`
            if [ $6 != "P" ] ; then
                break
            fi
            DEVICE=$3
            shift
            BRAND=$9
            TYPE=Serial
            ;;
        *)  break ;;
    esac
    while : ; do
        echo "\nDefault printer specification:"
        echo "$TYPE printer, Device $DEVICE, Brand is $BRAND"
        echo -n 'Is this correct <y/n>? '
        read xreply extra
        case $xreply in 
            y*|Y*) echo "`date` ended default printer specification without change" >> $ZAPLOG
                   > $PRINTERFLAG
                   exit $OK
                   ;;
            n*|N*) break 2 ;;
            *)     echo "Invalid response <$xreply>, please respecify\n" ;;
        esac
    done
done


while : ; do
   echo "One moment, please ..."
   IFS='
'                            # Newline

   # Get configured printers in numbered vars
   #    (provide a null parm, even if 0 exist)
   set `$SCPPATH/scplplist.ss $CFPROPOSED` ""
   IFS=$ORIGIFS

   case $1 in
      "")       # No printers are currently configured
         case $CASE in
            -Modify)
               echo '* * * No printers are currently configured for your system'
	       echo "* * Press RETURN to continue * *"
               read reply
            ;;
         esac
         # All cases
         sed "/^DFLTPRINT/c\\
DFLTPRINT,," < $CFPROPOSED > $CFPROPOSED\2
         mv $CFPROPOSED\2 $CFPROPOSED
         exit $OK
      ;;
   esac

   case $2 in
      "")       # Only one printer, it will be the default
         IFS=,
         set $1
         IFS=$ORIGIFS
         key1=$1  key2=$2
         case $1 in
            PRINTER0)
               DEVICE=$2
               LABEL=$3
               BRAND=$5
               TYPE=Parallel
            ;;
            TPORT*)
               DEVICE=$3
               LABEL=$5
               shift
               BRAND=$9
               TYPE=Serial
            ;;
         esac
         case $CASE in
            -Modify)
               echo '* * * Only a single printer exists.'
               echo $TYPE printer, Device $DEVICE, Brand is
               echo "        $BRAND"
               echo "This will become the default"
               echo '* * Press RETURN to continue * *'
               read reply
            ;;
         esac
         # All cases
         sed "/^DFLTPRINT/c\\
DFLTPRINT,$key1,$key2" < $CFPROPOSED > $CFPROPOSED\2
         mv $CFPROPOSED\2 $CFPROPOSED
         > $PRINTERFLAG                     # Indicate default is correct
         exit $OK
      ;;
   esac

   # Have multiple printers, choose the default
   # Put info. into the named variables a, b, ... i
   for prt in a b c d e f g h i ; do
      case $1 in
        "") break ;;
        *)
           eval $prt=\"$1\"
           shift ;;
      esac
   done

   # Print out a menu of currently configured printers
   while : ;  do
      clear
      cat <<-HERE
-----------------------------------------------------------------(Defaultprint)
                        Set default printer
   (all printing will go to the default printer, unless directed otherwise)

HERE
      for prt in a b c d e f g h i ; do
         eval x=\$$prt
         case $x in
            "") break ;;
         esac
         last=$prt
         IFS=,
         set $x
         IFS=$ORIGIFS
         case $1 in
            PRINTER0)
               DEVICE=$2
               LABEL=$3
               BRAND=$5
               TYPE=Parallel
            ;;
            TPORT*)
               DEVICE=$3
               LABEL=$5
               shift
               BRAND=$9
               TYPE=Serial
            ;;
         esac
         echo "        $prt   $TYPE printer, Device $DEVICE, Brand is"
         echo "               $BRAND"
      done
      cat <<-HERE

        q   Quit to main menu (and lose changes)
        ?   Get an explanation of these options

HERE
      echo -n "Enter selection? "
      read reply extra
      case $reply in
         q|Q)
            exit $QUIT
            ;;
         \?|help|Help)
            clear
            more -c $SCPPATH/scplpdf.0.hlp
            echo '* * Press RETURN to continue * *'
            read reply
            continue
         ;;
      esac
      origreply=$reply
      case $reply in              # Don't allow *
         *\**)  reply=""
      esac
      # only allow numbers, letters, (expr is not very forgiving)
      reply=`echo $reply | tr -d "[\001-\057][\072-\100][\133-\140][\173-\177]"`
      reply=`echo $reply | tr '[A-Z]' '[a-z]'`       # make lower case
      # Is reply 1 char long, and is it one of those listed?
      if [ `expr "$reply" : '.*'` = 1 -a `expr "$reply" : "[a-$last]"` = 1 ] ; then
         while : ; do
            echo "\nDefault printer specification:"
            eval x=\$$reply
            IFS=,
            set $x
            IFS=$ORIGIFS
            key1=$1 key2=$2
            case $1 in
               PRINTER0)
                  DEVICE=$2
                  LABEL=$3
                  BRAND=$5
                  TYPE=Parallel
               ;;
               TPORT*)
                  DEVICE=$3
                  LABEL=$5
                  shift
                  BRAND=$9
                  TYPE=Serial
               ;;
            esac
            echo "$TYPE printer, Device $DEVICE, Brand is $BRAND"
            echo -n 'Is this correct <y/n>? '
            read xreply extra
            case $xreply in
               y*|Y*) break 3 ;;
               n*|N*) break 2 ;;
               *) echo "Invalid response <$xreply>, please respecify\n" ;;
            esac
         done
      else
         echo "Invalid response <$origreply>, Press RETURN to continue"
         read reply
         continue
      fi
   done
done

sed "/^DFLTPRINT/c\\
DFLTPRINT,$key1,$key2" < $CFPROPOSED > $CFPROPOSED\2
mv $CFPROPOSED\2 $CFPROPOSED
> $PRINTERFLAG
echo "`date` ended default printer specification" >> $ZAPLOG
exit $OK
