# Copyright Intel Corp. 1985,1986
# Author:  Ed Reeder
#
# Modified:  Colin Hulme, Intel Corporation
#    4-17-86 to work with Xenix 3.4
#    5-02-86 to handle multiple communications boards
#    5-03-86 split out console portion to allow early detection of console type
#    2-16-87 br
#            modified to take advantage of info from gs (from spcic.ss) and
#            to query user on a board basis (prevents screen overflow, too).
#    3-04-87 seckin
#	     fix for PPR 3223: update printcap on printer baud rate changes.
#    3-19-87 seckin
#            count the number of enabled ports, and if more than 16, then force
#            the user back into port configuration for disabling some of them.
#    3-23-87 seckin
#            edited default terminal type message.
#
# scpport.ss_version[] ="@(#) scpport.ss $State: XENIX_R3-5 $ $Date: 87/06/11 14:16:33 $";

# TO DO:
#   Make sure help for dial-in modem clarifies 'uucp' usage
#   (e.g., specify a terminal anyway)

echo "`date` started ports " >> $ZAPLOG
# Use a temporary config. file, until user O.K.'s it
CFPROPOTMP=$SCPPATH/configurations/cf.propo.tmp

# For an existing system, use the current setting
cp $CFPROPOSED $CFPROPOTMP

MAXBOARDS=`egrep "^TERMBOARD" $CFPROPOSED | wc -l`
BOARD=0
IFS=,
while [ $BOARD -lt $MAXBOARDS ]
	do
	set `egrep "^TERMBOARD$BOARD" $CFPROPOTMP`
	eval TERMBOARD$BOARD=$2
	BOARD=`expr $BOARD + 1`
done
IFS=$ORIGIFS

# Review ALL comm. board ports
built=N		# If INITIAL = Y, indicate default terminal list not built
   # Construct default terminal list
   #   For an initial system, set all ports as unknown terminal at 9600 baud
   #   For an existing system, use existing configuration as base
   if [ -f $SWITCHFLAG -o $built = Y -o $INITIAL = N ] ; then
      echo -n ""		# do nothing
   else
      while :
	do
	clear
	echo "You may specify a default terminal type if you wish
(e.g., to identify your site's most common type of terminals).\n"
	echo -n "Do you wish to specify a default terminal type <y/n>? "
	PRINTTERMCAP=unknown
	TYPE=unknown
	read reply extra
	case "$reply" in
		n*|N*)	break ;;
		y*|Y*)
		MUSTANSWER=N $SCPPATH/scpterms.ss
		case "$?" in
			$OK)	IFS=,
				set `cat < $SCPPATH/configurations/term.tmp`
				TYPE=$1
				PRINTTERMCAP=$2
				IFS=$ORIGIFS
				break ;;
			$RESTART) continue ;;
			$QUIT)	break ;;
		esac
	esac
      done
      BOARD=0
      while [ $BOARD -lt $MAXBOARDS ]
         do
	 IFS=,
	 set `egrep "^TERMBOARD$BOARD" $CFPROPOTMP`
         IFS=$ORIGIFS
	 LINES=$4
	 eval temp=\$TERMBOARD$BOARD
	 if [ $temp -eq 546 ] ; then
		LINES=4
	 fi
	 case $SYSTYPE in
		310) SYSFIELD=3
		     if [ $LINES -gt 8 ] ; then
			# uses an io panel like AP's
			SYSFIELD=4
		     fi 
		     ;;
		AP) SYSFIELD=4 ;;
		320) SYSFIELD=5 ;;
	 esac
	 if [ -z "$LINES" ] ; then
		 awk -F, "{ \
		 printf \"TPORT$BOARD,%x,/dev/%s,%s, %s,T,E,9600,%s,%s\\n\", \
			device++, \$2, \$1, "\$$SYSFIELD",\
			\"$PRINTTERMCAP\", \"$TYPE\"\
		 }"  $SCPPATH/devices/p.$temp.$BOARD >> $CFPROPOTMP
	 else
		 awk -F, "{ \
		 while (device < $LINES){ \
		 printf \"TPORT$BOARD,%x,/dev/%s,%s, %s,T,E,9600,%s,%s\\n\", \
			device++, \$2, \$1, "\$$SYSFIELD",\
			\"$PRINTTERMCAP\", \"$TYPE\";\
		 next };
		 }"  $SCPPATH/devices/p.$temp.$BOARD >> $CFPROPOTMP

	fi
	BOARD=`expr $BOARD + 1`
      done
      built=Y
   fi

# BEGIN Ports
while :
 do
 BOARD=0

# BEGIN One pass over all ports
 while [ $BOARD -lt $MAXBOARDS ]
   do
   if [ -z "$TERMBOARD$BOARD" ] ; then
	continue
   fi
   clear
   cat  <<-HERE
----------------------------------------------------(Serial Controller #$BOARD)
	        PORT Label Baud    Usage          Type
HERE

   awk -F, '
       /^'TPORT$BOARD'/{if($6 == "I") { # make dial-in modem less confusing
                   usage = "Dial-in modem"}
                else note = ""
       if($6 == "U") usage = "Unused"; if($6 == "P") usage = "Printer"
       if($6 == "T") usage = "Terminal"
       if($6 == "M") usage = "Dial-out modem";if($6 == "O") usage = "Other output"
       printf "        %2s  %-6s %-5s   %-14s %-s  %s\n",$4,$5,$8,usage,$10,note}
   ' $CFPROPOTMP
       cat <<-HERE

	        y)   Accept these ports as shown above,
	             and continue with next set of ports
	        p)   Accept serial configuration and
	             proceed with system configuration
	        r)   Restart port configuration

	        q)   Quit to main menu (and lose changes)
	        ?)   Get an explanation of these options
HERE
        echo -n "Enter selection? "
   read reply extra
   case "$reply" in
      # Accept as is -- get next board set
      y*|Y*)
	BOARD=`expr $BOARD + 1`
        if [ $BOARD -lt $MAXBOARDS ] ; then
        continue
	fi
	break
        ;;

      # done with serial
      p*|P*)
	break
	;;

      # restart
      r*|R*)
	echo -n 'Are you certain you wish to restart port configuration <y/n>? '
        read reply
        [ "$reply" = "y" -o "$reply" = "Y" ] && exit $RESTART
        continue
        ;;

      # quit
      q*|Q*)
        exit $QUIT
	;;

      # help
      \?|help|Help)
	  clear
	  more -c $SCPPATH/scpport.0.hlp
          echo '* * Press RETURN to continue * *'
          read reply
          continue
	  ;;

      # all actions checked, see if it is a port
      *)
          origreply=$reply
          case $reply in		# Don't allow *
             *\**)  reply=""
          esac
          # only allow numbers, letters, and ?
          reply=`echo $reply | tr -d "[\001-\057][\072-\076]\100[\133-\140][\173-\177]"`
          reply=`echo $reply | tr '[A-Z]' '[a-z]'`   # make lower case
	     temp=\$TERMBOARD$BOARD
	     eval temp1=$temp
             egrep "^$reply," $SCPPATH/devices/p.$temp1.$BOARD > /dev/null
	     if [ $? -ne 0 ] ; then
		     echo "Invalid response <$origreply>, Press RETURN to continue\n"
		     read reply
		     continue
	     fi
             # got a legal port, parse its values.
             echo 'One moment, please ...'
             IFS=,
             set `egrep "^TPORT$BOARD,[0-9a-f]*,[^,]*,$reply," $CFPROPOTMP`
             DEV=$3 PORT=$4 LABEL=$5 USAGE=$6 ENABLED=$7 BAUD=$8 PRINTTERMCAP=$9
             shift; TYPE=$9
             IFS=$ORIGIFS
	  ;;
   esac

   # Review a single port
   while :  # BEGIN Port
      do
      clear
      # if a system generated reply exists, use it
      # These are used to anticipate what the user will need to specify next
      if [ -z "$sysreply" ] ; then
         cat  <<-HERE
========================================================================(Port)
	      Specify this communication port's configuration

HERE
         echo $PORT,$LABEL,$BAUD,$USAGE,$TYPE | \
         awk -F, 'BEGIN {printf "             PORT Label Baud    Usage          Type\n"}
         {if($4 == "I") { # make dial-n modem a little less confusing
            usage = "Dial-in modem"}
            else note = ""
         if($4 == "U") usage = "Unused"; if($4 == "P") usage = "Printer"
         if($4 == "T") usage = "Terminal"
         if($4 == "M") usage = "Dial-out modem";if($4 == "O")usage="Other output"
         printf "        FOR  %2s  %-6s %-5s   %-14s %-s  %s\n",$1,$2,$3,usage,$5,note}'
         cat  <<-HERE

        a)      Change usage to terminal
        b)      Change usage to printer
        c)      Change usage to Unused
        d)      Change usage to modem
        e)      Change usage to Other output
        f)      Change terminal or printer type
        g)      Change baud rate/speed

        q)      Quit to main menu (and lose changes)
        ?)      Get an explanation of these options

        y)      Accept this port as shown and re-examine all ports

HERE
         echo -n "Enter selection? "
         read reply extra
      else
         reply=$sysreply
         sysreply=
      fi
      case "$reply" in
         # Change to terminal use
         # (set default baud rate to 9600)
         a|A)
	   echo Y > $PRINTERFLAG	# Indicate printer info might change
           defaultbaud=`cat $SCPPATH/base/default.t.baud`
           sed "s|^\(TPORT$BOARD,[0-9a-f],$DEV,$PORT,$LABEL,\)$USAGE.*|\1T,E,$defaultbaud,,|" < $CFPROPOTMP > $CFPROPOTMP\2
           mv $CFPROPOTMP\2 $CFPROPOTMP
           USAGE=T
           ENABLED=E
           BAUD=$defaultbaud
           PRINTTERMCAP=
           TYPE=
           sysreply=f		# enable a prompt for terminal name
           continue
           ;;


         # Change to printer use
         b|B)
	   echo Y > $PRINTERFLAG	# Indicate printer info might change
           sed "s|^\(TPORT$BOARD,[0-9a-f],$DEV,$PORT,$LABEL,\)$USAGE.*|\1P,D,,,|"  < $CFPROPOTMP > $CFPROPOTMP\2
           mv $CFPROPOTMP\2 $CFPROPOTMP
           USAGE=P
           ENABLED=D
           BAUD=
           PRINTTERMCAP=
           TYPE=
           sysreply=f		# enable a prompt for printer name
           continue
           ;;


         # Change to Unused status
         c|C)
	   echo Y > $PRINTERFLAG	# Indicate printer info might change
           sed "s|^\(TPORT$BOARD,[0-9a-f],$DEV,$PORT,$LABEL,\)$USAGE.*|\1U,D,,,|" < $CFPROPOTMP > $CFPROPOTMP\2
           mv $CFPROPOTMP\2 $CFPROPOTMP
           USAGE=U
           ENABLED=D
           BAUD=
           PRINTTERMCAP=
           TYPE=
           continue
           ;;


         # Change to modem use
         d|D)
	   echo Y > $PRINTERFLAG	# Indicate printer info might change
           defaultbaud=`cat $SCPPATH/base/default.m.baud`
           while :
              do
	      clear
              cat <<-HERE
------------------------------------------------------------------------(Modem)
	                 Specify modem use

        a)      Dial-out use
        b)      Dial-in use  (will be associated with a terminal type)
        c)      Short-haul modem

        q)      Quit to main menu (and lose changes)
        ?)      Get an explanation of these options

HERE
              echo -n "Enter selection? "
              read reply extra
              case "$reply" in
                 # quit
                 q*|Q*)
                   exit $QUIT
	           ;;

                 # help
                 \?|help|Help)
		   clear
	           more -c $SCPPATH/scpmodem.0.hlp
                   echo '* * Press RETURN to continue * *'
                   read reply
	           ;;

                 a|A)
                   sed "s|^\(TPORT$BOARD,[0-9a-f],$DEV,$PORT,$LABEL,\)$USAGE.*|\1M,D,$defaultbaud,,-----|" < $CFPROPOTMP > $CFPROPOTMP\2
                   mv $CFPROPOTMP\2 $CFPROPOTMP
                   USAGE=M
                   ENABLED=D
                   BAUD=$defaultbaud
                   PRINTTERMCAP=
                   TYPE="-----"
                   break
                   ;;

                 b|B)
                   sed "s|^\(TPORT$BOARD,[0-9a-f],$DEV,$PORT,$LABEL,\)$USAGE.*|\1I,E,$defaultbaud,,|" < $CFPROPOTMP > $CFPROPOTMP\2
                   mv $CFPROPOTMP\2 $CFPROPOTMP
                   USAGE=I
                   ENABLED=E
                   BAUD=$defaultbaud
                   PRINTTERMCAP=
                   sysreply=f		# prompt for terminal name
                   break
                   ;;

                 c|C)
echo "A short-haul modem is treated as a standard usage of port, i.e., do not
consider it as a modem. Instead configure this port as if no modem existed.
* * Press RETURN to continue * *"
                  read reply
                  break
                  ;;

                 *)
                   echo Invalid response \<$reply\>, Press RETURN to continue
		   read reply
	           ;;
              esac
           done
           ;;


         # Change to Other output use (e.g., plotter)
         e|E)
	   echo Y > $PRINTERFLAG	# Indicate printer info might change
           sed "s|^\(TPORT$BOARD,[0-9a-f],$DEV,$PORT,$LABEL,\)$USAGE.*|\1O,D,,,-----|" < $CFPROPOTMP > $CFPROPOTMP\2
           mv $CFPROPOTMP\2 $CFPROPOTMP
           USAGE=O
           ENABLED=D
           BAUD=
           PRINTTERMCAP=
           TYPE="-----"
           sysreply=g		# prompt for baud rate
           continue
           ;;


         # Change terminal or printer type
         f|F)
	   echo Y > $PRINTERFLAG	# Indicate printer info might change
           # Change terminal type
           if [ $USAGE =  T -o $USAGE = I ] ; then
              case $USAGE in
                 I) NOTE='(for dial-in modem)' ;;
                 *) NOTE=
              esac
		PRINTTERMCAP=unknown
		TYPE=unknown
		while :
		  do
			MUSTANSWER=N $SCPPATH/scpterms.ss
			case "$?" in
				$OK)	IFS=,
					set `cat < $SCPPATH/configurations/term.tmp`
					TYPE=$1
					PRINTTERMCAP=$2
					IFS=$ORIGIFS
					break ;;
				$RESTART) continue ;;
				$QUIT)	break ;;
			esac
		done

              sed "s|^\(TPORT$BOARD,[0-9a-f],$DEV,$PORT,$LABEL,$USAGE,$ENABLED,$BAUD,\).*|\1$PRINTTERMCAP,$TYPE|" < $CFPROPOTMP > $CFPROPOTMP\2
              mv $CFPROPOTMP\2 $CFPROPOTMP
              [ -z "$BAUD" ] && sysreply=g	# prompt for baud rate
              continue

           # Change printer type
           elif [ $USAGE = P ] ; then
              idx=0
              while :
                 do
		 clear
                 cat  <<-HERE
----------------------------------------------------------------------(Lptype)
	                 Specify your type of printer

HERE
                 awk -F, '{printf "        %-6s  %s\n",$1,$2}' \
                    $SCPPATH/devices/Sprinters$idx
                 cat  <<-HERE

        q)      Quit to main menu (and lose changes)
        ?)      Get an explanation of these options

HERE
                 echo -n "Enter selection? "
                 read reply extra
                 case "$reply" in
                    # quit
                    q*|Q*)
                      exit $QUIT
	              ;;

                    # help
                    \?|help|Help)
		      clear
	              more -c $SCPPATH/scplp.1.hlp
                      echo '* * Press RETURN to continue * *'
                      read reply
	              ;;

                    *)
                      origreply=$reply
                      case $reply in		# Don't allow *
                         *\**)   reply=''
                      esac
                      # only allow numbers, letters, and ?
                      reply=`echo $reply |
                       tr -d "[\001-\057][\072-\076]\100[\133-\140][\173-\177]"`
                      reply=`echo $reply | tr '[A-Z]' '[a-z]'` # make lower case
	              egrep "^$reply\)," $SCPPATH/devices/Sprinters$idx > /dev/null
                      if [ $? -eq 1 -o -z "$reply" ] ; then
                         echo "Invalid response <$origreply>, Press RETURN to continue\n"
			 read reply
                         continue
                      fi

                      if [ $reply = o ] ; then
                         TYPE='unknown'
                      else
                         # reply is a printer name
                         IFS=,
                         set `egrep "^$reply\)," $SCPPATH/devices/Sprinters$idx`
                         TYPE=$2
                         IFS=$ORIGIFS
                      fi
                      break
	              ;;
                 esac
              done
              IFS=,
              set `egrep "^$reply\)," $SCPPATH/devices/Sprinters$idx`
              IFS=$ORIGIFS
              PRINTTERMCAP=$3 FILTER=$4 SPOOLDIR=$5
              sed "s|^\(TPORT$BOARD,[0-9a-f],$DEV,$PORT,$LABEL,$USAGE,$ENABLED,$BAUD,\).*|\1$PRINTTERMCAP,$TYPE,$FILTER,$SPOOLDIR|" \
                 < $CFPROPOTMP > $CFPROPOTMP\2
              mv $CFPROPOTMP\2 $CFPROPOTMP
              [ -z "$BAUD" ] && sysreply=g	# prompt for baud rate

           else
echo "* * Port is not currently a terminal or printer, please retry
* * Press RETURN to continue * *"
              read reply
           fi
           continue
           ;;

         # Change baud rate
         g|G)
	   echo Y > $PRINTERFLAG	# Indicate printer info might change
           while :
	      do
	      clear
              case $USAGE in
                I|M) NOTE="modem" ;;
                P)   NOTE="printer" ;;
                T)   NOTE="terminal" ;;
                O)   NOTE="'Other output' device" ;;
                *)
echo "* * Port'\''s use is not currently specified, please specify * *
* * Press RETURN to continue * *"
                     read reply
                     continue 2
                     ;;
              esac
              cat  <<-HERE
-------------------------------------------------------------------------(Baud)
	       Specify your $NOTE's baud rate

HERE

              if [ $PORT = "f0" ] ; then
                  awk -F, '{
                          printf("        %-5s\n", $1)
                  }' $SCPPATH/devices/baudrates
              else
############ Generate comm board configuration file name ###########
                  IFS=,
                  set `egrep "^TERMBOARD$BOARD" $CFPROPOSED`
                  IFS=$ORIGIFS
                  comboard=$2
                  if [ $comboard = 546 -o $comboard = 547 -o $comboard = 188aa ] ; then
                      comboard=188
                  fi
                  comboard=${SYSCFG}/c${comboard}.c

######## Extract legitimate baud rates from configuration file ######
                  grep "US_B" ${comboard} | awk -F, '{
                      for (i=1; i<=NF; i++){
                          if (($i != 0) && ($i != ""))
                              printf("        %-5s  baud\n", substr($i, index( $i, "US_B") + 4))
                      }
                  }'
              fi

              cat  <<-HERE

        q)      Quit to main menu (and lose changes)
        ?)      Get an explanation of these options

HERE
              echo -n "Enter selection? "
              read reply extra
              case "$reply" in
                 # quit
                 q*|Q*)
                   exit $QUIT
	           ;;

                 # help
                 \?|help|Help)
		     clear
	             more -c $SCPPATH/scpbaud.0.hlp
                     echo '* * Press RETURN to continue * *'
                     read reply
	             ;;
                 *)
                     origreply=$reply
                     case $reply in		# Don't allow *
                        *\**)  reply=''
                     esac
                     # only allow numbers, letters, and ?
                     reply=`echo $reply |
                       tr -d "[\001-\057][\072-\076]\100[\133-\140][\173-\177]"`
                     if [ $PORT = "f0" ] ; then
                         egrep "^$reply " $SCPPATH/devices/baudrates >/dev/null 2>&1
                         if [ $? -ne 0 -o -z "$reply" ] ; then
                             echo "Invalid response <$origreply>, Press RETURN to continue\n"
			     read reply
                             continue
                         fi
                     else
                         egrep "US_B${reply}," $comboard > /dev/null # check validity
                         if [ $? -eq 1 -o -z "$reply" ] ; then
                            egrep "US_B${reply}$" $comboard > /dev/null
                            if [ $? -eq 1 -o -z "$reply" ] ; then

                               echo "Invalid response <$origreply>, Press RETURN to continue\n"
			       read reply
                               continue
                            fi
                         fi
                     fi
                     BAUD=$reply
                     break
                 ;;
              esac
           done
           sed "s|^\(TPORT$BOARD,[0-9a-f],$DEV,$PORT,$LABEL,$USAGE,$ENABLED,\)[0-9]*\(,.*\)|\1$BAUD\2|" < $CFPROPOTMP > $CFPROPOTMP\2
           mv $CFPROPOTMP\2 $CFPROPOTMP
           continue
           ;;

         # Accept as is and re-examine port configuration
         y|Y)
           break	# Go review all ports
           ;;

         # quit
         q*|Q*)
           exit $QUIT
	   ;;

         # help
         \?|help|Help)
	     clear
	     more -c $SCPPATH/scpport.1.hlp
             echo '* * Press RETURN to continue * *'
             read reply
             continue
	     ;;

         # Invalid response
         *)
             echo Invalid response \<$reply\>, Press RETURN to continue
	     read reply
	     ;;
      esac
   done  # END Port

 done  # END One pass over all ports

 # count the number of ports for terminals that are enabled
 temp=`egrep '^TPORT.*,.*,.*,.*,.*.[TI],E' $CFPROPOTMP | wc -l`
 if [ $temp -lt 16 ] ; then
    break
 fi
 clear
 temp=`expr $temp + 1`
 temp1=`expr $temp - 16`
 echo "
* *  Currently $temp ports are enabled, as terminals and dial-in modems.  * *
* *  A maximum of 16 ports can be enabled, for 16 users.  In order to  * *
* *  bring down  the number of  users  on your system  to 16 or less,  * *
* *  please change the usage of $temp1 ports to dial-out modems, printers,  * *
* *  other output devices, or to unused status.                        * *\n
* *  Press RETURN to continue  * *"
 read reply
done  # END Ports

# User has O.K.'d port config., make the temporary proposal the 'real' one
mv $CFPROPOTMP $CFPROPOSED
echo "`date` ended ports " >> $ZAPLOG
