/*
 *	Convert a 214/215 configuration to 3.4 format.
*/
#include "/sys/h/param.h"
#include "/sys/h/buf.h"
#include "/sys/h/iobuf.h"
#include "/usr/save_dir/sys/h/i215g.h"
#include <stdio.h>

/*
 *	The partition and drive tables have unknown names, so we declare
 *	only i215minor[] and i215board[] here.  These are required by the
 *	driver to have certain names and point to the partition and drive
 *	information.
*/
extern struct i215minor i215minor[];
extern struct i215cfg  i215cfg[];

/*
 *	These are static variables that the driver requires to interpret
 *	the tables above.  We require them too.
*/
extern short i215maxmin;		/* Size of minor table		*/
extern int N215;			/* Size of board table		*/

extern char *malloc(unsigned);
extern char *realloc(char *, unsigned);

/*
 *	Tagged memory allocator.  We use this to create a copy of a data
 *	structure and remember (in the tag) which data structure is was.
 *	This is central to the imnplementation of the list manager.
*/

/*
 *	Allocate 'size' bytes of memory with tag 'tag'.
*/
char *
talloc(size, tag)
unsigned size;
char *tag;
{ register char **p;

  p = (char **)malloc(size+sizeof(tag));
  if (p == NULL)
    return(NULL);
  *p = tag;
  return((char *)(p+1));
}
/*
 *	Return the tag associated with a piece of memory allocated
 *	via talloc().
*/
char *
tag(ptr)
char *ptr;
{ char **p;

  p = (char **)ptr;
  return(*--p);
}

/*
 *	General list manager.  We implement a dynamically allocated list
 *	of pointers to 'size' byte structures.
*/

/*
 *	General list structure.  We implement a dynamically allocated list
 *	of pointers to 'size' byte structures.
*/
struct list {
  unsigned size;		/* Size of each cell		*/
  char **cells;			/* Pointer to array of cells	*/
  unsigned count;		/* Number of cells in array	*/
};

/*
 *	Macro used to inialize lists at compile time.
*/
#define MAKELIST(size)	{ size, NULL, 0 }

/*
 *	Insert or find 'ptr' in list 'list'.
*/
unsigned
ins_list(list, ptr, func)
struct list *list;
char *ptr;
void (*func)(char *);
{ register char **p;
  register i;

  if (list->cells == NULL)		/* Kludge initial case		*/
    list->cells = (char **)malloc(list->count * sizeof(list->cells[0]));
  for (i = 0; i < list->count; i++)
    if (tag(list->cells[i]) == ptr)
      return(i);			/* If found, return the index	*/
  /*
   *	Realloc() the list for one more element, then talloc() a
   *	new element and insert it.
  */
  list->cells = (char **)realloc((char *)list->cells,
				 (list->count+1) * sizeof(list->cells[0]));
  if (list->cells == NULL) {
    fprintf(stderr, "Out of memory for list management\n");
    abort();				/* Nice for debugging		*/
  }
  list->cells[list->count] = talloc(list->size, ptr);
  if (list->cells[list->count] == NULL) {
    fprintf(stderr, "Out of memory for list elements\n");
    abort();				/* Nice for debugging		*/
  }
  /*
   *	Initialize the new list element to be a copy of the structure at
   *	*ptr.  Call the user's function for additional initialization.
  */
  for (i = 0; i < list->size; i++)
    list->cells[list->count][i] = ptr[i];
  (*func)(list->cells[list->count]);
  /*
   *	Return the index of the new list element.
  */
  return(list->count++);
}

/*
 *	Initialize the new configuration to be null.
*/
struct list n215minor = MAKELIST(sizeof(*i215minor));
struct list n215cfg = MAKELIST(sizeof(*i215cfg));
	/*
	 *	NOTE - sizeof(i215cfg->c_drtab[0]) is wrong below for 3.0 if
	 *	we are compiling on 3.4 and vice-versa.  We must fudge the
	 *	size of n215drtab elements at run time.
	*/
struct list n215cdrt = MAKELIST(sizeof(i215cfg->c_drtab[0][0]));

/*
 *	Print the configuration which has been so painfully deciphered.
*/

/*
 *	Print the standard introduction.
*/
prologue()
{ long l;
  struct i215cdrt *p;

  printf("/*\n");
  printf(" *\tiSBC 215 Specific Configuration file.\n");
  printf(" *\tConfiguration ported from XENIX 3.0 to 3.4.\n");
  printf(" *\tModify this file at your own risk.\n");
  printf("*/\n");
  printf("#define\t\tCMI10MB\t\t0\n");
  printf("#define\t\tCMI15MB\t\t0\n");
  printf("#define\t\tQNTM36MB\t0\n");
  printf("#define\t\tPRIAM32\t\t0\n");
  printf("#define\t\tPRIAM62\t\t0\n");
  printf("#define\t\tMXTR72\t\t0\n");
  printf("#define\t\tMXTR89\t\t0\n");
  printf("#define\t\tMXTR121\t\t0\n");
  printf("#define\t\tTSBA86\t\t0\n\n");
#ifndef risky
  printf("#define\tMSDOS\t0\n");
#else risky
  p = i215cfg->c_drtab[0];
  l = (p->cdr_nfhead+p->cdr_nrhead) * p->cdr_nsec * p->cdr_secsiz;
					/* Compute size of a cylinder	*/
  l = p->cdr_part[5].p_nsec / l;	/* Convert K to cylinders	*/
  printf("#define\tMSDOS\t%ld\n\n",l);
#endif risky
  printf("#define\tROOTBLKS\t%ld\n\n",i215cfg->c_drtab[0]->cdr_part[1].p_nsec);
  printf("#include \"../h/param.h\"\n");
  printf("#include \"../h/buf.h\"\n");
  printf("#include \"../h/iobuf.h\"\n");
  printf("#include \"../h/i215g.h\"\n\n");
  /*
   *	This really needs to be created from /etc/bootcfg at run time to
   *	support the full flexibility desired of boot diskettes.  There is
   *	logic in the new sysadmin and instl to read /etc/bootcfg.
  */
  printf("#if CMI10MB\n");
  printf("#  define SECTORS 9\n");
  printf("#  define BYTES 1024\n");
  printf("#  define CYLINDERS 306\n");
  printf("#  define HEADS 4\n");
  printf("#  define INTLV 4\n");
  printf("#  define PERIPHS 5\n");
  printf("#  define SWPSIZE 1188\n");
  printf("#  define RTCGSZ 3172\n");
  printf("#  define RTXTENT 8\n");
  printf("#  define i215TYPE DEVWINIG\n");
  printf("#  define FLPYTYPE DEV5FLPY\n");
  printf("#  define TAPETYPE STREAMER\n");
  printf("#endif CMI10MB\n");
  printf("#if CMI15MB\n");
  printf("#  define SECTORS 9\n");
  printf("#  define BYTES 1024\n");
  printf("#  define CYLINDERS 306\n");
  printf("#  define HEADS 6\n");
  printf("#  define INTLV 4\n");
  printf("#  define PERIPHS 5\n");
  printf("#  define SWPSIZE 1188\n");
  printf("#  define RTCGSZ 2976\n");
  printf("#  define RTXTENT 8\n");
  printf("#  define i215TYPE DEVWINIG\n");
  printf("#  define FLPYTYPE DEV5FLPY\n");
  printf("#  define TAPETYPE STREAMER\n");
  printf("#endif CMI15MB\n");
  printf("#if QNTM36MB\n");
  printf("#  define SECTORS 9\n");
  printf("#  define BYTES 1024\n");
  printf("#  define CYLINDERS 512\n");
  printf("#  define HEADS 8\n");
  printf("#  define INTLV 4\n");
  printf("#  define PERIPHS 5\n");
  printf("#  define SWPSIZE 4104\n");
  printf("#  define RTCGSZ 2736\n");
  printf("#  define USRCGSZ 2959\n");
  printf("#  define RTXTENT 8\n");
  printf("#  define USRXTENT 8\n");
  printf("#  define i215TYPE DEVWINIG\n");
  printf("#  define FLPYTYPE DEV5FLPY\n");
  printf("#  define TAPETYPE STREAMER\n");
  printf("#endif QNTM36MB\n");
  printf("#if PRIAM32\n");
  printf("#  define SECTORS 12\n");
  printf("#  define BYTES 1024\n");
  printf("#  define CYLINDERS 525\n");
  printf("#  define HEADS 5\n");
  printf("#  define INTLV 4\n");
  printf("#  define PERIPHS 8\n");
  printf("#  define SWPSIZE 4104\n");
  printf("#  define RTCGSZ 2736\n");
  printf("#  define USRCGSZ 3075\n");
  printf("#  define RTXTENT 8\n");
  printf("#  define USRXTENT 8\n");
  printf("#  define i215TYPE DEVWINI\n");
  printf("#  define FLPYTYPE DEV8FLPY\n");
  printf("#  define TAPETYPE INVALID\n");
  printf("#endif PRIAM32\n");
  printf("#if PRIAM62\n");
  printf("#  define SECTORS 12\n");
  printf("#  define BYTES 1024\n");
  printf("#  define CYLINDERS 1050\n");
  printf("#  define HEADS 5\n");
  printf("#  define INTLV 4\n");
  printf("#  define PERIPHS 8\n");
  printf("#  define SWPSIZE 8208\n");
  printf("#  define RTCGSZ 2736\n");
  printf("#  define USRCGSZ 3476\n");
  printf("#  define RTXTENT 8\n");
  printf("#  define USRXTENT 8\n");
  printf("#  define i215TYPE DEVWINI\n");
  printf("#  define FLPYTYPE DEV8FLPY\n");
  printf("#  define TAPETYPE INVALID\n");
  printf("#endif PRIAM62\n");
  printf("#if MXTR72\n");
  printf("#  define SECTORS 9\n");
  printf("#  define BYTES 1024\n");
  printf("#  define CYLINDERS 1024\n");
  printf("#  define HEADS 8\n");
  printf("#  define INTLV 4\n");
  printf("#  define PERIPHS 5\n");
  printf("#  define SWPSIZE 9216\n");
  printf("#  define RTCGSZ 2736\n");
  printf("#  define USRCGSZ 3476\n");
  printf("#  define RTXTENT 8\n");
  printf("#  define USRXTENT 8\n");
  printf("#  define i215TYPE DEVWINIG\n");
  printf("#  define FLPYTYPE DEV5FLPY\n");
  printf("#  define TAPETYPE STREAMER\n");
  printf("#endif MXTR72\n");
  printf("#if MXTR89\n");
  printf("#  define SECTORS 9\n");
  printf("#  define BYTES 1024\n");
  printf("#  define CYLINDERS 918\n");
  printf("#  define HEADS 11\n");
  printf("#  define INTLV 4\n");
  printf("#  define PERIPHS 5\n");
  printf("#  define SWPSIZE 16416\n");
  printf("#  define RTCGSZ 2736\n");
  printf("#  define USRCGSZ 3476\n");
  printf("#  define RTXTENT 8\n");
  printf("#  define USRXTENT 8\n");
  printf("#  define i215TYPE DEVWINIG\n");
  printf("#  define FLPYTYPE DEV5FLPY\n");
  printf("#  define TAPETYPE STREAMER\n");
  printf("#endif MXTR89\n");
  printf("#if MXTR121\n");
  printf("#  define SECTORS 9\n");
  printf("#  define BYTES 1024\n");
  printf("#  define CYLINDERS 918\n");
  printf("#  define HEADS 15\n");
  printf("#  define INTLV 4\n");
  printf("#  define PERIPHS 5\n");
  printf("#  define SWPSIZE 16416\n");
  printf("#  define RTCGSZ 2736\n");
  printf("#  define USRCGSZ 3476\n");
  printf("#  define RTXTENT 8\n");
  printf("#  define USRXTENT 8\n");
  printf("#  define i215TYPE DEVWINIG\n");
  printf("#  define FLPYTYPE DEV5FLPY\n");
  printf("#  define TAPETYPE STREAMER\n");
  printf("#endif MXTR121\n");
  printf("#if TSBA86\n");
  printf("#  define SECTORS 9\n");
  printf("#  define BYTES 1024\n");
  printf("#  define CYLINDERS 830\n");
  printf("#  define HEADS 10\n");
  printf("#  define INTLV 4\n");
  printf("#  define PERIPHS 5\n");
  printf("#  define SWPSIZE 9216\n");
  printf("#  define RTCGSZ 2790\n");
  printf("#  define USRCGSZ 2970\n");
  printf("#  define RTXTENT  8\n");
  printf("#  define USRXTENT 8\n");
  printf("#  define i215TYPE DEVWINIG\n");
  printf("#  define FLPYTYPE DEV5FLPY\n");
  printf("#  define TAPETYPE STREAMER\n");
  printf("#endif TSBA86\n\n");
  printf("struct i215part Mmt0[] = {\n");
  printf("\t0,\t0\t/* Archive streamer */\n");
  printf("};\n\n");
  printf("struct i215cdrt i215t00[] = {\n");
  printf("/* present, no-op, no-op, no-op, no-op, !rew?, Part, NoPrt. */\n");
  printf("   01,      0,     0,     0,     0,     0,     Mmt0, 0,\n");
  printf("   01,      0,     0,     0,     0,     1,     Mmt0, 0\n");
  printf("};\n");
  printf("struct\ti215part Pf0[] = {\n");
  printf("#if PERIPHS == 8\t\t\t/* 8 in. System */\n");
  printf("\t8,\t1224,\t/* [0] (minor 07) dnf0\tTrack 1 through rest: 608K  */\n");
  printf("\t8,\t608,\t/* [1] (minor 08) f0\tTrack 1 through rest: 1224K */\n");
  printf("\t0,\t2002,\t/* [2] (minor 09) syf0\tall of disk: 250K  */\n");
  printf("\t26,\t3978,\t/* [3] (minor 10) dxf0\tTrack 1-rest: 994.5K */\n");
  printf("\t8,\t1224,\t/* [4] (minor 11) df0\tTrack 1 through rest: 1224K */\n");
  printf("\t26,\t780,\t/* [5] (minor 12) boot\t780 sectors: 195k */\n");
  printf("\t806,\t832\t/* [6] (minor 13) ram\t832 sectors: 208k */\n");
  printf("#else\t\t\t\t\t/* 5.25 System */\n");
  printf("\t8,\t632,\t/* [0] (minor 07) dnf0\tRMX         format\t*/\n");
  printf("\t4,\t316,\t/* [1] (minor 08) df0\tXenix R1    format\t*/\n");
  printf("\t0,\t640,\t/* [2] (minor 09) syf0\tbootstrap   format\t*/\n");
  printf("\t0,\t640,\t/* [3] (minor 10) dzf0\tMSDOS R1    format\t*/\n");
  printf("\t0,\t720,\t/* [4] (minor 11) dvf0\tMSDOS R2    format\t*/\n");
  printf("\t8,\t184,\t/* [5] (minor 12) boot\tboot floppy format\t*/\n");
  printf("\t192,\t112\t/* [6] (minor 13) ram\tboot floppy format\t*/\n");
  printf("#endif\tPERIPHS\n");
  printf("};\n\n");
  printf("struct\ti215cdrt i215f00[] = {\n");
  printf("#if PERIPHS == 8\t\t\t/* 8 in. System */\n");
  printf("/*Cyls, Fixed, Remov,  #Sec,  SecSiz, Nalt, Part , NoPrt  Unit: Drive-Type */\n");
  printf("  77,\t0,\t2,\t8,\t1024, FLPY_MFM, Pf0, 0, /* [0] dvf0 [07] */\n");
  printf("  77,\t0,\t1,\t8,\t1024, FLPY_MFM, Pf0, 0, /* [1] f0   [08] */\n");
  printf("  77,\t0,\t1,\t26,\t128,  FLPY_FM,  Pf0, 0, /* [2] syf0 [09] */\n");
  printf("  77,\t0,\t2,\t26,\t256,  FLPY_MFM, Pf0, 0, /* [3] dxf0 [10] */\n");
  printf("  77,\t0,\t2,\t8,\t1024, FLPY_MFM, Pf0, 0, /* [4] df0  [11] */\n");
  printf("  77,\t0,\t2,\t26,\t256,  FLPY_MFM, Pf0, 0, /* [5] boot [12] */\n");
  printf("  77,\t0,\t2,\t26,\t256,  FLPY_MFM, Pf0, 0  /* [6] ram  [13] */\n");
  printf("#else\t\t\t\t\t/* 5.25 System */\n");
  printf("/*Cyls, Fixed, Remov,  #Sec,  SecSiz,  Nalt, Part, NoPrt  Unit: Drive-Type */\n");
  printf("  40,\t0,\t2,\t8,\t512,  FLPY_MFM,\tPf0, 0, /* [0] dnf0 [07] */\n");
  printf("  40,\t0,\t2,\t4,\t1024, FLPY_MFM, Pf0, 0, /* [1] df0  [08] */\n");
  printf("  40,\t0,\t1,\t16,\t128,  FLPY_FM,  Pf0, 0, /* [2] syf0 [09] */\n");
  printf("  40,\t0,\t2,\t8,\t512,  FLPY_MFM, Pf0, 0, /* [3] dzf0 [10] */\n");
  printf("  40,\t0,\t2,\t9,\t512,  FLPY_MFM, Pf0, 0, /* [4] dvf0 [11] */\n");
  printf("  40,\t0,\t2,\t4,\t1024, FLPY_MFM, Pf0, 0, /* [5] boot [12] */\n");
  printf("  40,\t0,\t2,\t4,\t1024, FLPY_MFM, Pf0, 0  /* [6] ram  [13] */\n");
  printf("#endif\tPERIPHS\n");
  printf("};\n\n");
  printf("#define CYLSIZ\t(HEADS*SECTORS)\n");
  printf("#define DISKSIZ\t(CYLINDERS*CYLSIZ-CYLSIZ)\n");
  printf("#define W_ALTS\t(((CYLINDERS*HEADS)/50)/HEADS)\n");
  printf("#define DIAGSIZ\t(W_ALTS*CYLSIZ+CYLSIZ)\n");
  printf("#define UNXENIX (DIAGSIZ+(MSDOS*CYLSIZ))\n\n");
  printf("#ifndef USRCGSZ\n");
  printf("#  define ROOTSIZ (DISKSIZ-UNXENIX-SWPSIZE-2*SECTORS)\n");
  printf("#  define RESTSIZ 0\n");
  printf("#else\n");
  printf("#  define ROOTSIZ ROOTBLKS\n");
  printf("#  define RESTSIZ (DISKSIZ-UNXENIX-SWPSIZE-ROOTSIZ-2*SECTORS)\n");
  printf("#endif\n");
  printf("#  define NOPRT   7\n\n");
  printf("struct\ti215part Piw0[] = {\n");
  printf("\t0,\t\t\tDISKSIZ,\t/* [0] whole disk\t*/\n");
  printf("\tSECTORS,\t\tROOTSIZ,\t/* [1] \"root\"\t\t*/\n");
  printf("\tROOTSIZ+SECTORS,\tSWPSIZE,\t/* [2] \"swap\"\t\t*/\n");
  printf("\tSWPSIZE+ROOTSIZ+SECTORS, RESTSIZ,\t/* [3] \"rest\"\t\t*/\n");
  printf("\tSECTORS,\t\tRESTSIZ+SWPSIZE+ROOTSIZ,\n");
  printf("\t\t\t\t\t\t/* [4] mountable device\t*/\n");
  printf("\tDISKSIZ-UNXENIX,\tMSDOS*CYLSIZ,\t/* [5] \"MS-DOS\"\t\t*/\n");
  printf("\tSWPSIZE+ROOTSIZ+SECTORS+RESTSIZ, SECTORS/* [6] \"error\"\t\t*/\n");
  printf("};\n\n");
  printf("struct\ti215cdrt i215d00[] = {\n");
  printf("\tCYLINDERS, HEADS, 0, SECTORS, 1024, W_ALTS, Piw0, NOPRT\n");
  printf("};\n\n");
}

/*
 *	Print the partition table.
*/
partab(n)
{ unsigned i;
  struct i215cdrt *d;
  struct list *l;
  struct i215part *p;

  printf("struct i215part Part%d[] = {\n",n+1);
  printf("/* Sector,    Count */\n");
  d = (struct i215cdrt *)n215cdrt.cells[n];
  l = (struct list *)d->cdr_part;
  for (i = 0; i < l->count; i++) {
    p = (struct i215part *) (l->cells[i]);
    printf(" %8ld, %8ld,\n", p->p_fsec, p->p_nsec);
  }
  printf("};\n\n");
}

/*
 *	Print the drive table.
*/
drtab()
{ unsigned i;
  struct i215cdrt *p;

  /*
   *	Need magic to deduce error partition info for winis.  This could get
   *	ugly.  In fact, I'm pretty sure it's impossible to second guess
   *	safely.
  */
  printf("struct i215cdrt i215d01[] = {\n");
  printf("/* Cyls, Fixed, Remov,  #Sec,  SecSiz,  Nalt, Partab, Npart */\n");
  for (i = 0; i < n215cdrt.count; i++) {
    p = (struct i215cdrt *)n215cdrt.cells[i];
    printf("  %4d,   %2d,    %2d,     %2d,    %4d,    %2d,   Part%d,   %d,\n",
            p->cdr_ncyl, p->cdr_nfhead, p->cdr_nrhead, p->cdr_nsec,
            p->cdr_secsiz, p->cdr_nalt, i+1, 0/*p->cdr_pnum*/);
  }
  printf("};\n\n");
}

/*
 *	Print the board table.
*/
boardtab()
{ unsigned i, j;
  struct i215cfg *p;

  printf("struct\ti215cfg\t i215cfg[] = {\n");
  printf("/* WUA, DevCode[0], [1],       [2],   Int,\tDevice Table[unit] */\n");
  printf("0x01000L, i215TYPE, FLPYTYPE, TAPETYPE, 5,\ti215d00,/* Wini 0 */\n");
  p = (struct i215cfg *)n215cfg.cells[0];
  for (j = 1; j < NUMSPINDLE; j++) {
    switch (j) {
      case FIRSTFLOPPY:
        printf("\t\t\t\t\t\ti215f00,\n");
        break;
      case FIRSTTAPE:
        printf("\t\t\t\t\t\ti215t00,\n");
        break;
      default:
        if (p->c_drtab[j])
          printf("\t\t\t\t\t\ti215d01,\n");
        else
          printf("\t\t\t\t\t\t      0,\n");
    }
  }
  for (i = 1; i < n215cfg.count; i++) {
    p = (struct i215cfg *)n215cfg.cells[i];
    printf("  0x%05lxL,    %2d,  %2d,  %2d, %3d,\n",
            p->c_wua, p->c_devcod[0], p->c_devcod[1],
            p->c_devcod[2], p->c_level);
    for (j = 0; j < NUMSPINDLE; j++)
      if (p->c_drtab[j])
        printf("\t\t\t\t\t\ti215d0%d,\n",i);
      else
        printf("\t\t\t\t\t\t      0,\n");
  }
  printf("};\n\n");
}

/*
 *	Print the minor number table.
*/
minortab()
{ unsigned i;
  struct i215minor *p;

  printf("unsigned i215minor[] = {\n");
  for (i = 0; i < n215minor.count; i++) {
    p = (struct i215minor *)n215minor.cells[i];
    printf("\ti215MINOR(%2d,%2d,%2d,%2d),\t/* [%2d] */\n",
           p->board, p->unit, p->drtab, p->partition, i);
  }
  printf("};\n\n");
}

/*
 *	Print standard trailing stuff.
*/
epilogue()
{ printf("int i215retry = 10;\t/* Soft error retry count */\n\n");
  printf("#define NUM215 ((sizeof i215cfg) / (sizeof (struct i215cfg)))\n");
  printf("int N215 = NUM215;\t\t/* Number of configured boards */\n");
  printf("struct iobuf i215tab[NUM215];\t/* Buffer headers */\n");
  printf("struct iobuf i215tbuf[NUM215];\t/* Tape buffer headers */\n");
  printf("struct buf i215rbuf[NUM215];\t/* Raw buffer headers */\n");
  printf("struct i215dev *i215bdd[NUM215];/* Board-index to dev map */\n");
  printf("struct i215dev i215dev[NUM215];\t/* per-board data structures */\n");
  printf("short i215maxmin = ((sizeof i215minor)/(sizeof(i215minor[0])));\n\n");
  printf("#define i215TMEM 8\t/* Number of tape buffers */\n");
  printf("int i215tbuf_max = i215TMEM;\n");
  printf("struct i215tmem i215tmem[i215TMEM+1];\n");
}

/*
 *	The main driver for the printing.
*/
void
dump()
{ unsigned i;

  prologue();
  for (i = 0; i < n215cdrt.count; i++)
    partab(i);
  if (i > 0)
	drtab();
  boardtab();
  minortab();
  epilogue();
}

/*
 *	Helpers for ins_list.  These initialize new list entries as they
 *	are created.
*/
void
f_cdrt(p)
char *p;
{ register struct i215cdrt *dr;
  struct list *plst;
  register unsigned i;

  dr = (struct i215cdrt *)p;
  plst = (struct list *)malloc(sizeof(*plst));
  plst->size = sizeof(struct i215part);
  plst->cells = NULL;
  plst->count = 0;
  /*
   *	This is ugly and non-portable, but makes life simpler overall.
  */
  dr->cdr_part = (struct i215part *)plst;
}

void
f_null(p)
char *p;
{
}

/*
 *	Replicate minor number description in new tables.
*/
void
ins_minor(m)
unsigned m;
{ struct i215cfg  *obd;
  struct i215cdrt *odr, *pdrt;
  struct i215part *opt;
  struct list *plst;
  unsigned u, nbd, ndr, npt;

  if (i215minor[m].board <= 3) {
    switch (i215minor[m].unit) {
      case 0:		/* Units 0 (boot wini), 4 (floppy), and		*/
      case FIRSTFLOPPY: /* 8 (tape) are special cased for board 0.	*/
      case FIRSTTAPE:
        (void) ins_list(&n215cfg, (char *)&i215cfg[0], f_null);
	  			/* Force board 0 to be created		*/
        if (i215minor[m].board == 0)
          break;
        /* else treat as default case */
      default:		/* Other, insert in special tables		*/
        obd = &i215cfg[i215minor[m].board];
        /*
         *	The statement below assumes i215g.h is the same
         *	revision as c215g.o.
        */
        u = i215minor[m].unit;		/* Help the optimizer out - it	*/
        odr = obd->c_drtab[u];	 	/* gets carried away and then	*/
        odr = &odr[i215minor[m].drtab];	/* confuses itself.		*/
        opt = &odr->cdr_part[i215minor[m].partition];
        /*
         *	Create structures needed to describe the minor number
         *	if they don't already exist.
        */
        nbd = ins_list(&n215cfg, (char *)obd, f_null);
	  			/* Replicate board description		*/
        ndr = ins_list(&n215cdrt, (char *)odr, f_cdrt);
	  			/* Replicate unit description		*/
        pdrt = (struct i215cdrt *) n215cdrt.cells[ndr];
        plst = (struct list *) pdrt->cdr_part;
        npt = ins_list(plst, (char *)opt, f_null);
	  			/* Replicate unit description		*/
        /*
         *	Make an entry in the new minor table.  (Destroys old one...)
        */
        i215minor[m].board = nbd;
        i215minor[m].unit = u;
        i215minor[m].drtab = ndr;
        i215minor[m].partition = npt;
    } /* end of switch default case */
  } /* end if */
  (void) ins_list(&n215minor, (char *)&i215minor[m], f_null);
}

/*
 *	For each entry in the minor table, adjust the new tables as needed.
*/

static char cnv215_version[] ="@(#) cnv215 $State: XENIX_R3-5 $ $Date: 86/07/03 10:39:36 $";

int
main()
{ register unsigned m;

  for (m = 0; m < i215maxmin; m++)
    ins_minor(m);
  dump();			/* Print the new c215g.c		*/
  return(0);
}
