/*
 *	control sequences for CI-3500
 *
 *      Copyright 1985, Intel Corporation.
 *
 *	note:
 *		does not include vertical format specifications,
 *		
 *		or dot corresponding graphics mode
 *		Also, if an on seq. is used as an off seq., it must
 *		appear after the on seq in the table.
 *		Example, Printers char set 1 is def char set, and appears
 *		after all the other char sets in the table
 *
 *	ci3500ctl.c
 *
 *	Written by David Callahan for Intel 05/08/85
 */


static char ci3500ctl_version[] ="@(#) ci3500ctl $State: XENIX_R3-5 $ $Date: 86/05/30 08:43:47 $";
#include "ilpf.h"

struct escseq esctab[] = {
	0,3,2,0,0,"\33R\03","\33\67",		/* United Kingdom */
	0,3,2,0,0,"\33R\01","\33\67",		/* French/Belgium */
	0,3,2,0,0,"\33R\02","\33\67",		/* German */
	0,3,2,0,0,"\33R\06","\33\67",		/* Italian */
	0,3,2,0,0,"\33R\05","\33\67",		/* Swedish/Finish */
	0,3,2,0,0,"\33R\04","\33\67",		/* Danish/Norwegian */
	0,3,2,0,0,"\33R\07","\33\67",		/* Spanish */
	0,3,2,0,0,"\33R\10","\33\67",		/* Japan */
	0,3,2,0,0,"\33R\0","\33\67",		/* US ASCII */
	0,2,2,0,0,"\33\66","\33\67",		/* Printer char set 2 */
	0,2,0,0,0,"\33\67","",			/* Printer char set 1 */
	0,3,0,0,0,"\33U\0","",			/* bi directional print */
	0,3,0,0,0,"\33U\1","",			/* uni directional print */
	0,2,0,0,0,"\33<","",			/* home head */
	0,3,3,0,0,"\33+\01","\33+\0",		/* lq/dp mode */
	0,3,3,0,0,"\33+\0","\33+\0",		/* dp mode */
	0,2,2,0,0,"\33\115","\33\120",		/* 12 CPI */
	0,2,2,0,0,"\33\143","\33\120",		/* condensed */
	0,2,2,0,0,"\33\120","\33\120",		/* 10 CPI */
	0,3,2,0,0,"\33\141\0","\33\62",		/* 1/3 inch line feed */
	0,3,2,0,0,"\33\141\01","\33\62",	/* 1/4 inch line feed */
	0,2,2,0,0,"\33\60","\33\62",		/* 1/8 inch line feed */
	0,3,2,0,0,"\33\141\02","\33\62",	/* 1/9 inch line feed */
	0,3,2,0,0,"\33\141\03","\33\62",	/* 1/12 inch line feed */
	0,2,0,0,0,"\33\62","",			/* 1/6 inch line feed */
	0,3,2,0x0,0xff,"\33\101\33","\33\61",	/* n/72 inch line feed */
	0,3,2,0x0,0xff,"\33\73\33","\33\61",	/* n/144 inch line feed */
	0,3,2,0x0,0xff,"\33\63\33","\33\61",	/* n/216 inch line feed */
	0,2,0,0,0,"\33\61","",			/* 7/72 inch line feed */
	0,3,3,0x0,0x7f,"\33\103\33","\33\103\102",	/* page len by lines */
	0,4,4,0x0,0x18,"\33\103\0\33","\33\103\0\13",	/* page len by inches */
	0,4,4,0x0,0x7f,"\33x\33\33","\33x\5\6",	/* top/bottom margins */
	0,3,3,0x0,0x7f,"\33y\33","\33y\5",	/* top margin */
	0,3,3,0x0,0x7f,"\33z\33","\33z\6",	/* bottom margin */
	0,3,2,0,0x7f,"\33N\33","\33O",		/* perforation */
	0,3,3,0,0,"\33)\01","\33)\00",		/* vert tab set pres pos. */
	0,2,0,0,0,"\33d","",			/* vertical tab reset */
	0,3,0,0x0,0x7f,"\33\146\33","",		/* forward to line n */
	0,3,0,0x0,0x7f,"\33\147\33","",		/* forward by n lines */
	0,2,0,0,0,"\33\42","",			/* 1 line reverse */
	0,3,0,0,0,"\33J\33","",			/* n/144 inch dot low feed */
	0,3,0,0,0,"\33,\33","",			/* n/216 inch dot low feed */
	0,4,4,0x0,0xff,"\33k\33\33","\33k\3\113",	/* left/right margins */
	0,3,3,0x0,0xff,"\33\154\33","\33\154\3",	/* left margin */
	0,3,3,0x0,0xff,"\33Q\33","\33Q\113",	/* right margin */
	0,3,3,0,0,"\33(\01","\33(\00",		/* horizntl tab set pres pos. */
	0,2,0,0,0,"\33e","",			/* horizontal tab reset */
	0,3,3,0,0,"\33W\1","\33W\0",		/* horizontal enlarged */
	0,3,3,0,0,"\33V\1","\33V\0",		/* vertical enlarged */
	0,3,3,0,0,"\33X\1","\33X\0",		/* italic */
	0,2,2,0,0,"\33E","\33F",		/* bold */
	0,2,2,0,0,"\33G","\33H",		/* bold */
	0,3,3,0,0,"\33-\1","\33-\0",		/* underline */
	0,3,2,0,0,"\33S\1","\33T",		/* subscript */
	0,3,2,0,0,"\33S\0","\33T",		/* superscript */
	0,3,2,0,0xff,"\33!\33","\33r",		/* superscript */
	0,3,0,0,0,"\33[\0","",			/* LF + CR */
	0,3,0,0,0,"\33[\1","",			/* LF only */
	0,3,0,0,0,"\33]\0","",			/* LF only */
	0,3,0,0,0,"\33]\1","",			/* CR + LF*/
	0,2,0,0,0,"\33@","",			/* CR + LF*/
	TABEND,0,0,0,0,"0","0" };

unsigned char 	escs[] = "\33 ";

char	cntlcd[] = "\07\13\16\17\21\22\23\24\30";
/**	\07		buzzer
	\13		vertical tab
	\16		enlarged print
	\17		condensed print
	\21		on-line mode
	\22		end condensed print
	\23		off-line mode
	\24		end enlarged print
	\30		cancel	**/
