/*
 *	control sequences for Mannesmann Tally Matrix printer MT 420/440
 *
 *      Copyright 1985, Intel Corporation.
 *
 *	mt440ctl.c
 *
 *	note:
 *		if an on sequence is used as the default off sequence
 *		also.  the on sequence should appear in the table
 *		after all occurances of it in the off sequence.
 *		Example:
 *			US ASCII is the default char set.
 *			Therefore, is appears after all the
 *			other char sets.
 *
 *	Written by David Callahan for Intel 05/07/85
 */


static char mt440ctl_version[] ="@(#) mt440ctl $State: XENIX_R3-5 $ $Date: 86/05/30 08:43:25 $";
#include "ilpf.h"

struct escseq esctab[] = {
	0,4,4,0,0,"\33[2z","\33[1z",		/* 4 LPI */
	0,4,4,0,0,"\33[3z","\33[1z",		/* 6 LPI */
	0,4,4,0,0,"\33[4z","\33[1z",		/* 8 LPI */
	0,4,4,0,0,"\33[7z","\33[1z",		/* 12 LPI */
	0,4,4,0,0,"\33[1z","\33[1z",		/* 3 LPI */
	0,4,4,0,0,"\33[0w","\33[4w",		/* 5 CPI */
	0,4,4,0,0,"\33[1w","\33[4w",		/* 6 CPI */
	0,4,4,0,0,"\33[2w","\33[4w",		/* 8 CPI */
	0,4,4,0,0,"\33[5w","\33[4w",		/* 12 CPI */
	0,4,4,0,0,"\33[6w","\33[4w",		/* 16 CPI */
	0,4,4,0,0,"\33[4w","\33[4w",		/* 10 CPI */
	0,3,3,0,0,"\33(A","\33(B",		/* United Kingdom */
	0,3,3,0,0,"\33(R","\33(B",		/* French/Belgium */
	0,3,3,0,0,"\33(K","\33(B",		/* German */
	0,3,3,0,0,"\33(Y","\33(B",		/* Italian */
	0,3,3,0,0,"\33(H","\33(B",		/* Swedish/Finish */
	0,3,3,0,0,"\33(E","\33(B",		/* Danish/Norwegian */
	0,3,3,0,0,"\33(Z","\33(B",		/* Spanish */
	0,3,3,0,0,"\33(B","\33(B",		/* US ASCII */
	0,3,3,0,0,"\33%2","\33%1",		/* high density quality */
	0,4,4,0x30,0x39,"\33[\33s","\33[1s",	/* 1st print position */
	0,5,4,0x30,0x39,"\33[\33\33s","\33[1s",	/* 1st print position */
	0,6,4,0x30,0x39,"\33[\33\33\33s","\33[1s",	/* 1st print pos */
	0,5,7,0x30,0x39,"\33[;\33s","\33[;132s",	/* last print pos */
	0,6,7,0x30,0x39,"\33[;\33\33s","\33[;132s",	/* last print pos */
	0,7,7,0x30,0x39,"\33[;\33\33\33s","\33[;132s",	/* last print pos */
	0,4,5,0x31,0x39,"\33[\33t","\33[66t",	/* form length */
	0,5,5,0x30,0x39,"\33[\33\33t","\33[66t",	/* form length */
	0,6,5,0x30,0x39,"\33[\33\33\33t","\33[66t",	/* form length */
	0,5,6,0x30,0x39,"\33[;\33r","\33[;60r",	/* last print line */
	0,6,6,0x30,0x39,"\33[;\33\33r","\33[;60r",	/* last print line */
	0,7,6,0x30,0x39,"\33[;\33\33\33r","\33[;60r",	/* last print line */
	0,4,4,0x30,0x39,"\33[\33r","\33[6r",	/* 1st print line */
	0,5,4,0x30,0x39,"\33[\33\33r","\33[6r",	/* 1st print line */
	0,6,4,0x30,0x39,"\33[\33\33\33r","\33[6r",	/* 1st print line */
	0,4,4,0x30,0x39,"\33[\33u","\33[3g",	/* set horizontal tab */
	0,5,4,0x30,0x39,"\33[\33\33u","\33[3g",	/* set horizontal tab */
	0,6,4,0x30,0x39,"\33[\33\33\33u","\33[3g",	/* set horizontal tab */
	0,4,4,0x30,0x39,"\33[\33q","\33[3g",	/* clear horizontal tab */
	0,5,4,0x30,0x39,"\33[\33\33q","\33[3g",	/* clear horizontal tab */
	0,6,4,0x30,0x39,"\33[\33\33\33q","\33[3g",	/* clear horizontal tab */
	0,4,4,0x30,0x39,"\33[\33v","\33[4g",	/* set vertical tab */
	0,5,4,0x30,0x39,"\33[\33\33v","\33[4g",	/* set vertical tab */
	0,6,4,0x30,0x39,"\33[\33\33\33v","\33[4g",	/* set vertical tab */
	0,4,4,0x30,0x39,"\33[\33x","\33[4g",	/* clear vertical tab */
	0,5,4,0x30,0x39,"\33[\33\33x","\33[4g",	/* clear vertical tab */
	0,6,4,0x30,0x39,"\33[\33\33\33x","\33[4g",	/* clear vertical tab */
	0,3,0,0,0,"\33#3","",			/* 3 * ht & wd */
	0,3,0,0,0,"\33#5","",			/* 5 * ht & wd */
	0,3,0,0,0,"\33#7","",			/* 7 * ht & wd */
	TABEND,0,0,0,0,"0","0" };

unsigned char 	escs[] = "\33 ";

char	cntlcd[] = "\13\30\34\35\36";
/**	\13		vertical tab
	\30		cancel
	\34		lg char printing 3 * ht and wd
	\35		lg char printing 5 * ht and wd
	\36		lg char printing 7 * ht and wd	**/
