/*
 *	INTEL CORPORATION PROPRIETARY INFORMATION
 *
 *	This software is supplied under the terms
 *	of a license agreement or nondisclosure
 *	agreement with Intel Corporation and may
 *	not be copied nor disclosed except in
 *	accordance with the terms of that agreement.
 * Copyright (C) 1984, 1985, 1986 Intel Corp.
 */
/*
 * "@(#) filsys $State: XENIX_R3-5 $ $Date: 86/05/30 13:02:36 $"
 */

#define FS_CLEAN	106
#define	BMAPSIZE	994		/* Max size of CG bit map */
					/* Equals BSIZE-sizeof(struct cylinder)*/

#define MAXCGS		80		/* Max CG's per filsys */
#define	MAXEXTSIZE	32		/* Max extent size */
#define FNEWCG		64		/* When a file grows beyond FNEWCG KB,
					   allocate blocks from a new
					   cylinder group */
#define SNEWCG		512		/* Move to a new cylinder group after
					   every subsequent SNEWCG KB */

/* Cylinder group header */
struct cylinder {
    daddr_t	cg_doffset;	/* offset to first data block from start of filsys */
    daddr_t	cg_ioffset;	/* offset to first inode block from start of filsys */
    unsigned	cg_dblocks;	/* number of data blocks in cg */
    ino_t	cg_ifirst;	/* next free inode in linked list */
    char	cg_number;	/* cg sequence number in filsys */
    char	cg_currextent;	/* current extent size */
    unsigned	cg_lowat;	/* if free blocks drop below cg_lowat, recompute cg_currextent */
    unsigned	cg_hiwat;	/* if free blocks increase beyond cg_hiwat, recompute cg_currextent */
    unsigned	cg_erotor;	/* position of next candidate block for allocation */
    char	cg_ilock;	/* inode manipulation lock */
    char	cg_reserved[9]; /* reserved field. (9 to align on word boundary) */
    char	cg_bits[BMAPSIZE]; /* bit map. 0 = allocated. 1 = free */
};

/*
 * Contains global policy information.
 * Stored in the superblock.
 */
struct	cginfo {
    struct buf near	*fs_cgincore; /* points to buf structure containing cg header. Null if not in core */
    daddr_t	fs_cgblk;     /* disk address of cg header */
    unsigned	fs_cgffree;   /* number of free data blocks in cg */
    ino_t	fs_cgifree;   /* number of free inodes in cg */
    ino_t	fs_cgdirs;    /* number of directories in cg */
};

/*
 * Super block
 */
struct filsys {
    char	fs_fname[6];	/* file system name */
    char	fs_fpack[6];	/* pack name */
    daddr_t	fs_fsize;	/* number of data blocks in fs */
    unsigned	fs_cgblocks;	/* number of blocks per cg */
    daddr_t	fs_maxblock;	/* max disk block in fs */
    ino_t	fs_cginodes;	/* number of inodes per cg */
    ino_t	fs_maxino;	/* max inumber in fs */
    time_t	fs_time;	/* time last modified */
    char	fs_fmod;	/* modified flag */
    char	fs_ronly;	/* read-only fs */
    char	fs_clean;	/* fs was cleanly unmounted */
    char	fs_type;	/* fs type and version */
    unsigned	fs_fnewcg;	/* contains FNEWCG */
    unsigned	fs_snewcg;	/* contains SNEWCG */
    daddr_t	fs_ffree;	/* number of free data blocks in fs */
    ino_t	fs_ifree;	/* number of free inodes in fs */
    ino_t    	fs_dirs;	/* number of directories in fs */
    char	fs_extentsize;	/* native extent size */
    char	fs_cgnum;	/* number of cg's in fs */
    char	fs_cgrotor;	/* next cg to be searched */
    char	fs_reserved[15];/* reserved. (15 to align on word boundary) */
    struct cginfo fs_cylinder[MAXCGS];/* contains global policy information
					per cylinder group */
};

/*
 * fields for fs_type
 *
 *    xxxxxxxx
 *    vv		File-system version
 *      ss		Xenix version (sys3, sys5..)
 *	  bb		Block size
 *	    rr		Reserved. (Must be set to 0).
 *
 */

#define		FS_VERS		0x00	/* Version 1.0 */
#define		FS_VMASK	0xC0	/* mask for extracting version */
#define		FS_SYS		0x00	/* System III */
#define		FS_SMASK	0x30	/* mask for extracting OS version */
#define		FS_BSIZE	0x00	/* 1k block size */
#define		FS_BMASK	0x0C	/* mask for extracting block size */

/*
 * Information for computing current extent size
 */
struct extsizetab {
    int		lowmark;	/* recompute extent size when free blocks
				   in a cg fall below this */
    int		highmark;	/* recompute extent size when free blocks
				   in a cg rise above this */
    int		divfact;	/* scaling factor */
};
extern struct extsizetab extsizetab[];  /*  Needed for prefcgi()	*/

/*  
 *  The following two defines are used by cmpextsz() to determine which
 *  extsizetab to use.
 */
#define EXT_FILL	0	/*  Used to index into extsiztab */
#define EXT_FREE	1	/*  Used to index into extsiztab */

/* Constants used by freecgp
 */

#define	NOMOD	1	/* No change in cg header, donot write  */
#define	CYLSYNC	2	/* Write cg header to disk immediately  */
#define	DELAYED	3	/* Perform a delayed write on cg header */

/* Constants used by bmap() and alloc()
 */
#define	BLKALLOC	0	/* block will be used as reguler data */
#define	INDALLOC	1	/* block will be used as indirect/dir */
