/*
 * Copyright 1983, 1986, 1987 Intel Corporation
 *
 *	Modification History:
 *
 *	I000	clq	3/21/86
 *		Added #define for INCLOSE bit and WANTLINE bit
 *	I001	brad	10/11/86
 *		Added #define for TANRESP bit (currently 546/547/548 only)
 *	I002	brad	12/5/86
 *		Added #define for ABORT transmission
 *	I003	brad	12/10/86
 *		Added #define for SPL
 *		Added #define for I188LINES and used throughout
 *		Added #define for CARRF
 *		Added baud rates (0, 75, 134, 200, 1800)
 *	I004	brad	12/19/86
 *		Added i188special defines
 *
 * static char i188_version[] ="@(#) i188 $State: XENIX_R3-5 $ $Date: 87/04/07 17:45:42 $";
 */

#include "mmu.h"

#define MINORMSK        0x1F		/* reserve bit 7; bit 6 for modem */
#define	ECHOMSK		0x20		/* echo bit in minor number */
#define MODEMMSK        0x40		/* bit 6, minor number modem flag */
#define	RAWMSK		0x80		/* bit 7, open as raw device */
#define MODEMWAIT       0x40		/* wait for carrier modem op */
#define MAXBAUDS	18		/* maximum number baud rates defined */
#define NCL188		5		/* clock ticks for timeout character */
#define CLEAR		0		/* no message received or sent */
#define ALIVE		0x01		/* operational flag, board and line */
#define I188ERROR	0x02		/* hardware error flag */
#define INTR_CLR	0x04		/* clear 188/48 interrupt */
#define I188LIMIT	0xC000		/* size of 188/48 memory size - 1 */
#define I188ACC		DSA_DATA	/* access permit */
#define	I188TYPES	4		/* maximum varient board types */
#define	I188BOARD	1		/* board type for standard 188/48 */
#define	I548BOARD	2		/* board type for 547/548 board */
#define	I546BOARD	3		/* board type for 546 with clock */
#define RESET		1		/* board reset command/ response */
#define iSBX354s	0x0101		/* multimodule init request */
#define	CARRF		0x40		/* actual carrier state 	I003 */
#define I188LINES	12		/* max lines per 188 board 	I003 */
#define INSTOP		0x04		/* input interrupts stopped */
#define INBUSY		0x08		/* output stopped while input busy */
#define INCLOSE		0x10		/* I000 driver is currently in the
					   close routine */
#define WANTLINE	0x20		/* I000 driver has been called to
					   open a line but the INCLOSE bit
					   is set */

/*
 * This is a temporary location for this until
 * we decide what include file this should go in.
 */
#define	LPSTAT_GET	(('p'<<8)|0)	/* get the parallel port status */

/*
 * iSBC 188/48 Private Data Structures
 *------------------------------------
 *
 * These are the descriptions of the data structures required
 * by the 188/48 device driver to manage the 188/48 hardware.
 */

/*
 * The structure of the message buffer
 */
struct i188msg	{
	char	 m_type;		/* message command type */
	char	 m_line;		/* line on board being commanded */
	unsigned m_cnt;			/* byte count for data transfers */
	unsigned m_ptr;			/* offset pointer, from board base */
	unsigned m_buf[5];		/* variable message data segment */
	};

/*
 * line structure, one per line on a board
 */
struct i188line {
	char     l_state;		/* driver specific line state */
	char     l_wait;		/* output delay timeout value */
	unsigned l_ibsiz;		/* size of input line buffer */
	unsigned l_iba;			/* input line buffer base address */
	unsigned l_ibp;			/* input buffer data pointer */
	unsigned l_ibc;			/* input buffer data count */
	unsigned l_obsiz;		/* size of output line buffer */
	unsigned l_oba;			/* output line buffer base address */
	unsigned l_obp;			/* output buffer available pointer */
	unsigned l_obc;			/* output buffer usage count */
	unsigned l_ocnt;		/* output outstanding message count */
	};


/*
 * Structure of all the information to know the state of
 * the iSBC 188/48 board's firmware
 */
struct	i188board {
	int	 b_alive;	 	/* set at init if board is there */
	short	 b_select;		/* segment selector of board */
	unsigned b_port;		/* board's interrupt port */
	char	 b_type;		/* board type; 188 = 1; LCTC8 = 2 */
	char	 b_version;		/* firmware version */
	long	 b_addr;		/* physical address of board's memory */
	struct	 i188msg b_msg;		/* message buffer, task time */
	struct	 i188line b_line[I188LINES];	/* 1 per line state data I003 */
	};

/*
 * board configuration structure declaration
 * there is one structure for each board
 * with predefines configuration data.
 */
struct	i188cfg {
	long	c_addr;			/* board's physical address */
	int	c_port;			/* board's interrupt address */
	int	c_level;		/* board's interrupt level */
	};


/*
 * Macros to obtain the board and line numbers
 * from the the device number.
 */
#define	BRDNO(d)	((d&MINORMSK) / I188LINES)	/* I003 */
#define	LINNO(d)	((d&MINORMSK) % I188LINES)	/* I003 */


/*
 * Commands Message Types to the iSBC 188/48
 */
#define	INIT		0x01	/* initialization message */
#define	ENABLE		0x02	/* enable line message */
#define	DISABL		0x03	/* disable line message */
#define	CONFIG		0x04	/* configure line parameters message */
#define	OUTPUT		0x05	/* transmit buffer message */
#define	ABORT		0x06	/* abort transmission in progress  I002 */
#define	SUSPND		0x07	/* suspend transmit message */
#define	RESUME		0x08	/* resume transmit message */
#define	DTRAST		0x09	/* assert DTR message */
#define	DTRCLR		0x10	/* clear DTR message */
#define BRKSET		0x11	/* set BREAK condition on output */
#define BRKCLR		0x12	/* clear BREAK condition on output */
#define	INPUT		0x15	/* clear receive buffer message */

/*
 * Commands Message Types from the iSBC 188/48
 */
#define	OUTCMP		0x01	/* transmit complete message */
#define	INAVIL		0x02	/* input available message */
#define	ONCARR		0x04	/* carrier detect message */
#define	OFCARR		0x05	/* carrier loss message */
#define INTCMP		0x06	/* initialization complete message */
#define	SPCOND		0x08	/* special character received message */


/*
 * line configuration message parameters
 */
#define	PNO		0x00	/* no parity */
#define	PODD		0x03	/* odd parity */
#define	PEVEN		0x02	/* even parity */
#define C6BITS		0x00	/* 6 bit data */
#define	C7BITS		0x04	/* 7 bit data */
#define	C8BITS		0x08	/* 8 bit data */
#define	STBITS		0x00	/* stop bits - default 		I003 */
#define	STBITS1		0x00	/* stop bits - 1 		I003 */
#define	STBITS15	0x10	/* stop bits - 1.5 		I003 */
#define	STBITS2		0x20	/* stop bits - 2 		I003 */
#define	LNDISP		0x100	/* line discipline, for firmware */

/*
 * mode bits for firmware configuration
 */

#define	TANDEM		0x01	/* tandem mode, for firmware I004 */
#define	SPCHAR		0x02	/* special characters, for firmware */
#define	TANRESP		0x04	/* tandem response mode, for firmware I001 */

#define SPHIWAT		0x100	/* special character high water mark */

/*
 * Supported baud rates of the 188 device driver
 */
#define US_B0		0		/* I003 */
#define US_B50		50
#define US_B75		75		/* I003 */
#define US_B110		110
#define US_B150		150
#define US_B134		134		/* I003 */
#define US_B200		200		/* I003 */
#define US_B300		300
#define US_B600		600
#define US_B1200	1200
#define US_B1800	1800		/* I003 */
#define US_B2400	2400
#define US_B4800	4800
#define US_B9600	9600
#define US_B19200	19200
#define US_B38400	38400
#define US_B51800	51800
#define US_B76800	-1

/*
 * iSBC 188/48 board buffer addresses from the board base
 * and buffer sizes for each line and the aggregate per board.
 */
#define BDTEST		   16	/* board's static structure test area */
#define	BDSTART		16384	/* start of dual port for 188 boards */

#define OUTQBASE	 1808	/* base offset of output message queue */
#define INQBASE		  272	/* base offset of input message queue */

#define OUTBASE_8LINES	18864	/* base address of line output buffers */
#define OUTSIZE_8LINES	 1737	/* size of one line's output buffer */
#define OUTBASE_12LINES	26624	/* base address of line output buffers */
#define OUTSIZE_12LINES	 1877	/* size of one line's output buffer */

#define INBUFBASE	 3344	/* base address of line input buffers */
#define INLINSIZ	 1940	/* size of one line's input buffer */

/*
 * special configuration bit masks for i188special in cfg/c188.c
 *	f/c == flow control
 */

#define	I188_OUT_XONBOARD	0x1	/* TANDEM RESPONSE (output f/c) mode */
#define	I188_INP_XONBOARD	0x2	/* TANDEM (input f/c) mode */
#define	I188_8_PLUS_PARITY	0x4	/* Split out PARITY setting from CS# */

/*
 * The values in these bits defines a timeout time so we don't hang in close
 *     This has particular value when you have a port that sends an XOFF
 *	character then goes away.
 *	The default is 0x1700 which is (0x1700/HZ ticks == ~2 minutes)
 */

#define	I188_TMOUT_MSK		0xff00	/* mask for timeout in close */
#define	I188_WAIT_FOREVER	0xff00	/* don't setup a timeout in i188close */
#define	I188_WAIT_DEFAULT	0x1700	/* default timeout time */
