
/*
 * "@(#) i208 $State: XENIX_R3-5 $ $Date: 86/05/30 13:09:51 $"
 */

struct i208minor {
	unsigned 	partition:	4;	/* partition number */
	unsigned	drtab:		4;	/* drtab number */
	unsigned	unit:		4;	/* unit drtabs */
	unsigned	board:		4;	/* board number */
};

#define MAXSECTORS	26		/* max number sectors per track  A005 */
#define NUMUNITS	4		/* max number devices per board */

#define	UNIT(dev)	(i208minor[minor(dev)].unit)  /* dev-> unit # map */
#define DRTAB(dev)	(i208minor[minor(dev)].drtab) /* dev->drtab-index map */
#define PARTITION(dev)  (i208minor[minor(dev)].partition) /*dev->partition map*/
#define BOARD(dev)	(i208minor[minor(dev)].board) /* dev->board index map */
#define i208MINOR(bnum,unum,drnum,panum) ((bnum<<12)|(unum<<8)|(drnum<<4)|panum)

#define LOW(x)		((x) & 0x0FF)		/* low byte */
#define HIGH(x)		((x) >> 8 & 0x0FF)	/* high byte */

#define ON		0x0F			/* on signal for minifloppy */
#define OFF		0x00			/* off signal for minifloppy */

#define DEV8FLPY	0			/* 8" floppy */
#define DEV5FLPY	1			/* 5-1/4" floppy */

#define OPEN		0x01			/* unit open */
#define READY 		0x02			/* unit ready */

/* FDC commands */
#define SPECIFY		0x03			/* specify command */
#define SDS		0x04			/* sense drive status command */
#define WRITE_OP	0x05			/* write command */
#define READ_OP		0x06			/* read command */
#define RECALIBRATE     0x07			/* recalibrate command */
#define SIS		0x08			/* sense interrupt status cmd */
#define FORMAT_OP	0x0D			/* format command */
#define SEEK		0x0F			/* seek command */
#define FILLER		0xFF			/* Filler byte */

#define ACC_MASK	0xFFF0			/* Mask for access bits */
#define ACCESS		0xFFD0			/* interrupt caused by access */
#define RDYBITS		0x00C0			/* FDC master I/O line */
#define BUSY		0x001F			/* FDC busy bit */

/* Port addresses */
#define C0_CAR		0x00		/* channel 0 current address reg */
#define C0_CWR		0x01		/* channel 0 current word count reg */
#define C2_CAR		0x04		/* channel 2 current address reg */
#define C2_CWR		0x05		/* channel 2 current word count reg */
#define C3_CAR		0x06		/* channel 3 current address reg */
#define C3_CWR		0x07		/* channel 3 current word count reg */
#define CMD_DMA		0x08		/* DMAC command register */
#define STAT_DMA	0x08		/* DMAC status register */
#define REQ_DMA		0x09		/* DMAC request register */
#define MASK_DMA 	0x0A		/* DMAC Mask register(set/reset) */
#define MODE_DMA 	0x0B		/* DMAC Mode register */
#define CLRFF_DMA 	0x0C		/* Clear DMAC first/last flipflop */
#define CLEAR_DMA 	0x0D		/* DMAC Master clear */
#define LMASK_DMA 	0x0F		/* Load DMAC Mask register */
#define STAT_FDC 	0x10		/* FDC status register */
#define CMDPORT		0x11		/* FDC data register */
#define AUXPORT		0x12		/* Controller auxilliary port */
#define RESET		0x13		/* Controller reset */
#define SAR_LOW		0x14		/* segment address register-low byte */
#define SAR_HIGH	0x15		/* segment address register-high byte */


/* Result Status bits */
#define	US	   0x0003		/* drive unit causing interrupt */
#define AT         0x0040		/* abnormal termination (st0) */
#define IC         0x00C0		/* bits for termination (st0) */
#define INVALID    0x0080		/* invalid command (st0) */
#define SE	   0x0020		/* seek end (st0) */
#define EC         0x0010		/* equipment check (st0) */
#define DE 	   0x0020		/* data error (st1) */
#define NR	   0x0008		/* not ready */
#define EN	   0x0080		/* multitrack transfer (st1) */
#define NW         0x0002		/* not writable (st1) */
#define WPROT      0x0040		/* write protected (st3) */

#define IO_OP(bp) ((bp->b_flags & B_READ) ? READ_OP : ((bp->b_flags & B_FORMAT)\
			? FORMAT_OP : WRITE_OP)) /* Extract op from buffer */

#define	IO_IDLE		0			/* buffer flag - device idle */
#define IO_BUSY		1			/* buffer flag - device busy */
#define IO_OPEN_WAIT	2			/* buf flag - waiting on open */

#define B_FORMAT	040000			/* format op for buffer */

#define FLPY_FM		0			/* single density */
#define FLPY_MFM	1			/* double density */

#define DSDD		0xC0			/* double sided-double density*/
#define SSDD		0x40			/* single sided-double density*/
#define DSSD      	0x80			/* double sided-single density*/
#define SSSD		0x00			/* single sided-single density*/

#define WRITERDY	0x0080			/* write ready flag */
#define READRDY		0x00C0			/* read ready flag */

#define READ_MODE	0x44			/* read mode */
#define WRITE_MODE	0x48			/* write mode */

#define SF_OPEN		0x01			/* device open */
#define SF_READY        0x02			/* device ready */


#define I208_IOC_FMT    (('W' << 8) | 0)

/* constant drive table characteristics - one per board */
struct i208cdrt {
	unsigned	cdr_ncyl;	/* number cylinders */
	char		cdr_mode;	/* density */
	unsigned     	cdr_bps;	/* bytes per sector */
	char		cdr_secsiz;	/* sector size */
	unsigned	cdr_spt;	/* sectors per track */
	char		cdr_nhead;	/* number heads (sides) */
	char		cdr_gplrw;	/* gap3 length - R/W */
	char		cdr_gplfor;	/* gap3 length - FORMAT */
	char		cdr_size;	/* media size (8") */
	struct i208part *cdr_part;	/* pointer to partition table */
};



/* configuration table - one per board */
struct i208cfg {
	unsigned	c_addr;		/* base port address */
	char		c_devcod;	/* 8" or 5-1/4" drives */
	char		c_level;	/* interrupt level */
	char		c_hut;		/* head unload time */
	char		c_hlt;		/* head load time */
	char 		c_srt;		/* step rate time */
        struct i208cdrt *c_drtab[NUMUNITS]; /* drive characteristic table ptr */
};


/* partition table - one for every media type */
struct i208part {
	daddr_t		p_fsec;		/* first sector */
	daddr_t		p_nsec;		/* number sectors */
};


/* state table - one per board */
struct i208state {
									/*A005*/
	char    	s_opening;		/* flag - recal on open   A004*/
	char    	s_exists;		/* flag that board exists */
	unsigned	s_addr;			/* base port address */
	char    	s_timeout;	        /* motor on/off for minifloppy*/
									/*A005*/
	char    	s_flags[NUMUNITS];	/* unit open/ready */
	char    	s_popen[NUMUNITS];	/* bit[i] == partition[i] open*/
	char            s_devcod;		/* type of floppy drive */
	char    	s_pcn[NUMUNITS];	/* present cylinder number */
									/*A005*/
	char 		s_reset;		/* seek called from reset flag*/
	char		s_recal[NUMUNITS];	/* flag to issue 2nd recal */
	unsigned 	s_retries;		/* retries on soft error */
	struct i208cdrt **s_cdrtab;     	/* ptr to drive char table */
	struct iobuf    *s_bufh;	       	/* buffer header */
};

/* parameter block - one per board */
struct i208iopb {
	char            i_cmd;		/* command */
	char     	i_trackno;	/* track number */
	char    	i_headdrive;	/* head & drive */
	char	        i_secno;	/* sector number */
	char		i_eot;		/* last sector */
	char		i_secsiz;	/*  sector length (128 * 2**N) */
        char    	i_spt;		/* sectors per track */
	char    	i_bps;		/* bytes per sector */
	char    	i_gpl;		/*  gap3 length */
	unsigned	i_numbytes;	/* number bytes to be moved */
	unsigned	i_highaddr;	/* high order address */
	unsigned	i_lowaddr;	/* low order address */
};

/* result table - one per board */
struct i208result {	/* result table */
	char		r_pcn;		/* present cylinder number */
	char		r_st0;		/* status register 0 */
	char		r_st1;		/* status register 1 */
	char		r_st2;		/* status register 2 */
	char		r_st3;		/* status register 3 */
	char		r_c;		/* cylinder number */
	char		r_h;		/* head address */
	char		r_r;		/* sector number (record) */
	char		r_n;		/* sector length */
};

/* format table - one per board */
struct i208format {
	char		f_track;	/* track number */
	char		f_head;		/* head number */
	char		f_secno;	/* sector number */
	char	 	f_secsiz;	/* sector size */
	};

/* device table - one per board */
struct i208dev {
	struct i208state	d_state;		/* state table */
	struct i208iopb		d_iopb;			/* parameter block */
	struct i208format	d_format[MAXSECTORS];	/* format table  A005 */
	struct i208result	d_result;		/* result table */
};

/* user format structure */
struct i208ftk {
	int	f_trck;			/* track number */
	int	f_intl;			/* interleave factor */
	int	f_skew;			/* track skew */
	int	f_type;			/* format type-code */
	int	f_pat[4];		/* pattern data */
};

