
/*
 * "@(#) i226 $State: XENIX_R3-5 $ $Date: 86/08/31 14:28:04 $"
 */

/*
 *	INTEL CORPORATION PROPIETARY INFORMATION
 *
 *	This software is supplied under the terms
 *	of a license agreement or nondisclosure
 *	agreement with Intel Corporation and may
 *	not be copied nor disclosed except in
 *	accordance with the terms of that agreement.
 */

/**********************************************************************
 *
 * TITLE:	iSBC 226 Device Driver
 *
 * DATE:	January 4, 1985
 *
 * AUTHOR:	Richard N. Bryant
 *
 * SUMMARY:
 *	iSBC 226 Device Driver include file.
 *
 * MODIFICATIONS:
 *	I001	01/08/86	jdm
 *		Added bad block testing data structure.
 *      I002    06/04/86        pats	
 *	        Added subfunction field to support the read defect map
 *              function.
 *	
 **********************************************************************/

/* 
 * Define constant values used by the iSBC 226 driver.
 * General flags and general purpose bit patterns.
 */
#define	PRTMASK		0x00ff		/* partition mask for minor number */
#define	NIBBLE		0x000f		/* lower nibble mask */
#define	LOBYTE		0x00ff		/* lower byte mask */
#define	LOWORD		0xffff		/* low word mask */
#define	ERROR	       -1		/* function error return */
#define	LOC_BUF		1		/* local buffer flag for getablk */
#define	EXP_BUF		0		/* exported buffer flag for getablk */
#define	FORMAT_READBADBK 0x02		/* read defect map of new drive I002 */

/*
 * Constants used in the controllers drtab flag field,
 * and aliases to use buffer header fields for driver
 * specific purposes.
 */
#define	IO_IDLE		0		/* controller is idle - field 0 */
#define	IO_BUSY		1		/* controller is busy - bit 0 sit */
#define	IO_WAIT		2		/* wait for idle - bit 1 sit */
#define	IO_READY	4		/* request pending - bit 2 sit */

#define	b_command	b_error		/* used to hold IOPB command */
#define	b_subfn  	b_res		/* used to hold IOPB subfunc. *I002 */
#define	b_retry		b_resid		/* holds soft error retry count */

/*
 * Calculates drive number, board number, or
 * partition number component from the minor
 * number of the device.
 */
#define DRVNO(m)	(((m)>>4)&3)
#define	BRDNO(m)	(((m)>>6)&3)
#define	PRTNO(m)	(m&NIBBLE)

/*
 * I/O parameter block structure.
 * defines the communication interface
 * of the iSBC 226 controller.
 * Size is 34 byte per IOPB entry.
 * Last three structure parameters are
 * driver specific.
 */
struct	i226iopb {
	unsigned char
		i_comm,			/*  0 Disk Command / Ready 	*/
		i_imode,		/*  1 Interrupt Mode		*/
		i_stat1,		/*  2 Status Byte 1		*/
		i_stat2,		/*  3 Status Byte 2		*/
		i_throt,		/*  4 Throttle			*/
		i_drive,		/*  5 Drive Type, Unit Select	*/
		i_head,			/*  6 Head Address		*/
		i_sector;		/*  7 Sector Address		*/
	unsigned short
		i_cyl,			/*  9,8 Cylinder Address 	*/
		i_count,		/*  B,A Sector Count 		*/
		i_addr,			/*  D,C Data Offset Addr	*/
		i_reloc;		/*  F,E Data Relocation		*/
	unsigned char
		i_hdoffs,		/* 10 Head Offset		*/
		i_resv;			/* 11 Reserved			*/
	unsigned short
		i_niopb,		/* 13,12 IOPB Chain Address	*/
		i_eccmask,		/* 15,14 ECC Mask Pattern	*/
		i_eccoffs;		/* 17,16 ECC Offset		*/
	struct buf
		*i_bp;			/* 18,19 Buffer Header Pointer,
					   added for driver's use.	*/
	struct i226iopb
		*i_link;		/* 1A,1B Queue List Link.	*/
} ;
	

/* COMMAND BYTE (IOPB BYTE 0) */
#define	AUD		0x80		/* automatic update of IOPB */
#define RELOC		0x40		/* generate 24-bit addresses */
#define	CHEN		0x20		/* enable command chaining */
#define IEN		0x10		/* enable interrupts */
#define WRITE		0x01		/* write command */
#define READ		0x02		/* read command */
#define	WRITEHDR	0x03		/* write track headers */
#define	READHDR		0x04		/* read track headers */
#define	DRIVCLR		0x06		/* clear drive command */
#define WRITEFMT 	0x07		/* write-format command */
#define	READHDRDATA	0x08		/* read headers data and ecc */
#define SETSIZE		0x0b		/* set drive size command */
#define	SELFTEST 	0x0c		/* controller selftest command */

/* IMODE BYTE (IOPB BYTE 1) */
#define	IEI		0x40		/* interrupt on each IOPB */
#define	IERR		0x20		/* interrupt on error */
#define	ASR		0x08		/* auto seek retry */
#define	EEF		0x04		/* enable overlap seeks, sector slip */
#define	ECM		0x02		/* does correct ECC errors */

/* STATUS BYTE 1 (IOPB BYTE 2) */
#define	PB_DONE		0x01		/* set when IOPB is done */
#define	PB_ERR		0x80		/* IOPB processing error flag */

/* STATUS BYTE 2 (IOPB BYTE 3) */
#define	NOERROR		0x00		/* no, reserved, or unknown error */
#define	SOFTERR		0x01		/* soft error type */
#define	HARDERR		0x02		/* hard error type */
#define	RESETERR	0x40		/* error requires a drive reset */
#define	MAX_ERR		0x25		/* maximum IOPB error number */
#define	MAX_RETRY	1		/* maximum soft error retries */
#define	DRV_DOWN	0x16		/* drive not ready error number */


/* THROTTLE BYTE (IOPB BYTE 4) */
#define THROT		0x02		/* transfer burst throttle */
#define	ZERO_INTRLV	0x87		/* zero interleave mask */
#define	INTERLEAVE	0x01		/* 1:1 interleave factor, default */
#define	BXFER		0x80		/* byte transfer mode for DMA */
#define	WDXFER		0x00		/* word transfer mode for DMA */

/* DRIVE BYTE (IOPB BYTE 5) */
#define	AFE		0x10		/* adaptive (sector offset) format */
#define	FHD_DRIV	0x00		/* drive type - fixed part */
#define	RHD_DRIV	0x01		/* drive type - removable part */
#define	MAX_DTYPE	0x04		/* maximum number of drive types */
#define SET_DTYPE(p, t)	((p)->i_drive |= ((t&3)<<6)) /* set drive type in IOPB */
#define GET_DTYPE(p)	(((p)->i_drive>>6)&3)	/* drive type from IOPB */
#define	UNIT(p)		(((p)->i_drive)&3)	/* unit number from IOPB */

/* DRIVE STATUS (IOPB BYTE A) */
#define	DNRDY		0x40		/* bit 6 clear if drive is ready */
#define	DFLT		0x04		/* set on drive fault condition */

/* HEADER OFFSET (IOPB BYTE 10) */
#define	IGN_DTYPE	0x40		/* ignore drive type in drive header */
#define	SERVO		0x80		/* embedded servo (hard sector) flag */

/* SUBFUNCTION (IOPB BYTE 11)     *I002  */
#define	NOSUBFN  	0x00		/* no subfunction of the command     */
#define	SUBFN_1		0x01		/* sub function 1 of the command     */


/*
 * The IOPB queues are described with the following structure.
 * The macros that follow are to add and delete IOPB's from
 * the specified queue. These macros should be called the
 * spl level raised to splbuf to prevent race conditions.
 */
struct i226pb_queue {
	struct i226iopb *pb_head;	/* pointer to first IOPB in queue */
	struct i226iopb *pb_tail;	/* pointer to last IOPB in queue */
	int pb_qmax;			/* maximum number of IOPB's for queue */
	int pb_cnt;			/* number of IOPB's in queue */
	};


/* test for queue empty */
#define	PB_QEMPTY(q)	((q)->pb_cnt == 0)

/* test for a full queue */
#define	PB_QFULL(q)	((q)->pb_cnt >= (q)->pb_qmax)

/* puts an IOPB block at the end of the specified queue */
#define	PB_ENQUE(q, p) { \
	(p)->i_niopb = 0; \
	(p)->i_link = 0; \
	if((q)->pb_tail == 0) \
		(q)->pb_head = p; \
	else { \
		((q)->pb_tail)->i_link = p; \
		((q)->pb_tail)->i_niopb = (ushort)(ktop(p)); \
	} \
	(q)->pb_tail = p; \
	(q)->pb_cnt++; \
}

/* removes an IOPB block from the beginning of the specified queue */
#define	PB_DEQUE(q, p) { \
	p = (q)->pb_head; \
	if((q)->pb_cnt) { \
		if(((q)->pb_head = (p)->i_link) == 0) \
			(q)->pb_tail = 0; \
		(q)->pb_cnt--; \
	} \
}


/*
 * The controller interface is defined by a set of
 * register address offsets form a given base, and
 * the meaning of bits in the controller status
 * register flag byte.
 */

/*
 * Structure to access address bytes in
 * IOPB long address.
 */
struct	i226pb_addr {
	union	u_addr {
		unsigned long	pb_l;
		unsigned char	pb_c[4];
	}u_pb;
};
#define	u_pl		pb_a.u_pb.pb_l
#define	u_pc		pb_a.u_pb.pb_c


/* CONTROLLER I/O REGISTER OFFSETS */
#define IOPBRL		0x00		/* IOPB Relocation Register, Low Byte */
#define IOPBRH		0x01		/* IOPB Relocation Register, High Byte */
#define IOPBAL		0x02		/* IOPB Address Register, Low Byte */
#define IOPBAH		0x03		/* IOPB Address Register, High Byte */
#define IOPBCS		0x04		/* IOPB Control and Status Register */
#define IOPBCR		0x05		/* IOPB Controller Reset Register */

/* CONTROLLER STATUS REGISTER FLAGS */
#define GBSY		0x80		/* set to begin a transfer */
#define ERR		0x40		/* an error has occurred */
#define DERR		0x20		/* a double error has occurred */
#define IPND		0x10		/* interrupt currently pending */
#define	ADRM		0x08		/* 24 bit addressing when set */
#define	DRDY		0x01		/* drive ready flag */


/*
 * load IOPB address into controllers relocation registers
 * and starts controller
 *
 * arguments: base address, iopb queue pointer
 */
#define	PB_START(b, q) {\
	struct i226pb_addr pb_a;\
	u_pl = ktop((q)->pb_head);\
	((q)->pb_tail)->i_comm &= ~CHEN;\
	outb((b)+IOPBAL, u_pc[0]);\
	outb((b)+IOPBAH, u_pc[1]);\
	outb((b)+IOPBRL, u_pc[2]);\
	outb((b)+IOPBRH, u_pc[3]);\
	outb((b)+IOPBCS, GBSY);\
}


/* read the control status register of controller */
#define	GET_CSR(b)	inb((b)+IOPBCS)

/* write to control status register */
#define	SET_CSR(b, v)	outb((b)+IOPBCS, v)


/*
 * An array of this structure defines a list of all of the error
 * messeges for errors known to the controller. It contains the
 * messege to be printed at the console, the type of error, and
 * the Xenix error number to go in u.u_error.
 */
struct i226errlst {
	char	err_num;		/* Xenix corresponding error number */
	char	err_type;		/* NOERR, SOFTERR, or HARDERR type */
	char	*err_msg;		/* error message printed by driver */
	};




/*
 * This represents the structure of a sector header.
 * It is used to overlay the header data in a buffer
 * the iSBC 226 controller has read from the drive
 * track.
 */
struct i226sec_hdr {
	union u_h {
		struct h_F {
			ushort		h_cyl;
			unsigned char	h_head;
			unsigned char	h_snum;
		}h_F;
		unsigned long h_hdr;
	}u_h;
};

#define	SH_SPARE	0xddddddddL	/* spare sector header value */
#define	SH_BAD		0xeeeeeeeeL	/* bad sector header value */
#define	SH_SEC		0x3f		/* sector number mask */
#define	SH_DTYPE	0x00c0		/* device type mask */
#define	SH_USED		0xffff		/* sector used status */
#define	SH_INVAL	0x00ee		/* invalid sector status */
#define	sh_hdr		u_h.h_hdr
#define	sh_cyl		u_h.h_F.h_cyl
#define	sh_head		u_h.h_F.h_head
#define	sh_snum		u_h.h_F.h_snum




/*
 * These are driver specific data structures.
 * They represent the configuration data, the
 * per board information required by the driver,
 * and format of drive size characterists and
 * drive partitioning.
 */


/*
 * drive size structure.
 * there is one structure for
 * each drive supported by the driver
 * Note:
 *	The number of sectors per track
 *	is the total physical sectors
 *	minus number of alternate sectors.
 */
struct i226size {
	ushort	 s_ncyl;		/* number of cylinders one drive */
	ushort	 s_fhds;		/* number of fixed heads (surfaces) */
	ushort	 s_rhds;		/* number of removable heads (surfaces) */
	unsigned char
		 s_hdoff;		/* removable platters head offset,
					   high bit must be set for embedded
					   servo (hard sector) drives */
	ushort	 s_sec;			/* number virtual sectors per track */
	ushort	 s_secsiz;		/* number bytes per sector */
	ushort	 s_altsec;		/* number reserved sectors per track */
	};


/*
 * drive partition structure.
 * contains beginning and size
 * each partition on drive.
 * partitions must begin on
 * cylinder boundries.
 */
struct i226part {
	daddr_t p_fsec;			/* first sector of partition */
	daddr_t p_nsec;			/* number of sectors in partition */
	};

#define	MAX_PART	15		/* maximum drive partitions, zero based */

/*
 * driver configuration structure.
 * contains pointers to drive size table
 * and partition table entries for each drive.
 */
struct i226drive {
	unsigned char     d_state;	/* state variable for drive */
	unsigned char	  d_pnum;	/* number of partitions on drive */
	unsigned short    d_part;	/* open partition map */
	unsigned char	  d_intlv;	/* interleave for drive */
	unsigned char	  d_dtype;	/* drive type, lo nibble fhd, hi rhd */
	struct i226size  *d_sp;		/* pointer to drive size table entry */
	struct i226part  *d_pp;		/* pointer to drive partition table */
	};

#define	NDRV		4		/* number of drivers per controller */

#define	DR_ALIVE	0x01		/* 1 - alive state flag for drive */
#define	DR_READY	0x02		/* 2 - drive ready state flag for drive */
#define	DR_OPEN		0x04		/* 4 - drive open state flag for drive */
#define	DR_LOCK		0x08		/* 8 - drive lock, mutual exclusion on open */
#define	DR_PSEC		0x10		/* 16 - physical sectors mode - sector
					   calculations = (s_sec + s_altsec) */
#define	DR_AFMT		0x20		/* 32 - adaptive formatted drive */
#define	DR_RESET	0x40		/* 64 - drive error requires a reset */

#define	GET_FHDTYPE(d)	\
	((d->d_dtype&NIBBLE) - 1)

#define	SET_FHDTYPE(d, t) \
	(d->d_dtype = ((d->d_dtype&(~NIBBLE))|(t + 1)))

#define	GET_RHDTYPE(d) \
	(((d->d_dtype>>4)&NIBBLE) - 1)

#define	SET_RHDTYPE(d, t) \
	(d->d_dtype = ((d->d_dtype&NIBBLE)|((t + 1)<<4)))

#define	HAVE_FHD(d) \
	((d->d_dtype&NIBBLE) > 0)

#define	HAVE_RHD(d) \
	(((d->d_dtype>>4)&NIBBLE) > 0)

/*
 * board structure.
 * one structure per controller.
 */
struct i226board {
	unsigned short b_state;		 /* controller's current state */
	unsigned short b_level;		 /* board's interrupt level */
	unsigned short b_pbase;		 /* base address of board's CSR ports */
	struct i226pb_queue b_runq;	 /* iopb chain run queue */
	struct i226pb_queue b_errq;	 /* iopb chain error queue */
	struct i226drive    b_drv[NDRV]; /* size and partition table pointers */
	};

#define	BD_ALIVE	0x01		 /* alive state flag for board */


/*
 * controller configuration structures.
 * contains interrupt level, base address,
 * and drive configuration structures.
 */
struct i226dr_cfg {
	unsigned char	 dc_pnum;	/* number of partitions on drive */
	struct i226size *dc_sp;		/* pointer to drive size table entry */
	struct i226part *dc_pp;		/* pointer to drive partition table */
	};


struct i226cfg	{
	unsigned short   c_level;	/* board's interrupt level */
	unsigned short   c_pbase;	/* base address of board's I/O ports */
	struct i226dr_cfg c_drv[NDRV];	/* size and partition table pointers */
	};

/***********************************************************************
	I001 -- DEBUGGING: this is the structure which is used
			to fake bad blocks for testing.
 */
struct i226_bb_fake {
	daddr_t		f_block_num;	/* block number for fake */
	dev_t		f_device;	/* device num (maj/min) for fake */
	unsigned char	f_stat1;	/* to be put in IOPB stat1 field */
	unsigned char	f_stat2;	/* to be put in IOPB stat2 field */
};

#define	HARDECCERR	0x06		/* Results in EBBHARD error */
#define	SOFTECCERR	0x1e		/* Results in EBBSOFT error */

/*
	I001 -- ends
************************************************************************/
