
/*
 * "@(#) i534 $State: XENIX_R3-5 $ $Date: 86/05/30 13:10:30 $"
 */

/*
 *	@(#) io/i534.h 1.3 84/05/07 Copyright Microsoft Corporation, 1983
 *
 *	This Module contains Proprietary Information of Microsoft
 *	Corporation and AT&T, and should be treated as Confidential.
 */



#define SPL	spl5		/* keep interrupts away */

#define ISPEED		13	/* initial baud rate of 9600==(13);300==(7)*/
#define MINORMSK 	0x1F	/* reserve bit 7 ; bit 6 for modem */
#define MODEMMSK	0x40	/* bit 6 of the minor number sets modem op */

/*
 * Structures for the iSBC534
 * ____________________________
 *
 * Commands used for operation and initialization of
 * pic's,pit's,usarts and the ppi
 *
 * ____________________________
 *
 * Written by J. Chorn 		December 7th 1981
 * 
 * MODIFICATION HISTORY
 *   I001	llk	1/25/84
 *	Ported to system 3 UNIX.
 *
 * ____________________________
 *
 * Refer to iSBC534 Hardware Reference Manual Chapter Three I/O Address
 * Assignments for further information.
 *
 * Device PHYSICAL port layout
 * Based on Data Block Select
 *
 *
 * Usart I/O functions:
 *
 *	Write: I/O
 *	Read:  I/O
 *
 * PIC status port functions:
 *
 * 	Write:	ICW1,OCW2, and OCW3
 * 	Read:	Status and Poll
 *
 * PIC mask port functions:
 *
 * 	Write:	ICW2 and OCW1(mask)
 *	Read:	OCW1
 *
 *
 * structure db534:
 *
 * 	8251 Usarts [4] 	{2 bytes wide each}
 * 	8259 PIC's  [2] 	{2 bytes wide each}
 *   	Board command port's	{4 bytes}
 *
 */

struct db534{
	struct	{
		char	data;		/* serial I/O data port  */
		char	cntrl;		/* serial control port  */
	} USART[4];
	struct	{
		char	csr;		/* PIC status port  */
		char	msr;		/* PIC mask port    */
	} PIC[2];
	char	selcntr;		/* Select control block */
	char	seldata;		/* Select data block */
	char	stestmd;		/* Select/deselect test mode  */
	char	reset;			/* Board Reset port */
};	
	
/*
 * Refer to iSBC534 Hardware Reference Manual Chapter Three I/O Address
 * Assignments for further information.
 *
 * Device Physical Port Layout
 * Based on Control Block Select
 * 
 *
 * 8253 PIT Functions:
 *
 * 	Load/Read count 
 *
 * 8255 PPI Functions:
 *	
 * Write :
 * 	Port A None
 *	Port B None
 *	Port C Data out
 *	Port (Control) Control commands
 *
 * Read :
 *
 *	Port A Data in
 *	Port B Data in
 *	Port C Data status
 *	Port (Control) None
 *
 *
 * structure cd534:
 *
 * 	8253 PIT's [2]			{4 byte wide each}
 *  	8255 PPI Parallel port		{4 bytes}
 *   	Board command port's		{4 bytes}
 *
 */

struct	cb534{
	struct	{
		char	timer[3];	/* read/load BDG[?] & BDG[?+3] */
		char	pcr;		/* PIT control register */
	} PIT[2];
	char	porta;			/* read port a data in */
	char	portb;			/* read port b data in */
	char	portc;			/* read/write port c */
	char	ppi_pcr;		/* PPI control register */
	char	selcntr;		/* select control block */
	char	seldata;		/* select data block */
	char	stestmd;		/* select/deselect test mode */
	char	reset;			/* board reset */
};

/* 
 * Structure for base assignments of boards in the configuration
 * This is used to associate the board with the base address
 */
struct i534cfg{
	int	c_base;
};


/*
 *
 * 8253 PIT commands
 *
 *	RATEMD0:	read/load timer0(or 4) for mode 3 (baud rate generator)
 *
 *	U534SPEED:	int constant of 1600 pit count in hex.
 *
 *	NOTE:	mode zero can only be used on timers 4 or 5 !! (according to 
 *		the manual ) and this is garf. Program for mode 3 for two second
 *		clock signal.
 *
 */

#define	RATEMD0		0x36
#define U534SPEED	0x0640

/*
 * 8251 USART command instructions
 *	I001  split instructions into separate bits
 */

#define	S_TXEN	0x01		/* transmitter enable */
#define	S_DTR	0x02		/* data terminal ready */
#define	S_RXEN	0x04		/* receiver enable */
#define	S_SBRK	0x08		/* send break char */
#define	S_ER	0x10		/* error reset */
#define	S_RTS	0x20		/* request to send */
#define	S_IR	0x40		/* internal reset */



#define S_TXRDY		0x01		/* transmitter empty */
#define S_RXRDY		0x02 		/* receiver has data */
#define S_PERROR	0x08		/* parity error */
#define S_OVERRUN	0x10		/* overrun */
#define S_FRERROR	0x20		/* framing error */
#define	S_DSRDY		0x80		/* carrier present */

/*
 * 8251 USART mode instructions
 *	I001  split instructions into separate bits
 */

#define S_BAUDF		0x02	/* baud rate factor = 16x */
#define S_5BPC		0x00	/* 5 bits per char */
#define S_6BPC		0x04	/* 6 bits per char */
#define S_7BPC		0x08	/* 7 bits per char */
#define S_8BPC		0x0C	/* 8 bits per char */
#define S_PAREN		0x10	/* parity enable */
#define S_PAREVEN	0x20	/* even parity */
#define S_1STOP		0x40	/* 1 stop bit */
#define S_2STOP		0xC0	/* 2 stop bits */

/*
 * 8259 PIC commands
 *
 * ICW'S 
 *
 * PICICW1:	Format = 4,single pic,edge triggered.
 * PICICW2:	Initialization address.
 * 		Picicw3 and picicw4 not needed.
 * OCW's
 *
 * MASKINT:	Set 8259 to mask all interrupt levels.
 *
 * GETINT :	Set 8259 for polled mode
 * 		read requesting device on next rd pulse.
 * GOODINT:	Mask to check bit 7 ;produce from the 'getint' command.
 * 		Bit 7 set == valid interrupt at this pic.
 *
 * LETRING:	Pic mask which allows ring interrupts to get through
 *
 */

#define PICICW1	0x16
#define PICICW2	0
#define MASKINT	0xFF
#define GETINT	0x0C
#define	GOODINT	0x80
#define LETRING	0xFB

/*
 * the following used to be in a file called usart.h.
 * since the 534 is the only driver that uses this stuff, it's
 * been moved into this include file.
 */

/*
 * Baud rate selection.  These go into the clock counter.
 */

#define	US_B38400	2
#define	US_B19200	4
#define	US_B9600	8
#define	US_B4800	16
#define	US_B2400	32
#define US_B1800	48
#define	US_B1200	64
#define	US_B600		128
#define	US_B300		256
#define US_B200		384
#define	US_B150		512
#define	US_B110		698
#define	US_B75		1024
#define US_B50		1536
#define US_B0		0
