
/*
 * "@(#) i544 $State: XENIX_R3-5 $ $Date: 87/02/13 11:44:12 $"
 */

/*
 * Copyright (C) 1983 Intel Corp.
 */

#ifndef GDT_SEL
#include "mmu.h"
#endif

#define ISPEED          13      /* initial baud rate of 9600 == (13) */
#define MINORMSK        0x1F    /* reserve bit 7; bit 6 for modem */
#define	ECHOMSK		0x20	/* echo bit in minor number */
#define MODEMMSK        0x40    /* bit 6 of the minor number sets modem op */
#define MAXBAUDS	15	/* maximum numbe of baud rates defined */
#define NCL544		5	/* ticks to clear the raw queue */
#define OBLIMIT		0x03ff	/* number of bytes free before interrupt */
#define INBUSY		01	/* input in progress on line */
#define INSTOP		02	/* input interrupts stopped on line */

#define I544LIMIT	0x3FFF	/* size - 1 */
#define I544ACC		DSA_DATA	/* access permit */

/*
 * Structures for the iSBC 544 Board
 *------------------------------------
 *
 * The board structure consists of board existence flag and
 * physical memory address. Within the board structure are
 * structures consisting of a firmware command/status/information structure
 * and 4 line (units) structures.
 *
 *	I001 - 10/17/83
 *		rnb - modified structure for system III
 *			add parameter for input control
 */

struct i544line {
		char  enb;      /* enable bits */
		char  parm;     /* parameter bits */
		char  state;    /* state of line */
		char  error;    /* error bits */
		int   ibaud;    /* input baud rate */
		int   obaud;    /* output baud rate */
		int   iba;      /* input buffer address */
		int   ibs;      /* input buffer size */
		int   ibp;      /* input buffer pointer */
		int   ibc;      /* input buffer count */
		int   ibn;      /* number of bytes input */
		int   oba;      /* output buffer address */
		int   obs;      /* output buffer size */
		int   obp;      /* output buffer pointer */
		int   obc;      /* output buffer count */
		int   obn;      /* number of bytes output */
		int   obl;      /* output byte limit */
		char  scmd;	/* pending command to board */
		char  wtime;	/* output delay timeout value */
		int   leng;	/* pending command data length */
		int   instate;	/* input state of line */
		};

struct i544firm {
	char  cmd;              /* command from host */
	char  status;           /* status to host */
	char  cmdsem;           /* command semaphore */
	char  stsem;            /* status semaphore */
	char  cunit;            /* unit number associate with command */
	char  sunit;            /* unit number associate with status */
	char  oper;             /* operational status */
	char  deverr;           /* error reporting */
	int   firstl;           /* addr of first line structure */
	int   lsize;            /* size of each line structure */
	char  nlines;           /* number of lines on board */
	char  intenb;           /* interrupt enable */
	int   vers;             /* firmware version number */
	char  diagn;            /* firmware diagnostic number */
	char  diage;            /* failed diagnostic information */
	int   joaddr;           /* jump out command address */
	char  portno;           /* Port I/O command port number */
	char  portv;            /* Port I/O command data */
	};

/*
 * Structure of all the information to know the of
 * an iSBC 544 board's firmware
 */
struct	i544board {
	int alive;		   /* set at init if board is there */
	long physaddr;		   /* physical address of board in 16 Meg */
	short selector;		   /* segment selector of board */
	struct i544firm firm; 	   /* copy of current firmware values */
	struct i544line line[4];   /* per line state data */
	};


/*
 * Command for the command/status/information structure
 * ----------------------------------------------------
 * 
 */

/*
 * Command from the host
 *
 *   RESET   resets the entire board and runs micro diagnostics
 *
 *   INPUT   signals that the host has transferred ibn bytes from the
 *           input buffer
 *   
 *   OUTPUT  signals that the host has placed obn bytes in the output
 *           buffer for output
 *
 *   PARAM   signals that the host has changed parameters parm or ibaud
 *
 *   JPOUT   causes the 544 to call the subroutine indicated by
 *           'joaddr'
 *
 *   CONTI   causes the reset to continue, ignoring the error
 *
 *   PRTOUT  for diagnostic use only
 *
 *   PRTIN   for diagnostic use only
 */

#define   RESET    0x01
#define   INPUT    0x02
#define   OUTPUT   0x03
#define   PARAM    0x04
#define   JPOUT    0x05            /* for diagnostic only */
#define   CONTI    0x06
#define   PRTOUT   0x07            /* for diagnostic only */
#define   PRTIN    0x08            /* for diagnostic only */
#define   OFLUSH   0x09		   /* flush output buffer */

/*
 * Status to the host from the firmware
 *
 *   CMDACP  acknowledges the execution of the last command
 *
 *   INVCMD  indicates that the last command was in error
 *
 *   INRDY   indicates that the input is ready (ibc bytes)
 *
 *   OUTRDY  indicates that the output is ready (obc bytes)
 *
 *   RING    indicates that the line sunit is ringing
 *
 *   CARIER  indicates that there has been a carrier loss on line
 *
 *   ABAUDR  indicates that the baud rate has been recognized on line
 */

#define   CMDACP   0x01
#define   INVCMD   0x02
#define   INRDY    0x03
#define   OUTRDY   0x04
#define   RING     0x05
#define   CARIER   0x06
#define   ABAUDR   0x07
 
/*
 * Diagnostic information
 *
 *   ROMERR  ROM checksum error
 *
 *   RAMFIL  RAM failure
 *   
 *   PARINT  parallel port initialization failure
 *
 *   PAROPR  parallel port operating state failure
 * 
 *   ROMSUM  sum of ROM bytes
 *   
 *   RAMVAL  value that should have been in RAM location
 *
 *   PARLV1, PARLV2  value from port A of parallel interface
 *
 *   DIAGRUN running diagnostic
 *
 *   DIAGSUC diagnostics succeeded
 *
 *   DIAGERR diagnostic error
 */

#define   ROMERR   0
#define   RAMFIL   0x01
#define   PARINT   0x02
#define   PAROPR   0x03
#define   ROMSUM   0
#define   RAMVAL   0x01
#define   PARLV1   0x02
#define   PARLV2   0x03
#define   DIAGRUN  0x01
#define   DIAGSUC  0x02
#define   DIAGERR  0x03
 
/* Mis. information for the command/status/information structure */

#define   CLEAR    0
#define   ENBINTR  0x01

/* 
 * Line (Unit) structure information
 * ---------------------------------
 *
 *   ENBOUT  enable output
 *
 *   XOFFON  enable Control-S/Control-Q
 *
 *   DTRDY   Data Terminal Ready
 *   
 *   PNO     no parity
 *
 *   PODD    odd parity
 *
 *   PEVEN   even parity
 *
 *   CARRIER Carrier state set
 *
 *   DSRDY   data set ready
 *   
 *   IBUFOV  input buffer overflowed
 *
 *   RECINT  turn off receive input interrupts
 *
 *   NOMODEM turn off ring detect and carrier detect interrupts
 *
 */

#define   ENBOUT  0x01
#define   XOFFON  0x02
#define   DTRDY   0x01
#define   PNO     0x00
#define   PODD    0x02
#define   PEVEN   0x04
#define   CARRIER 0x01
#define   DSRDY   0x02
#define   IBUFOV  0x01
#define	  RECINT  0x10
#define   NOMODEM 0x08

/*
 * Supported baud rate of the 544 device driver
 *
 *   US_B110     110 BAUD
 *   US_B150     150 BAUD
 *   US_B300     300 BAUD
 *   US_B600     600 BAUD
 *   US_B1200   1200 BAUD
 *   US_B2400   2400 BAUD
 *   US_B4800   4800 BAUD
 *   US_B9600   9600 BAUD
 *   US_B19200 19200 BAUD
 *   
 */
#define US_B110		110
#define US_B150		150
#define US_B300		300
#define US_B600		600
#define US_B1200	1200
#define US_B2400	2400
#define US_B4800	4800
#define US_B9600	9600
#define US_B19200	19200

/*
 * Offsets into the firmware's control structure
 * command area
 */
#define i544CMND	0
#define i544STAT	1
#define CUNIT		4
#define SUNIT		5


/*
 * Macros that return the offset to the desired
 * line parameter in the memory mapped firmware's
 * line control structure.
 */
#define o_enb(x)	((x+1) << 5)
#define o_parm(x)	(((x+1) << 5) + 1)
#define o_state(x)	(((x+1) << 5) + 2)
#define o_ibaud(x)	(((x+1) << 5) + 4)
#define o_ibp(x)	(((x+1) << 5) + 12)
#define o_ibc(x)	(((x+1) << 5) + 14)
#define o_ibn(x)	(((x+1) << 5) + 16)
#define o_obp(x)	(((x+1) << 5) + 22)
#define o_obc(x)	(((x+1) << 5) + 24)
#define o_obn(x)	(((x+1) << 5) + 26)
#define o_obl(x)	(((x+1) << 5) + 28)

