
/*
 * "@(#) mmu $State: XENIX_R3-5 $ $Date: 86/05/30 13:04:24 $"
 */

/*
 *	@(#) h/mmu.h 1.5 84/05/07 Copyright Microsoft Corporation, 1983
 *
 *	This Module contains Proprietary Information of Microsoft
 *	Corporation and AT&T, and should be treated as Confidential.
 */

/*
 * THIS FILE CONTAINS CODE WHICH IS SPECIFIC TO THE
 * INTEL 286/310 COMPUTER AND MAY REQUIRE MODIFICATION
 * WHEN ADAPTING XENIX TO NEW HARDWARE.
 */


/*
 * mmu.h
 *	Memory Management Definitions
 *
 * used by mmu.c
 * param.h has basic page-size/etc definitions.
 *
 * definitions for 286 memory managment model
 *
 */

/*
 * Modification History
 *
 *	I000	jdm	10/14/85
 *		Moved histogram selectors out of device driver
 *		selector area and removed "LAST_SEL"
 */

/*
 * descriptor table defines
 * NOTE: the RPL in USEG_SEL and ULDT_SEL is 0. This
 *	 does not ENFORCE protection
 *
 * NOTE: DEV_DSEL must (repeat must) be aligned so that the
 *       last byte of it (in the GDT) hits a paragraph boundry.
 *       An easier way of saying this is to say it must have an 8
 *       for the last digit.
 */
#define	NULL_SEL	0	/* 'invalid' selector */
/* gdt defines
 * WARNING - these must agree with the layout of the gdt in tabs.s
 */
#define	GDT_SEL		0x140	/* selector for GDT alias */
#define	IDT_SEL		0x148	/* selector for IDT alias */
#define KDS_SEL         0x150   /* Kernel data seg selector */
#define KCS_SEL         0x158   /* Kernel code seg selector for SEG 1 */
				/* Kernel code seg 2-5 are here */
#define KSS_SEL         0x180   /* Kernel stack seg selector */
#define KTSS            0x188   /* Kernel TSS selector */
#define KTSS_SEL        0x190   /* Kernel TSS alias selector */
#define KWORK0          0x198   /* work segment selector */
#define KWORK1          0x1a0   /* work segment selector */
#define KWORK2          0x1a8   /* work segment selector */
#define SWAP1_SEL       0x1b0   /* work selector for the swapper */
#define SWAP2_SEL       0x1b8   /* work selector for the swapper */
#define CLDT_SEL        0x1c0   /* current process' ldt selector */
#define KENTRY          0x1c8   /* kernel entry selector (call gate) */
#define MAPWKSEL        0x1d0   /* selector for map work */
#define MSS_GATE        0x1d8   /* monitor entry for ss interrupt */
#define MBRK_GATE       0x1e0   /* monitor entry for brkpt intr */
#define COPY_SEL        0x1e8   /* used by copyio */
#define BUF_SEL		0x1f0	/* selector for mapped out bufers */
#define BUF2_SEL	0x1f8	/* selector for mapped out bufers */
#define BUF3_SEL	0x200	/* selector for mapped out bufers */
#define BUF4_SEL	0x208	/* selector for mapped out bufers */
#define BUF5_SEL	0x210	/* selector for mapped out bufers */
#define BUF6_SEL	0x218	/* selector for mapped out bufers */
#define BUF7_SEL	0x220	/* selector for mapped out bufers */
#define I215WORK        0x228   /* Temporary descr for 215 */
#define TTY0_SEL        0x230   /* marks first tty slot in the gdt */
#define KHISTO_SEL	0x238	/* Kernel histogram selector */	/* I000 */
#define DHISTO_SEL	0x240	/* Disk histogram selector */	/* I000 */
		/* The rest of the selectors are RESERVED for driver use. */
#define DEV_DSEL        0x248   /* first device driver selector  */
/*	I000 - LAST_SEL has gone away.	*/
/* 
 * ldt defines 
 * These selector definitions should not be touched - they are chip
 * and linker defined
 */
#define	ULDT_SEL	0x0C	/* ldt based, ldt alias */
#define	USEG_SEL	0x14	/* LDT based U seg selector */
#define	UEXEC_SEL	0x1C	/* used for execing */
#define	UTEMP_SEL	0x2C	/* used for swapping, etc */
#define	UOVL_SEL	0x34	/* used with overlay files */
#define	FIRSTU_SEL	0x3F	/* user's initial selector */

#define NCSEGS		5	/* number of text segments the kernel may have*/

/*
 * Descriptor Types
 */
#define	DST_ATASK	0x1	/* avail task */
#define	DST_TABLE	0x2	/* descriptor table */
#define	DST_BTASK	0x3	/* busy task */
#define	DST_CGATE	0x4	/* call gate */
#define	DST_TAGATE	0x5	/* task context gate */
#define	DST_IGATE	0x6	/* interrupt gate */
#define	DST_TRGATE	0x7	/* trap gate */


#include "relsym86.h"

/*
 * access byte fields (DSA_*) are defined in relsym86.h
 */

/*
 * flag register bits
 */
#define	FL_NCX		0x4000	/* nested context */
#define	FL_IOPL		0x3000	/* I/O priv level */
#define	FL_OFLOW	0x800	/* overflow bit */
#define	FL_DIR		0x400	/* direction of string ops */
#define	FL_INTEN	0x200	/* interrupt enable */
#define	FL_TRAP		0x100	/* single step flag */
#define	FL_SIGN		0x80	/* sign bit */
#define	FL_ZERO		0x40	/* zero bit */
#define	FL_AUXCARRY	0x10	/* auxillary carry */
#define	FL_PARITY	0x4	/* parity bit */
#define	FL_CARRY	0x1	/* carry bit */
#define IOPL		FL_IOPL	/* set current IOPL to 3 (DEBUG ECC) */

/*
 * parts of a selector see relsym86.h
 */
#define	SEL_TI		SEL_LDT	/* table indicator (0-gdt,1-ldt) */

/*
 * descriptor table structure: see relsym86.h
 */
#define NPSEGM	0		/* no segments */

/*
 * Misc constants.
 */
#define OFFMASK	(MMPGSZ-1)	/* develop offset within page from addr */

#define RO (DSA_CODE|DSA_RING3) /* tells mmuset to setup code to make */
#define RW (DSA_DATA|DSA_RING3) /* kernel access read-only, or rd/wr */

#define	MMUOK	((mloc_t) 0)
#define	MMUERR	((mloc_t) -1)
extern mloc_t mmupget();
extern mloc_t mmuget();

/*
 * macros to determine the size and location of the U seg
 */

extern long dscraddr();
extern unsigned short dscrlimit();
